/**

  To run program:
    a) Simple Input
      genphot | objs/sun4u-SunOS-5/testBinaryPipe -0p
    b) Simple Input/Output
      genphot | objs/sun4u-SunOS-5/testBinaryPipe -1
    c) Simple add header field
      not yet tested, blah blah
    d) Simple add Packet to bpipe
      genphot | objs/sun4u-SunOS-5/testBinaryPipe -3

*/

#include <cstdlib>
#include <iostream>

#include <BinaryPipe/BinaryPipe.h>
#include <BinaryPipe/MyPhoton.h>

using namespace std;

/**
 * Open a bpipe to read the input only
 */
void simpleInput( const char* input, bool printit )
  throw ( BinaryPipeException ) {

    try {

      BinaryPipe< MyPhoton > bpipe( input );
      MyPhoton* photon = (MyPhoton*) bpipe;
      DVector3*  position = photon->get_position( );
      while( bpipe.read_dpkts( ) ) {
	if ( printit )
	  cout << photon << endl;
      }

    } catch( BinaryPipeException& bpe ) {

      throw;

    }

}

/**
 * Open a bpipe and modify some existing content then output the modify data
 */
void simpleInputOutput( const char* input, const char* output, bool printit )
  throw ( BinaryPipeException ) {

    try {

      BinaryPipe< MyPhoton > bpipe( input, output );
      MyPhoton* photon = (MyPhoton*) bpipe;
      DVector3*  position = photon->get_position( );
      while( bpipe.read_dpkts( ) ) {
	position->x = 1.0;
	position->y = 2.0;
	position->z = 3.0;
	bpipe.write_dpkt( );
	if ( printit )
	  cout << bpipe << endl;
      }

    } catch( BinaryPipeException& bpe ) {

      throw;

    }

}

/**
 * The data packet to add
 */
DataPacket datapacket[] = {
  "foo", BPDType_double, NULL,
  "bar", BPDType_int,    NULL,
  NULL,  BPDType_char,   NULL
};

/**
 *
 */
class TestPhoton : public MyPhoton {

public:

  double* get_foo( ) { return foo; }
  int*    get_bar( ) { return bar; }

  TestPhoton( BPipe* bpipe, void* core ) throw ( BinaryPipeException ) :
    MyPhoton( bpipe, core ) {

      foo = (double*) dpktf_data( bpipe, core, (char*) datapacket[0].name );
      
      bar = (int*) dpktf_data( bpipe, core, (char*) datapacket[1].name );
  }

protected:

   double *foo;
   int *bar;

};

void addDataPacket( const char* input, const char* output, bool printit )
  throw ( BinaryPipeException ) {

  try {    

    BinaryPipe< TestPhoton > bpipe( input, output, NULL, datapacket );
    TestPhoton* photon = (TestPhoton*) bpipe;
    double* foo = photon->get_foo( );
    int*    bar = photon->get_bar( );
    *foo = 1.3;
    *bar = 4;
    while( bpipe.read_dpkts( ) ) {
      bpipe.write_dpkt( );
	if ( printit )
	  cout << bpipe << endl;      
    }

  } catch( BinaryPipeException& bpe ) {

    throw;

  }

}

int main( int argc, char** argv ) {

  bool printit = false;
  bool test0 = false;
  bool test1 = false;
  bool test2 = false;
  bool test3 = false;
  char c;

  while( --argc > 0 && (*++argv)[0] == '-' )
    while( (c = *++argv[0]) )
      switch (c) {
      case '0':
	test0 = true;
	break;
      case '1':
	test1 = true;
	break;
      case '2':
	test2 = true;
	break;
      case '3':
	test3 = true;
	break;
      case 'p':
        printit = true;
        break;
      default:
	cerr << "Usage : " << argv[0] << " [-12p]\n";
	return EXIT_FAILURE;
      }

  try {

    if ( test0 )
      simpleInput( "stdin", printit );

    if ( test1 )
      simpleInputOutput( "stdin", "foo", printit );

    if ( test3 )
      addDataPacket( "stdin", "foo", printit );

  } catch( BinaryPipeException& bpe ) {

    cerr << bpe << endl;
    return EXIT_FAILURE;
    
  }

  return EXIT_SUCCESS;

}
