// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of Exception
//
// Exception is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// Exception is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <sstream>

#include <errno.h>

#include "Exception.h"

Exception& Exception::operator= ( const Exception& rhs ) {

  if ( this == &rhs )
    return *this;

  this->runtime_error::operator=( rhs );
  exception_queue = rhs.exception_queue;

  return *this;

}

void Exception::print( ostream& os ) const {
  for ( size_t i = 0; i < exception_queue.size( ); i++ )
    os << exception_queue[ i ];
}

void Exception::set_message( const string& msg ) {
  exception_queue.push_front( msg );
}

void Exception::set_rethrow_message( const string& file, const int n ) {
  ostringstream ost;
  ost << "Re-throw exception from file '" << file << "' at line " << n << '\n';
  exception_queue.push_front( ost.str( ) );
}

const char* Exception::what() const throw() {

  if ( exception_queue.size( ) )
    return exception_queue[ 0 ].c_str( );
  else
    return (const char*) NULL;

}
