use warnings;
use strict;

use Test::More tests => 8;
BEGIN { use_ok('MST::Raytrace', qw( get_enen ENEN_R ENEN_F ) ) };

my ( $results, %expected );

# bad file
eval {
      get_enen( 'non_existent_file', ENEN_R );
     
};
ok( $@ && $@ =~ /error open/, "non existent file" );

# bad file type
eval {
      get_enen( 'data/test.enen_r', 33 );
     
};
ok( $@ && $@ =~ /unknown enen type/, "bad type" );


#########################################################


%expected = (
          '25' => {
                    'n' => '1990',
                    'weight' => '1989.59581',
                    'fraction' => '0.70378'
                  },
          '1' => {
                   'n' => '4',
                   'weight' => '3.01347',
                   'fraction' => '0.00107'
                 },
          '30' => {
                    'n' => '2827',
                    'weight' => '2827.00000',
                    'fraction' => '1.00000'
                  },
          '10' => {
                    'n' => '336',
                    'weight' => '335.63633',
                    'fraction' => '0.11873'
                  },
          '2' => {
                   'n' => '14',
                   'weight' => '13.40270',
                   'fraction' => '0.00474'
                 },
          '20' => {
                    'n' => '1286',
                    'weight' => '1285.30319',
                    'fraction' => '0.45465'
                  },
          '15' => {
                    'n' => '706',
                    'weight' => '705.92604',
                    'fraction' => '0.24971'
                  },
          '5' => {
                   'n' => '81',
                   'weight' => '80.77279',
                   'fraction' => '0.02857'
                 }
        );

$results = get_enen( 'data/test.enen_r', ENEN_R );
ok( eq_hash($results, \%expected), "enen_r: string keys");

$expected{$_+0} = delete $expected{$_} foreach keys %expected;


$results = get_enen( 'data/test.enen_r', ENEN_R, { FloatKeys => 1 } );
ok( eq_hash($results, \%expected), "enen_r: float keys");
ok( exists $results->{30.00}, "enen_r: float keys: exists" );

#########################################################


%expected = (
          '0.60000' => {
                         'n' => '1697',
                         'radius' => '23.0462',
                         'weight' => '1696.20000'
                       },
          '0.70000' => {
                         'n' => '1979',
                         'radius' => '24.9278',
                         'weight' => '1978.90000'
                       },
          '0.50000' => {
                         'n' => '1414',
                         'radius' => '21.1696',
                         'weight' => '1413.50000'
                       },
          '0.20000' => {
                         'n' => '566',
                         'radius' => '13.2366',
                         'weight' => '565.40000'
                       },
          '0.90000' => {
                         'n' => '2545',
                         'radius' => '28.4341',
                         'weight' => '2544.30000'
                       },
          '0.30000' => {
                         'n' => '849',
                         'radius' => '16.4175',
                         'weight' => '848.10000'
                       },
          '0.40000' => {
                         'n' => '1131',
                         'radius' => '18.7688',
                         'weight' => '1130.80000'
                       },
          '1.00000' => {
                         'n' => '2827',
                         'radius' => '29.9931',
                         'weight' => '2827.00000'
                       },
          '0.10000' => {
                         'n' => '283',
                         'radius' => '9.24942',
                         'weight' => '282.70000'
                       },
          '0.80000' => {
                         'n' => '2262',
                         'radius' => '26.7187',
                         'weight' => '2261.60000'
                       }
        );

$results = get_enen( 'data/test.enen_f', ENEN_F );
ok( eq_hash($results, \%expected), "enen_f: string keys");

$expected{$_+0} = delete $expected{$_} foreach keys %expected;
$results = get_enen( 'data/test.enen_f', ENEN_F, { FloatKeys => 1 } );
ok( exists $results->{0.60}, "enen_f: float keys: exists" );
