use Test::More tests => 8;
use Test::Exception;

use strict;
use warnings;

my @energies;

BEGIN { use_ok('MST::Raytrace', 'read_energy_rdb') };

# bad file
throws_ok { read_energy_rdb( file => 'does not exist' ); } 
	    qr/error opening/, 'bad file';

# bad column
throws_ok { read_energy_rdb( file => 'data/energy.rdb',
			     ecol => 'foo'
			 ); } qr/nonexistent column/, 'bad column';

# alternative energy column
@energies = read_energy_rdb( file => 'data/energy.rdb', ecol => 'energy2' );
is_deeply( \@energies, [2,4,6,8,10], 'alternative column name' );


# all energies
@energies = read_energy_rdb( file => 'data/energy.rdb' );
is_deeply( \@energies, [1,2,3,4,5], 'all energies' );

# min cut off
@energies = read_energy_rdb( file => 'data/energy.rdb', emin => 3 );
is_deeply( \@energies, [3,4,5], 'min cutoff' );

# max cut off
@energies = read_energy_rdb( file => 'data/energy.rdb', emax => 3 );
is_deeply( \@energies, [1,2], 'max cutoff' );

# min and max cut off
@energies = read_energy_rdb( file => 'data/energy.rdb', emin => 2, emax => 5 );
is_deeply( \@energies, [2,3,4], 'min & max cutoff' );
