use strict;
use File::Compare;

use constant TEST_RDB => 't/test1.rdb';

use vars qw( $loaded $tests $test);

BEGIN { 
  require 't/common.pl';
  $tests = 4; 
  $| = 1; 
  print "1..$tests\n"; 
}
END {print "not ok 1\n" unless $loaded; unlink glob 'tmp-*'; }

use vars qw( @COMMENTS );

@COMMENTS = ( 'a', 'b', 'c' );

use RDB;

$verbose = 1;
$test = 1;
$loaded = 1;
print "ok 1\n";


sub setup
{
  $test++;
  my $rdb = new RDB ( @_ ) or return undef;
}


# if any of the subs croak or die, there will be no return,
# which will cause the eval to return undef

# add some comments
ok eval { 

  my $rdb = setup() or return undef;

  # add comments
  $rdb->add_comments( @COMMENTS );

  cmp_array( [ $rdb->comments ], \@COMMENTS );
};

# add some comments, then delete them
ok eval { 

  my $rdb = setup() or return undef;

  # add comments
  $rdb->add_comments( @COMMENTS );

  # delete them
  $rdb->comments( undef );

  # better not be any there!
  0 == $rdb->comments;
};


# try adding comments with comments()
# see what happens!
ok eval { 

  my $tfile = "tmp-$test";
  my $rdb = setup( TEST3_RDB ) or return undef;

  my $out = new RDB $tfile, '>' or return undef;

  # erase old comments
  my @comments = $rdb->comments;
  $rdb->comments( undef );

  $out->init( $rdb );
  $out->comments( @comments );

  $out->write_hdr;
  my %data;
  $out->write(\%data)
    while( $rdb->read( \%data ) );
  $out->close;

  ! compare( $tfile, TEST3_RDB );
};
