use strict;
use File::Compare;
use Data::Dumper;

use vars qw( $loaded $tests $test);

BEGIN { 
  require 't/common.pl';
  $tests = 8;
  $| = 1;
  print "1..$tests\n"; 
}

END {print "not ok 1\n" unless $loaded; unlink glob 'tmp-*'; }

use RDB;

$verbose = 1;
$test = 1;
$loaded = 1;
print "ok 1\n";

sub setup
{
  $test++;
  my $rdb = new RDB TEST3_RDB or return undef;
}

# if any of the subs croak or die, there will be no return,
# which will cause the eval to return undef

ok eval { 

  my $rdb = setup(@_) or return undef;

  # should have a couple of header variables
  $rdb->getvar( 'var1' ) eq 'val1' &&
  $rdb->getvar( 'var2' ) eq 'val2';
};


# test getvars, vars, getvar
ok eval {
  my $rdb = setup(@_) or return undef;

  my @vars = $rdb->vars;
  my $vars = $rdb->getvars;

  while( @vars )
  {
    my $var = shift @vars;
    die( "$var isn't in getvars" ) unless exists $vars->{$var};
    die( "$var doesn't have same value" ) 
      if $vars->{$var} ne $rdb->getvar( $var );

    delete $vars->{$var};
  }

  die ("more vars in getvars than vars" )
    if keys %$vars;

  1;
};

# test delvar

ok eval { 

  my $rdb = setup() or return undef;

  # var1 had better exist.
  defined( $rdb->getvar( 'var1' ) ) or return undef;

  $rdb->delvar( 'var1' );
  
  !defined $rdb->getvar( 'var1' );
};


# 1. read in file
# 2. change header variables
# 3. write out new file
# 4. read in new file and make sure the variables are changed

ok eval { 

  my $rdb = setup(@_) or return undef;
  my $out = new RDB "tmp-$test", '>' or return undef;

  $out->init( $rdb );
  $out->setvar( var1 => 'val01' );
  $out->setvar( var2 => 'val02' );

  $out->write_hdr;
  my %data;
  $out->write(\%data)
    while( $rdb->read( \%data ) );
  $out->close;

  $rdb = new RDB "tmp-$test" or return undef;
  $rdb->getvar( 'var1' ) eq 'val01' &&
  $rdb->getvar( 'var2' ) eq 'val02';
};


# 1. read in file
# 2. add header variables
# 3. write out new file
# 4. read in new file and make sure the new variables are there

ok eval { 

  my $rdb = setup(@_) or return undef;
  my $out = new RDB "tmp-$test", '>' or return undef;

  $out->init( $rdb );
  $out->setvar( var3 => 'val03' );
  $out->setvar( var4 => 'val04' );

  $out->write_hdr;
  my %data;
  $out->write(\%data)
    while( $rdb->read( \%data ) );
  $out->close;

  $rdb = new RDB "tmp-$test" or return undef;
  $rdb->getvar( 'var3' ) eq 'val03' &&
  $rdb->getvar( 'var4' ) eq 'val04';
};


# test variable loading when creating new comments
# 1. read variables
# 2. zap comments
# 3. reload comments using comments();
# 4. check out 
ok eval { 

  my $tfile = "tmp-$test";
  my $rdb = setup(@_) or return undef;

  # save values of all of the variables
  my @vars = sort $rdb->vars;
  my %var;
  @var{ @vars } = map { $rdb->getvar( $_ ) } @vars;

  # erase old comments
  my @comments = $rdb->comments;
  $rdb->comments( undef );

  # delete old variables
  $rdb->delvar( $_ ) foreach @vars;

  # reload comments
  $rdb->comments( @comments );

  # get new variable names
  my @newvars = sort $rdb->vars;

  # make sure we've got the same variables
  return undef unless cmp_array( \@newvars, \@vars );

  # now check values
  0 == grep { $rdb->getvar( $_ ) ne $var{ $_ } } @vars;
};

# test variable loading when creating new comments
# 1. read variables
# 2. zap comments
# 3. reload comments using add_comments();
# 4. check out 
ok eval { 

  my $tfile = "tmp-$test";
  my $rdb = setup(@_) or return undef;

  # save values of all of the variables
  my @vars = sort $rdb->vars;
  my %var;
  @var{ @vars } = map { $rdb->getvar( $_ ) } @vars;

  # erase old comments
  my @comments = $rdb->comments;
  $rdb->comments( undef );

  # delete old variables
  $rdb->delvar( $_ ) foreach @vars;

  # reload comments
  $rdb->add_comments( @comments );

  # get new variable names
  my @newvars = sort $rdb->vars;

  # make sure we've got the same variables
  return undef unless cmp_array( \@newvars, \@vars );

  # now check values
  0 == grep { $rdb->getvar( $_ ) ne $var{ $_ } } @vars;
};

