use strict;
use File::Compare;

use vars qw( $loaded $tests $test $verbose );

BEGIN { 
  require "t/common.pl";
  $tests = 16; 
  $| = 1; print "1..$tests\n"; }
END {print "not ok 1\n" unless $loaded; unlink glob 'tmp-*'; }

use RDB;


$verbose = 1;
$test = 1;
$loaded = 1;
print "ok 1\n";


sub setup
{
  ++$test;
  my $file = "tmp-$test";

  my $rdb = new RDB $file, '>' or return ();
  ( $rdb, $file );
}

# if any of the subs croak or die, there will be no return,
# which will cause the eval to return undef

# initialization from an array, using various means of
# per-column initialization
ok eval { 

  my ( $rdb, $file ) = setup(@_) or return undef;

  $rdb->init( c1 => 'N',
	      c2 => { type => 'N', width => 3, desc => 'Go SkyHooks!' },
	      c33 => [ 'S', 3, '>', "A description" ]
	    );
  $rdb->write_hdr;
  $rdb->close;

  ! compare( $file , TEST1_RDB );
};

# initialization from another RDB object
ok eval {

  my $rdb1 = new RDB TEST1_RDB;
  my ( $rdb, $file ) = setup(@_) or return undef;
  $rdb->init( $rdb1 );
  $rdb->write_hdr;
  $rdb->close;

  ! compare( $file , TEST1_RDB );
};

# initialization from a template
ok eval {
  my ( $rdb, $file ) = setup(@_) or return undef;
  test( "template init 1" );
  $rdb->init_tpl( TEST1_TPL );
  $rdb->write_hdr;
  $rdb->close;

  ! compare( $file , TEST1_RDB );
};

# initialization from a template
ok eval {
  my ( $rdb, $file ) = setup(@_) or return undef;
  test( "template init 2" );
  $rdb->init_tpl( TEST2_TPL );
  $rdb->write_hdr;
  $rdb->close;

  ! compare( $file , TEST2_RDB );
};

# initialization from a template
ok eval {

  my ( $rdb, $file ) = setup(@_) or return undef;
  test( "template init 3" );
  $rdb->init_tpl( TEST3_TPL );
  $rdb->write_hdr;
  $rdb->close;

  ! compare( $file , TEST3_RDB );
};

# initialization from a template
ok eval {

  my ( $rdb, $file ) = setup(@_) or return undef;
  test( "template init 4" );
  $rdb->init_tpl( TEST4_1_TPL );
  $rdb->write_hdr;
  $rdb->close;

  ! compare( $file , TEST4_RDB );
};

# initialization from a template scalar
ok eval {

  my ( $rdb, $file ) = setup(@_) or return undef;
  test( "template init 5" );

  local $/ = undef;
  open( TPL, TEST4_1_TPL ) or return undef;
  my $tpl = <TPL>;
  close TPL;

  $rdb->init_tpl( \$tpl );
  $rdb->write_hdr;
  $rdb->close;

  ! compare( $file , TEST4_RDB );
};


# writing of template
ok eval {

  $test++;
  test( "template write 1" );
  my $rdb = new RDB;
  my $file = "tmp-$test.tpl";
  $rdb->init_tpl( TEST1_TPL );
  $rdb->write_tpl( $file );
  $rdb->close;

  ! compare( $file , TEST1_TPL );
};

# writing of template
ok eval {

  $test++;
  test( "template write 2" );
  my $rdb = new RDB;
  my $file = "tmp-$test.tpl";
  $rdb->init_tpl( TEST2_TPL );
  $rdb->write_tpl( $file );
  $rdb->close;

  ! compare( $file , TEST2_TPL );
};

# writing of template
ok eval {

  $test++;
  test( "template write 3" );
  my $rdb = new RDB;
  my $file = "tmp-$test.tpl";
  $rdb->init_tpl( TEST3_TPL );
  $rdb->write_tpl( $file );
  $rdb->close;

  ! compare( $file , TEST3_TPL );
};

# writing of template
ok eval {

  $test++;
  test( "template write 4" );
  my $rdb = new RDB;
  my $file = "tmp-$test.tpl";
  $rdb->init_tpl( TEST4_TPL );
  $rdb->write_tpl( $file );
  $rdb->close;

  ! compare( $file , TEST4_TPL );
};

# test various failures
ok eval {

  $test++;
  my $rdb = new RDB or return undef;

  eval { $rdb->init( 'c1' ); };
  return $@;
};

# test various failures
ok eval {

  $test++;
  my $rdb = new RDB or return undef;

  eval { $rdb->init( 'c1', undef ); };
  return $@;
};

ok eval {

  $test++;
  my $rdb = new RDB or return undef;

  eval { $rdb->init( 'c1' => \&sub3 ); };
  return $@;
};

ok eval {

  $test++;
  my $rdb = new RDB or return undef;

  eval { $rdb->init( 'c1' => \*STDOUT ); };
  return $@;
};
