/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of aperture
 *
 * aperture is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * aperture is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>

#include <lua.h>
#include <luasup/luasup.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <bpipe/bpipe.h>

#include "module.h"
#include "ray_transforms.h"
#include "aperture.h"
#include "common.h"


/* forward declaration of function local to this file */
static void pass(void);
static void init(Aperture *aperture, void* amip);
static ProcessResult process(Aperture *aperture);
static void cleanup(Aperture *aperture);

/*
   Define the ApertureModule structure for this module.
*/
ApertureModule pass_module =
{
  "pass",		/* the lua name of the instantiation function */
  pass,		/* the C lua-callable instantiation function */
  init,			/* the aperture initialization function */
  process,		/* the aperture photon processing function */
  cleanup		/* the aperture cleanup function */
};


/*
   this function is called from lua to create an instantiation of
   this aperture type.
*/
static void
pass( void ) {

  tf_enter("pass");

  slua_args( errfunc, "pass", NULL );

  aperture_new( &pass_module, NULL, NULL );

  tf_leave();

}

/*

   This routine is called once, before any photons arrive.  It should
   be used to determine offsets to whatever data this aperture needs
   from the photon data packet.  It should also initialize anything
   that needs initialization.
   
*/

static void 
init(Aperture *aperture, void* ami)
{
}

/*

   This routine is called for every photon.  It should be as efficient
   as possible.
   
*/

static ProcessResult
process( Aperture *aperture )
{
  return PASSED;
}

/*
   This routine is called once for every instantiatian of this aperture.
   It should free instantiation specific resources (like those allocated
   in the lua-called instantiation creation routine).
*/

static void
cleanup (Aperture *aperture)
{
}
