#ifndef MATRIX_H
#define MATRIX_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of aperture
 *
 * aperture is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * aperture is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

typedef enum { DIM0, DIM1, DIM2, DIM3, DIM4 } Dim;
typedef enum { XAXIS, YAXIS, ZAXIS } XYZaxis;

typedef double** XMatrix;

XMatrix XMatrix_new( void );
void	XMatrix_free( XMatrix matrix );
XMatrix XMatrix_dup( XMatrix from );
void    XMatrix_print( XMatrix a );
XMatrix XMatrix_mult( XMatrix a, XMatrix b );
XMatrix Xmatrix_identity( void );
XMatrix Xmatrix_inverse( XMatrix a );
XMatrix Xmatrix_rotation( XYZaxis axis, double theta );
XMatrix Xmatrix_translation( double *tran );

#define XMatrix_copy(from,to)	       		\
  do {						\
    int _xmci, _xmcj;				\
    for ( _xmci = 0; _xmci < DIM4; _xmci++ )	\
      for( _xmcj = 0; _xmcj < DIM4; _xmcj++ )	\
	to[_xmci][_xmcj] = from[_xmci][_xmcj];	\
  } while(0)

#define XMatrix_set(matrix,value)		\
  do {						\
    int _xmci, _xmcj;				\
    for ( _xmci = 0; _xmci < DIM4; _xmci++ )	\
      for( _xmcj = 0; _xmcj < DIM4; _xmcj++ )	\
	matrix[_xmci][_xmcj] = value;	\
  } while(0)

#endif /* ! MATRIX_H */
