// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of src
//
// src is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// src is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <funtools.h>
#include <float.h>

#include <Exception/Exception.h>
#include <tracefctxx/TraceFct.h>
#include <suplibxx/str.h>

#include "FunParamWrap.h"

// we can't use this, as char* will be converted to bool, with really
// bad results. luckily we actually don't have a use for this (yet?)
//
// int FunWrapParamPut(Fun fun, char *name, int n, bool value, char *comm, int append)
// { return FunParamPutb(fun, name, n, value, comm, append); }

int FunWrapParamPut(Fun fun, char *name, int n, int value, char *comm, int append)
{ return FunParamPuti(fun, name, n, value, comm, append); }

int FunWrapParamPut(Fun fun, char *name, int n, unsigned int value, char *comm, int append)
{ return FunParamPuti(fun, name, n, value, comm, append); }

int FunWrapParamPut(Fun fun, char *name, int n, long unsigned int value, char *comm, int append)
{ return FunParamPuti(fun, name, n, value, comm, append); }

int FunWrapParamPut(Fun fun, char *name, int n, double value, char *comm, int append, int prec )
{ return FunParamPutd(fun, name, n, value, prec, comm, append); }

int FunWrapParamPut(Fun fun, char *name, int n, string value, char *comm, int append)
{ return FunParamPuts(fun, name, n, (char*) value.c_str(), comm, append); }

int FunWrapParamPut(Fun fun, char *name, int n, char *value, char *comm, int append)
{ return FunParamPuts(fun, name, n, value, comm, append); }

void ParamPut( 
  Fun fun, 
  string name,
  int idx,
  double val,
  char* comm,
  int append,
  int prec = DBL_DIG
  ) throw( Exception ) {

  TraceFct tf( "Bp2Fits::paramPut (double)" );

  int funret = FunWrapParamPut( fun, (char*) name.c_str(), idx, val, (char*) comm, append, prec );  

  if ( ! funret )
    throw Exception( (string) "Error writing FITS Header parameter " + name);
}


void ParamPutType( 
  Fun fun, 
  char type,
  string name,
  int idx,
  string val,
  char* comm,
  int append,
  int prec
  ) throw ( Exception ) {

  try {

    switch( type ) {

    case 'A':
      ParamPut( fun, name, idx, val.c_str(), comm, append );  
      break;

    case 'D':
      ParamPut( fun, name, idx, suplib::str2d(val.c_str()), comm, append, prec ); 
      break;

    case 'J':
      ParamPut( fun, name, idx, suplib::str2i(val.c_str()), comm, append ); 
      break;

    case 'V':
      ParamPut( fun, name, idx, suplib::str2ul(val.c_str()), comm, append ); 
      break;

    default:
      throw( Exception( string( "Unhandled FITS datatype `" ) + &type + "'" ) );
    }
  }
  catch ( Exception& e )
  {
    e.set_message( string( "Error converting value for FITS parameter " ) + name );
    throw( e );
  }
}

