// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of src
//
// src is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// src is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <suplib/file.h>
#include <Exception/Exception.h>
#include <paramxx/ParFile.h>
#include <tracefctxx/TraceFct.h>
#include "par_names.h"
#include "Bp2Fits.h"
#include "config.h"

using namespace std;

extern int help( void );
extern int usage( void );

int main( int argc, char* argv[] ) {
  ParFile pf;
  bool debug = false;
  
  try { 
    pf.init_ParFile( argc, argv );
    debug = pf.pgetb( DEBUGPAR );

    if ( pf.pgetb( USAGE ) ) {
      usage( );
      return EXIT_SUCCESS;

    }
    if ( pf.pgetb( HELP ) ) {
      help( );
      return EXIT_SUCCESS;

    }
    if ( pf.pgetb( "version" ) ) {
      cout << base_name( argv[0], NULL ) << "\t" << PACKAGE_VERSION << endl;
      return EXIT_SUCCESS;

    }

  } catch ( ParFileException& pfe ) {
    TraceFct tfpar( argv[0], 0, 1 );
    tfpar.exit( EXIT_FAILURE, pfe );

  }

  TraceFct tf( argv[0], 0, 1 );

  try{
    string input = pf.pgetstring( INPUT );

    Bp2Fits bp2fits( 
      input, 
      pf.pgetstring( OUTPUT ), 
      pf.pgetstring( TRANS ),
      debug
      );

    bp2fits.doublepass( pf.pgetb( DOUBLEPASS ) );
    bp2fits.longstrn(   pf.pgetb( LONGSTRN   ) );
    bp2fits.reserved(   pf.pgetb( RESERVED   ) );
    bp2fits.vectors(    pf.pgetb( VECTORS    ) );
    bp2fits.verbose(    pf.pgeti( VERBOSE    ) );

    bp2fits.add_exact_dpktfs( pf.pgetstring( ADD_EXACT_DPKTFS ) );
    bp2fits.add_regex_dpktfs( pf.pgetstring( ADD_REGEX_DPKTFS ) );
    bp2fits.del_exact_dpktfs( pf.pgetstring( DEL_EXACT_DPKTFS ) );
    bp2fits.del_regex_dpktfs( pf.pgetstring( DEL_REGEX_DPKTFS ) );

    bp2fits.writeTable( );

  } catch ( ParFileException& pfe ) {
    tf.exit( EXIT_FAILURE, pfe );

  } catch ( Exception& E ) {

    tf.exit( EXIT_FAILURE, E );

  } catch ( exception& e ) {
    tf.exit( EXIT_FAILURE, e.what( ) );

  }

  return EXIT_SUCCESS;

}
