// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of src
//
// src is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// src is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "Bp2Fits.h"

#include <rdbxx/RDBErr.h>

// map a string to an rdb column.  we ignore errors
// as we don't care about columns that don't exist in
// the table
static void
mapcoldata( RDB& rdb, const char* field, string* strptr )
{
  try { rdb.getColumn(field)->mapData( strptr ); } 
  catch( RDBErr& err ) { }
}

// Read the RDB translation file...
void Bp2Fits::readOverRide( 
  void
  ) throw ( Bp2FitsErr, exception ) {

  TraceFct tf( "Bp2Fits::readOverRide" );
  
  string bpipefield, fitsfield, unit, disp, null, lmin, dmin, dmax,
    lmax, scale, zero;

  // Must have these columns...
  _rdb.getColumn("bpipefield")->mapData( &bpipefield );
  _rdb.getColumn("fitsfield")->mapData( &fitsfield );

  mapcoldata( _rdb, "units", &unit );
  mapcoldata( _rdb, "scal",  &scale );
  mapcoldata( _rdb, "zero",  &zero );
  mapcoldata( _rdb, "disp",  &disp );
  mapcoldata( _rdb, "lmin",  &lmin );
  mapcoldata( _rdb, "dmin",  &dmin );
  mapcoldata( _rdb, "dmax",  &dmax );
  mapcoldata( _rdb, "lmax",  &lmax );

  while ( _rdb.read( ) ) {
    // Only assign values if they are present in the RDB table...
    if ( fitsfield.size( ) ) {

      OverrideInfo* orinfo = new OverrideInfo;


      orinfo->fitsfield = fitsfield;
      orinfo->unit  = unit;
      orinfo->null  = null;
      orinfo->scale = scale;
      orinfo->zero  = zero;
      orinfo->disp  = disp;
      orinfo->lmin  = lmin;
      orinfo->dmin  = dmin;
      orinfo->dmax  = dmax;
      orinfo->lmax  = lmax;

      _orinfo[bpipefield] = orinfo;
      
    }

    bpipefield.erase( );
    fitsfield.erase( );
    unit.erase( );
    disp.erase( );
    null.erase( );
    lmin.erase( );
    dmin.erase( );
    dmax.erase( );
    lmax.erase( );
    scale.erase( );
    zero.erase( );
  }
}
