// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of src
//
// src is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// src is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "Bp2Fits.h"

template<class Type>
static void transferData( 
  ColumnInfo *ci,
  Type data_offset
  ) {
#ifdef DEBUG
  TraceFct tf( "Bp2Fits::transferData" );
#endif

  Type* src = (Type*) ci->src;
  Type* min = (Type*) ci->min;
  Type* max = (Type*) ci->max;

  for ( size_t idx = 0 ; idx < ci->nelem; idx++, src = (Type*) ((char*) src + ci->stride)) {

    // derive min and max of data
    if ( *min > *src ) *min = *src; 
    if ( *max < *src ) *max = *src; 

    *src -= data_offset;

    // need to copy the data if we allocated memory
    if ( ci->allocate )
      ((Type*) ci->data)[idx] = *src;

  }
}

void Bp2Fits::readwriteData(
  void
  ) throw( Bp2FitsErr, exception ) { 

  // we've already read in a data packet to use to initialize
  // min and max fields.

  do {
    _last = NULL;

    vector<ColumnInfo*>::iterator cip;
    for ( cip = _ci.begin(); cip != _ci.end(); cip++ )
    {
      ColumnInfo *ci = *cip;

      switch( ci->cxxdt )
      {
      case DT_int:
	transferData( ci, (int) 0 ); 
	break;

      case DT_uint:
	transferData( ci, (unsigned int) 2147483648U );
	break;

      case DT_double:
	transferData( ci, (double) 0 );
	break;

      case DT_char:
	transferData( ci, (char) 0 );
	break;
      }

    }
    
    if ( ! FunTableRowPut( _fun, _data, 1, _nrows++, NULL ) )
    {
      stringstream ss;
      ss << "Error writing row " << _nrows-1 << endl;
      throw( Exception( ss.str() ) );
    }
    
  } while ( bpipe_read_dpkts( _bpipe, _dpkt, 1 ) );

}


