#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>

/* simple print error and exit routine */
#define error(string)                                                   \
  do {                                                                  \
      fprintf( stderr, __FILE__ " %d: %s\n", __LINE__, string );        \
      exit( EXIT_FAILURE );                                             \
     } while (0)

#define POSX 2
#define POSY 3

int 
main (int argc, char *argv[])
{
  BPipe *bpipe;
  BPipeOutput *bpo;
  double *weight;
  DVector3 *position;
  DVector3 *direction;
  unsigned int *id;

  void *data;

  size_t i;
  size_t nphots = 20;
  
  if ( NULL == ( bpipe = bpipe_new( ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* create new data packet fields */

  {
    BPMatrix* matrix = bpipe_matrix_new_va( 2, 1, 1 );
    if ( bpipe_dpktf_add( bpipe,  "weight",       BPDType_double,   matrix ) )
      error( bpipe_strerror( bpipe_errno ) );
  }

  {
    BPMatrix* matrix = bpipe_matrix_new_va( 2, POSX, POSY );

    if ( bpipe_dpktf_add( bpipe,  "position",  BPDType_DVector3,   matrix ) )
      error( bpipe_strerror( bpipe_errno ) );
  }

  if ( bpipe_dpktf_add( bpipe,  "direction",  BPDType_DVector3,   NULL ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( bpipe_dpktf_add( bpipe,  "id",       BPDType_uint,   NULL ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( bpo = bpipe_output( bpipe, "stdout" ) ) )
    error( bpipe_strerror( bpipe_errno ) );
    
  /* map data packet fields */
  if ( NULL == ( data = bpipe_map_alloc( bpipe, 1, NULL ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* output header */
  if ( bpipe_write_hdr(bpipe) )
    error( bpipe_strerror( bpipe_errno ) );

  /* get handles to data packet fields */
  weight = bpipe_dpktf_datap( bpipe, data, "weight", BPDType_double );
  position = bpipe_dpktf_datap( bpipe, data, "position", BPDType_DVector3 );
  direction = bpipe_dpktf_datap( bpipe, data, "direction", BPDType_DVector3 );
  id = bpipe_dpktf_datap( bpipe, data, "id", BPDType_uint );

  /* write the data packets out */
  for (i = 0; i < nphots ; i++)
  {
    size_t j;
    size_t k;
    size_t idx;

    *id  = i;
    *weight = i + 0.5;

    for ( idx = j = 0 ; j < POSX ; j++ )
      for ( k = 0 ; k < POSY ; k++, idx++ )
      {
	position[idx].x = i + idx;
	position[idx].y = i + idx + 1;
	position[idx].z = i + idx + 2;
      }

    direction->x = i;
    direction->y = 2*i;
    direction->z = 3*i;

    if ( bpipe_write_dpkt( bpipe, data, bpo ) )
      error( "error writing photon" );
  }

  bpipe_delete(bpipe);

  return EXIT_SUCCESS;
}

