/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of src
 *
 * src is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * src is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"

void
dump_dpkts( BPipe *bpipe, FieldInfo** fields, FILE *output )
{
  void* core;
  FieldInfo** fip;

  tf_enter( "dump_dpkts" );

  /* allocate memory for the core image */
  if ( NULL == ( core = bpipe_map_alloc( bpipe, 1, NULL ) ) )
    tf_die( bpipe_strerror( bpipe_errno ) );

  /* update data pointer for the fields, for some small efficiency */
  for ( fip = fields ; *fip ; fip++ )
    (*fip)->data = bpipe_dpktf_data( (*fip)->dpktf, core ) + (*fip)->offset;

  /* loop through data packets */
  while( bpipe_read_dpkts( bpipe, core, 1 ) )
  {
    char buf[1024];
    int first = 1;

    for ( fip = fields ; *fip ; fip++ )
    {
      if ( first )
	first = 0;
      else
	fputc( '\t', output );

      if ( (*fip)->putchar )
	fputc( bpipe_dpktf_val( (*fip)->dpktf, core, char ), output );
      else
      {
	bpipe_sprintf( buf, (*fip)->data, (*fip)->type, NULL );
	fputs( buf, output );
      }
    }
    fputc( '\n', output );
  }

  /*
    we'll get an error out of bpipe_read_dpkts if something's gone awry.
    bpipe_errno should have been equal to BPNOERROR when entering the
    above loop
   */
  if ( bpipe_errno != BPNOERROR )
    tf_exit( ExitERR_misc, bpipe_strerror( bpipe_errno ) );
}
