/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of src
 *
 * src is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * src is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"

void
dump_hdrs( BPipe *bpipe, FILE *output )
{

  BPMatrix *matrix;
  size_t index;
  void *last;
  char *name;

  tf_enter( "dump_hdrs" );

  /* step through header fields and print out their data */

  /* last is used by bpipe_hdrf_next to keep track of where it is */
  last = NULL;	/* initialize it for first time through */

  /* loop until there are no more header fields available */
  while ( bpipe_hdrf_next( bpipe, &name, &index, &last ) )
  {
    char buf[1024];

    /*
      extract the data type and a pointer to the data.  these should
      be valid since we've been handed name and index by bpipe_hdrf_next
     */
    BPDataType type = bpipe_hdrf_type( bpipe, name, index );
    void *     data = bpipe_hdrf_data( bpipe, name, index );

    if ( bpipe_errno != BPNOERROR )
      tf_exit( ExitERR_misc, "unexpected error while outputting header: %s",
	       bpipe_strerror( bpipe_errno ) );

    /*
      extract matrix spec.  since it's copying the matrix, there's a
      possibility that we're out of memory, which we check for
     */
    if ( NULL == (matrix = bpipe_hdrf_matrix ( bpipe, name, index ) ) )
      tf_exit( ExitERR_misc, "error getting matrix spec: %s",
	       bpipe_strerror( bpipe_errno ) );

    /* dump the data */

    /* currently, ignore multidimensional data */
    if ( 1 == matrix->nd )
    {
      if ( BPDType_char == type || 1 == matrix->ne )
      {
	bpipe_sprintf( buf, data, type, NULL );
	fprintf( output, "#: %s = %s\n", name, buf );
      }
    }
    /*
      we're responsible for deleting the matrix spec returned by
      bpipe_hdrf_matrix
     */
    bpipe_matrix_delete( matrix );
  }

  tf_leave( );
}
