/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of src
 *
 * src is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * src is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"

static void	print_hdr( FILE *output, DpktField *dpktf, int defs );
static void	dump_hdr(  FieldInfo* fields[], FILE *output, int defs );

void
dump_rdb_hdr( FieldInfo* fields[], FILE *output )
{
  tf_enter( "dump_rdb_hdr" );

  dump_hdr( fields, output, 0 );
  dump_hdr( fields, output, 1 );

  tf_leave( );

}

static void
dump_hdr(  FieldInfo* fields[], FILE *output, int defs )
{
  FieldInfo** fip;
  int first = 1;

  tf_enter( "dump_hdr" );
  
  for ( fip = fields ; *fip ; fip++ )
  {
    if ( first )
      first = 0;
    else
      fputc( '\t', output );

    print_hdr( output, (*fip)->dpktf, defs );
  }

  fputc( '\n', output );

  tf_leave( );
}

static void
print_hdr( FILE *output, DpktField *dpktf, int defs )
{
  BPDataType type;
  const char *name;

  tf_enter( "print_hdr" );

  type = bpipe_dpktf_type( dpktf );
  name = bpipe_dpktf_name( dpktf );

  /* this is hideous, but until bpipe provides this function,
     we need to do this */
  if ( defs )
    switch( type )
    {
    case BPDType_char:
      fputs( "S", output );
      break;

    case BPDType_double:
    case BPDType_int:
    case BPDType_uint:
      fputs( "N", output );
      break;
    
    case BPDType_DVector2:
    case BPDType_IVector2:
    case BPDType_UIVector2:
    case BPDType_DComplex:
      fputs( "N\tN", output );
      break;
    
    case BPDType_DVector3:
    case BPDType_IVector3:
    case BPDType_UIVector3:
      fputs( "N\tN\tN", output );
      break;


    case BPDType_DCVector2:
      fputs( "N\tN\tN\tN", output );
      break;
    
    case BPDType_DCVector3:
      fputs( "N\tN\tN\tN\tN\tN", output );
      break;

    default:
      tf_assert( 0 );
    }

  else
    switch( type )
    {
    case BPDType_char:
    case BPDType_double:
    case BPDType_int:
    case BPDType_uint:
      fputs( name, output );
      break;
    
    case BPDType_DVector2:
    case BPDType_IVector2:
    case BPDType_UIVector2:
      fprintf( output, "%s_x", name );
      fputc( '\t', output);
      fprintf( output, "%s_y", name );
      break;
    
    case BPDType_DVector3:
    case BPDType_IVector3:
    case BPDType_UIVector3:
      fprintf( output, "%s_x", name );
      fputc( '\t', output);
      fprintf( output, "%s_y", name );
      fputc( '\t', output);
      fprintf( output, "%s_z", name );
      break;
    
    case BPDType_DComplex:
      fprintf( output, "%s_r\t%s_i", name, name );
      break;
    
    case BPDType_DCVector2:
      fprintf( output, "%s_q1_r\t%s_q1_i", name, name );
      fputc( '\t', output);
      fprintf( output, "%s_q2_r\t%s_q2_i", name, name );
      break;
    
    case BPDType_DCVector3:
      fprintf( output, "%s_q1_r\t%s_q1_i", name, name );
      fputc( '\t', output);
      fprintf( output, "%s_q2_r\t%s_q2_i", name, name );
      fputc( '\t', output);
      fprintf( output, "%s_q3_r\t%s_q3_i", name, name );
      break;
    
    default:
      tf_assert( 0 );
    }

  tf_leave( );
}
