/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of bp2rdb
 *
 * bp2rdb is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * bp2rdb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>

#include <bpipe/bpipe.h>

#include "main.h"
#include "field_info.h"


FieldInfo *new_FieldInfo( DpktField* dpktf )
{
  FieldInfo* fi = malloc( sizeof(FieldInfo) );

  if ( NULL == fi )
    return NULL;

  fi->dpktf  = dpktf;
  fi->offset = 0;
  fi->type   = bpipe_dpktf_type( dpktf );
  fi->size   = bpipe_datatype_size( fi->type );

  fi->matrix = bpipe_dpktf_matrix( dpktf, BPDSite_CORE, NULL );

  if ( NULL == fi->matrix )
  {
    free( fi );
    return NULL;
  }

  fi->putchar = BPDType_char == bpipe_dpktf_type(dpktf) && fi->matrix->nd == 1;

  return fi;
}


void
free_FieldInfo( FieldInfo *fi )
{
  if ( NULL == fi )
    return;

  if ( NULL != fi && fi->matrix )
    bpipe_matrix_delete( fi->matrix );

  free( fi );
}


void
free_FieldInfo_arr( FieldInfo *fields[] )
{
  FieldInfo** fip;

  if ( NULL == fields )
    return;

  for( fip = fields ; *fip ; fip++)
    free_FieldInfo( *fip );

  free( fields );
}

int
dump_FieldInfo( FieldInfo* fi )
{
  if ( fi->dpktf )
    fprintf( stderr, "name = %s\n", bpipe_dpktf_name( fi->dpktf ) );

  fprintf( stderr, "type = %s\n", bpipe_datatype_name( fi->type ) );
  fprintf( stderr, "offset = %lu\n", fi->offset );

  return 0;
}
