/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of src
 *
 * src is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * src is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <string.h>
#include <errno.h>

#include <suplib/str.h>

#include <tparam/tparam.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>


#include "main.h"

static TokListToken output_element[] =
{
  { "all"   , OE_All          },
  { "data"  , OE_Data         },
  { "hdr"   , OE_Hdr          },
  { "hdrvar", OE_HdrVariables }
};
static TokList output_elements = GenTokList(output_element);

void
gparam(
  int argc,
  char *argv[],
  Param *param
  )

{
  char buffer[NAME_LENGTH+1];
  paramfile pf;

  tf_enter( "gparam" );

  pf = tparamopen( NULL, argv, argc, "r" );
  
  /* ------------------------------------ */
  /* only get other stuff if version isn't set */
  if (
          ( param->version = tpgetb( pf, "version" ) )
       || ( param->usage   = tpgetb( pf, "usage" ) )   
       || ( param->help    = tpgetb( pf, "help" ) )
     )
  {
    tf_leave();
    return;
  }

  tpgetstr( pf, "input",  param->input,  NAME_LENGTH, 1 );
  tpgetstr( pf, "output", param->output, NAME_LENGTH, 1 );

  /* ------------------------------------ */
  /* which fields to output */
  tpgetstr( pf, "fields", buffer, NAME_LENGTH, 0 );

  /* param->fields is freed in process() */
  param->fields = NULL;	/* so that later on str_tokbqenize_free doesn't
			   segv if there are no fields read in */

  {
    int error;
    struct str_tokbqdata tbqd;
    param->nfields = str_tokbqenize( buffer, &param->fields, ",",
				     "[", "]", '\\', STRTOK_SKIP, &tbqd,
				     &error
				     );
    switch( error )
    {
    case ENOMEM:
      tf_exit( ExitERR_alloc,
	       "out of memory getting tokens for fields parameter\n" );
      break;

    case EINVAL:
      tf_exit( ExitERR_misc, "unbalanced brackets in fields parameter" );
      break;
    }
  }    

  /* ------------------------------------ */
  /* which elements to output */
  tpgetstr( pf, "outelem", buffer, NAME_LENGTH, 0 );
  if ( 0 == strlen(buffer) )
    strcpy( buffer, "all" );
  {
    int ii;
    char **token;
    int ntoks = str_tokenize( buffer, ",", &token, 1, 0 );


    param->elements = OE_None;
    for ( ii = 0 ; ii < ntoks ; ii++ )
    {
      int match = tokmatch( token[ii], &output_elements );
      if ( -1 == match )
	tf_exit( ExitERR_illegalArg, "%s: unknown output element\n",
		 token[ii] );
      param->elements |= match;
    }

    str_tokenize_free( token, 0 );
  }

  /* ------------------------------------ */

  paramclose( pf );
  tf_leave( );
}
