/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of src
 *
 * src is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * src is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <string.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <suplib/str.h>

#include "main.h"

void
process (Param *param)
{
  BPipe *bpipe;
  FieldInfo** fields;
  FILE *fout = stdout;


  tf_enter( "process" );

  /* create BPipe object */
  bpipe = attach_bpipe( param->input );

  
  /*--------------------------------------*/
  /* open output file */
  if ( strcmp( "stdout", param->output ) )
    fout = fopen( param->output, "wb" );

  if ( NULL == fout )
    tf_die( "%s: unable to open output stream\n", param->output );


  /*--------------------------------------*/
  /* determine output fields */
  fields = get_output_dpktfs( bpipe, param->nfields, param->fields );

  str_tokenize_free( param->fields, 1 );

  if ( NULL == fields )
    tf_die( "no output fields specified or remaining\n" );


  if ( param->elements & OE_Hdr )
  {
    /* output header variables */
    if ( param->elements & OE_HdrVariables )
      dump_hdrs( bpipe, fout );

    /* output RDB table header */
    dump_rdb_hdr( fields, fout );
  }

  /* output data packets */
  if ( param->elements & OE_Data )
    dump_dpkts( bpipe, fields, fout );

  free_FieldInfo_arr( fields );

  bpipe_delete( bpipe );
  
  fclose( fout );

  tf_leave();
}

