/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of bp2rdb
 *
 * bp2rdb is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * bp2rdb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>

/* simple print error and exit routine */
#define error(string)                                                   \
  do {                                                                  \
      fprintf( stderr, __FILE__ " %d: %s\n", __LINE__, string );        \
      exit( EXIT_FAILURE );                                             \
     } while (0)

#define POSX 2
#define POSY 3

int 
main (int argc, char *argv[])
{
  BPipe *bpipe;
  BPipeOutput *bpo;
  double *weight;
  DVector3 *position;
  DVector3 *direction;
  unsigned int *id;

  void *data;

  size_t i;
  unsigned int nphots = 20;
  
  if ( argc > 1 )
    nphots = atoi( argv[1] );

  if ( NULL == ( bpipe = bpipe_new( ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* add header field(s) */
  bpipe_hdrf_add( bpipe, "nphots", BPDType_uint, NULL, &nphots, 0 );

  /* create new data packet fields */
  {
    BPMatrix* matrix = bpipe_matrix_new_va( 2, 1, 1 );
    if ( bpipe_dpktf_add( bpipe,  "weight",       BPDType_double,   matrix ) )
      error( bpipe_strerror( bpipe_errno ) );
  }

  {
    BPMatrix* matrix = bpipe_matrix_new_va( 2, POSX, POSY );

    if ( bpipe_dpktf_add( bpipe,  "position",  BPDType_DVector3,   matrix ) )
      error( bpipe_strerror( bpipe_errno ) );
  }

  if ( bpipe_dpktf_add( bpipe,  "direction",  BPDType_DVector3,   NULL ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( bpipe_dpktf_add( bpipe,  "id",       BPDType_uint,   NULL ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( bpo = bpipe_output( bpipe, "stdout" ) ) )
    error( bpipe_strerror( bpipe_errno ) );
    
  /* map data packet fields */
  if ( NULL == ( data = bpipe_map_alloc( bpipe, 1, NULL ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* output header */
  if ( bpipe_write_hdr(bpipe) )
    error( bpipe_strerror( bpipe_errno ) );

  /* get handles to data packet fields */
  weight = bpipe_dpktf_datap( bpipe, data, "weight", BPDType_double );
  position = bpipe_dpktf_datap( bpipe, data, "position", BPDType_DVector3 );
  direction = bpipe_dpktf_datap( bpipe, data, "direction", BPDType_DVector3 );
  id = bpipe_dpktf_datap( bpipe, data, "id", BPDType_uint );

  /* write the data packets out */
  for (i = 0; i < nphots ; i++)
  {
    size_t j;
    size_t k;
    size_t idx;

    *id  = i;
    *weight = i + 0.5;

    for ( idx = j = 0 ; j < POSX ; j++ )
      for ( k = 0 ; k < POSY ; k++, idx++ )
      {
	position[idx].x = i + idx;
	position[idx].y = i + idx + 1;
	position[idx].z = i + idx + 2;
      }

    direction->x = i;
    direction->y = 2*i;
    direction->z = 3*i;

    if ( bpipe_write_dpkt( bpipe, data, bpo ) )
      error( "error writing photon" );
  }

  bpipe_delete(bpipe);

  return EXIT_SUCCESS;
}

