/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of bpipe
 *
 * bpipe is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * bpipe is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

/* simple print error and exit routine */
#define error(string)							\
  do {									\
      fprintf( stderr, __FILE__ " %d: %s\n", __LINE__, string );	\
      exit( EXIT_FAILURE );						\
     } while (0)

int 
main (int argc, char *argv[])
{
  BPipe *bpipe;
  BPipeOutput *bpo;
  void *data;
  char *name;

  size_t i;

  if ( argc < 2 )
  {
    fprintf( stderr, "usage: %s <hdr field name>\n", argv[0] );
    exit(EXIT_FAILURE);
  }
  name = argv[1];

  if ( NULL == ( bpipe = bpipe_new( ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( bpipe_input( bpipe, "stdin" ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( bpo = bpipe_output( bpipe, "stdout" ) ) )
    error( bpipe_strerror( bpipe_errno ) );


  if ( NULL == bpipe_dpktf( bpipe, name )  )
  {
    fprintf( stderr, "dpktf `%s' not found, should have\n", name );
    exit(EXIT_FAILURE);
  }

  bpipe_dpktf_delete( bpipe, name, WHERE, bpo );

  if (NULL == (data = bpipe_map_alloc( bpipe, 1, NULL ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( WHERE == BPDSite_CORE && NULL != bpipe_dpktf( bpipe, name )  )
  {
    fprintf( stderr, "dpktf `%s' found, shouldn't have\n", name );
    exit(EXIT_FAILURE);
  }
  
  if ( WHERE == BPDSite_OUTPUT && NULL == bpipe_dpktf( bpipe, name )  )
  {
    fprintf( stderr, "dpktf `%s' not found, should have\n", name );
    exit(EXIT_FAILURE);
  }

  if ( bpipe_write_hdr(bpipe) )
    error( bpipe_strerror( bpipe_errno ) );

  /* loop through data packets */
  i = 0;
  bpipe_errno = BPNOERROR;
  while( bpipe_read_dpkts( bpipe, data, 1 ) )
  {
    i++;
    
    /* write the data packet out */
    if ( bpipe_write_dpkt( bpipe, data, bpo ) )
    {
      fprintf( stderr, "error writing photon %d\n", i );
      exit(EXIT_FAILURE);
    }
  }

  if ( bpipe_errno != BPNOERROR )
    error( bpipe_strerror( bpipe_errno ) );

  bpipe_delete(bpipe);

  return EXIT_SUCCESS;
}
