// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of fits2bp
//
// fits2bp is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// fits2bp is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <string>
#include <bpipe/bpipe.h>
#include <suplib/str.h>
#include <tracefctxx/TraceFct.h>

#include "bpmap.h"

using namespace std;

int
bpList_node_dump( void *node )
{
  BPipeMap_bpList_Entry* entry = (BPipeMap_bpList_Entry*) node;
  cerr << "bpipe name = " << entry->name << endl;
  cerr << "type = " << bpipe_datatype_name(entry->type) << endl;
  if ( entry->mx )
    cerr << "extent = " << entry->mx->extent[0] << endl;
  cerr << "used = " << entry->used << endl;
  cerr << endl;

  return 0;
}

int
mapList_node_dump( void *node )
{
  BPipeMap_mapList_Entry* entry = (BPipeMap_mapList_Entry*) node;
  cerr << "map name = " << entry->name << endl;
  cerr << "type = " << bpipe_datatype_name(entry->type) << endl;
  cerr << "alien type = " << entry->alien_type << endl;
  cerr << "offset = " << entry->offset << endl;
  cerr << "col = " << entry->col << endl;
  cerr << "extent = " << entry->extent << endl;
  if ( entry->bp )
    bpList_node_dump( entry->bp );

  cerr << endl;
  return 0;
}


int
bpList_node_cmp( const void *dp1, const void *dp2 )
{
  return 
    ((BPipeMap_bpList_Entry*) dp1)->name.compare(
                       ((BPipeMap_bpList_Entry*) dp2)->name );
}

int
bpList_key_cmp( const void *key, const void *dp2 )
{
  return ((const string*) key)->compare(
		 ((BPipeMap_bpList_Entry*) dp2)->name );
}

int
mapList_node_cmp( const void *dp1, const void *dp2 )
{
  return ((BPipeMap_mapList_Entry*) dp1)->name.compare(
		 ((BPipeMap_mapList_Entry*) dp2)->name );
}

int
mapList_key_cmp( const void *key, const void *dp2 )
{
  return ((const string*) key)->compare(
		 ((BPipeMap_mapList_Entry*) dp2)->name );
}



void
mapList_delete( BPipeMap_mapList_Entry* entry )
{
  delete entry;
}


BPipeMap_mapList_Entry*
new_mapList_Entry( const string& err_pfx,
		   const string& name,
		   BPipeMap_bpList_Entry *bps, 
		   size_t offset, 
		   BPDataType type,
		   size_t col )
{
  BPipeMap_mapList_Entry* mps = new BPipeMap_mapList_Entry;

  mps->name = name;
  mps->bp = bps;
  mps->offset = offset;
  mps->col = col;
  mps->type = type;
  mps->alien_type = 0;

  return mps;
}

BPipeMap_bpList_Entry*
new_bpList_Entry( const string& err_pfx,
		  const string& name,
		  BPDataType type,
		  int ext, int used )
{
  TraceFct tf( "new_bpList_Entry" );

  BPipeMap_bpList_Entry* bps = new BPipeMap_bpList_Entry;

  bps->name = name;
  
  if ( ext > 1 )
  {
    size_t* extent = (size_t *) bpipe_extent_new( 1, ext );
    if ( NULL == extent )
      tf.exit( 1, err_pfx + "unable to allocate extent" );

    bps->mx = bpipe_matrix_new( 1, extent );
    if ( NULL == bps->mx )
      tf.exit( 1, err_pfx + "unable to allocate matrix" );
  }
  else
    bps->mx = NULL;

  bps->used = used;
  bps->type = type;

  return bps;
}

void
bpList_delete( BPipeMap_bpList_Entry* entry )
{
  if ( entry->mx )
  {
    if (entry->mx->extent)
      free(entry->mx->extent);
    free( entry->mx );
  }

  delete entry;
}

