// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of fits2bp
//
// fits2bp is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// fits2bp is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
using namespace std;

#include <tracefctxx/TraceFct.h>
#include <linklist/linklist.h>
#include <fitsio.h>
#include "fits2bp.h"
#include "bpmap.h"

static int bp2fits_type( BPDataType type );

void
clean_list( SLinkList bp_list, SLinkList map_list )
{
  SLLNode next_node;

  // remove entries in map_list which aren't being used
  // update bp used flag while we're at it.
  for ( SLLNode node = sll_head_node( map_list ) ;
	node ;
	node = next_node )
  {
    BPipeMap_mapList_Entry* entry = 
      (BPipeMap_mapList_Entry*) sll_node_get_data( node );


    next_node = sll_next_node( map_list, node );

    if ( 0 == entry->col )
    {
      sll_destroy_node( map_list, node );
      mapList_delete( entry );
    }
    else
    {
      entry->bp->used++;
      entry->alien_type = bp2fits_type( entry->type );
    }
  }

  // remove entries in bp_list which aren't being used
  for ( SLLNode node = sll_head_node( bp_list ) ;
	node ;
	node = next_node )
  {
    BPipeMap_bpList_Entry* entry = (BPipeMap_bpList_Entry*)
      sll_node_get_data( node );
    next_node = sll_next_node( bp_list, node );
    if ( 0 == entry->used )
    {
      sll_destroy_node( bp_list, node );
      bpList_delete( entry );
    }
  }
}

Column *
create_column_arr( SLinkList map_list )
{
  TraceFct tf( "create_column_arr" );

  int ncols = sll_count(map_list );
  Column *column = (Column *) malloc( ncols * sizeof(Column) );
  if ( NULL == column )
    tf.exit(1, "unable to allocate Column array" );

  SLLNode node = sll_head_node( map_list );

  for ( int i = 0 ; 
	i < ncols ;
	i++,
	  node = sll_next_node( map_list, node ) 
	)
  {
    BPipeMap_mapList_Entry* entry = 
      (BPipeMap_mapList_Entry*) sll_node_get_data( node );

    column[i].type = entry->alien_type;
    column[i].addr = entry->addr;
    column[i].col  = entry->col;
    column[i].extent = entry->extent;

#if 0
    cerr << "Column " << i << endl;
    cerr << "type = " << column[i].type << endl;
    cerr << "col = " << column[i].col << endl;
    cerr << "extent = " << column[i].extent << endl;
#endif

  }

  return column;
}

static int
bp2fits_type( BPDataType type )
{
  TraceFct tf("bp2fits_type");
  switch( type )
  {
  case BPDType_char: 	return TBYTE;
  case BPDType_double:	return TDOUBLE;
  case BPDType_int:	return TLONG;
  case BPDType_uint:	return TULONG;
  }

  tf.exit( 1, "Something wicked this way comes...\n" );
  // NOT REACHED
  return -1;
}
