// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of fits2bp
//
// fits2bp is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// fits2bp is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <libgen.h>

using namespace std;

#include <tracefctxx/TraceFct.h>
#include <linklist/linklist.h>
#include <paramxx/ParFile.h>
#include <Exception/Exception.h>

#include "fits2bp.h"
#include "bpmap.h"
#include "input.h"
#include "output.h"
#include "parse_spec.h"
#include "clean_list.h"
#include "xfer_data.h"

#include "config.h"

int
main( int argc, char *argv[] )
{
  TraceFct tf( argv[0], 0, 0 );
  
  try {

    SLinkList bp_list;
    SLinkList map_list;
    
    ParFile parfile( argc, argv );
    
    if ( parfile.pgetb( "version" ) )
    {
      cout << basename( argv[0] ) << " " << VERSION << endl;
      return EXIT_SUCCESS;
    }
    
    if ( parfile.pgetb( "help" ) )
    {
      usage();
      return EXIT_SUCCESS;
    }
    
    parse_spec( parfile.pgetstring( "spec" ).c_str(), bp_list, map_list );
    
    char *extname;
    {
      int len =  parfile.pgetstring( "extname" ).size( );
      extname = new char[ len + 1 ];
      parfile.pgetstring( "extname" ).copy( extname, string::npos );
      extname[len] = 0;
      }
    Input input = open_fits( parfile.pgetstring( "input" ).c_str(),
			     bp_list, 
			     map_list, 
			     extname,
			     parfile.pgetb( "auto_create" ) ); 


#if 0
    cerr << "bpipe list\n" ;
    sll_traverse( bp_list, bpList_node_dump );
    cerr << "\nmap list\n" ;
    sll_traverse( map_list, mapList_node_dump );
#endif


    delete [] extname;
    clean_list( bp_list, map_list );
    

    Output output = open_bpipe( parfile.pgetstring( "output" ).c_str(),
				bp_list,
				map_list ); 
    
    Column *column = create_column_arr( map_list );
    
    xfer_data( input, output, column, sll_count( map_list ) ); 
    
    close_fits( input );
    
    close_bpipe(output);
    
    free( column );
    
    sll_delete( bp_list, (SLL_DELETE) bpList_delete );
    sll_delete( map_list, (SLL_DELETE) mapList_delete );
    
  } catch ( ParFileException& pfe ) {

    cerr << pfe << endl;
    tf.exit( EXIT_FAILURE, "ParFileException caught\n" );

  } catch ( Exception& E ) {

    cerr << "Unexpected Exception caught\n" << endl;
    tf.exit( EXIT_FAILURE, "Unexpected object of class 'Exception' caught\n" );

  } catch ( exception& e ) { 

    cerr << "Unexpected exception caught\n" << endl;
    tf.exit( EXIT_FAILURE, "Unexpected object of class 'exception' caught\n" );

  }      
}
