// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of fits2bp
//
// fits2bp is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// fits2bp is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <bpipe/bpipe.h>
#include <linklist/linklist.h>
#include <tracefctxx/TraceFct.h>

#include "fits2bp.h"
#include "output.h"
#include "bpmap.h"

Output
open_bpipe( const char*file, SLinkList bp_list, SLinkList map_list )
{
  Output out;
  
  TraceFct tf( "open_bpipe" );

  out.bp = bpipe_new();
  if ( NULL == out.bp )
    tf.exit( 1, "unable to create bpipe" );

  for ( SLLNode node = sll_head_node( bp_list ) ;
	node ;
	node = sll_next_node( bp_list, node ) )
  {
    BPipeMap_bpList_Entry* entry = (BPipeMap_bpList_Entry*)
      sll_node_get_data( node );

    int ret = bpipe_dpktf_add( out.bp, entry->name.c_str(), entry->type,
			       entry->mx);

    if ( ret )
      tf.exit( 1, "error adding bpipe field `%s': %s",
	       bpipe_strerror(bpipe_errno) );
    
    // matrix now handled by bpipe; erase knowledge of it
    entry->mx = NULL;

  }


  out.bpo = bpipe_output( out.bp, (char*) file );
  if ( NULL == out.bpo )
      tf.exit( 1, "error adding bpipe output `%s': %s",
	       bpipe_strerror(bpipe_errno) );
    
  out.data = bpipe_map_alloc(out.bp, 1, NULL);
  if ( NULL == out.bpo )
      tf.exit( 1, "error mapping bpipe: %s",
	       bpipe_strerror(bpipe_errno) );

  bpipe_write_hdr( out.bp );

  
  // grab absolute address of core memory spot
  for ( SLLNode node = sll_head_node( map_list ) ;
	node ;
	node = sll_next_node( map_list, node ) )
  {
    BPipeMap_mapList_Entry* entry = (BPipeMap_mapList_Entry*)
      sll_node_get_data( node );

    DpktField *dpktf = bpipe_dpktf( out.bp, entry->bp->name.c_str() );
    if ( NULL == dpktf )
      tf.exit( 1, "field doesn't exist:  '%s'", entry->bp->name.c_str() );

    entry->addr = bpipe_dpktf_data( dpktf, out.data ) + entry->offset;
  }


  return out;
}

void
close_bpipe( Output out )
{
  TraceFct tf( "close_bpipe" );
  
  bpipe_delete( out.bp );
}

