// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of fits2bp
//
// fits2bp is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// fits2bp is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
using namespace std;

#include <bpipe/bpipe.h>
#include <fitsio.h>
#include <tracefctxx/TraceFct.h>

#include "fits2bp.h"

#include "fits_error.h"

#include "xfer_data.h"

void
xfer_data( Input& input, Output& output, Column *column, int ncols )
{
  TraceFct tf( "xfer_data" );
  long nrows;
  int status = 0;

  fits_get_num_rows( input.fptr, &nrows, &status );
  fits_error( "get_num_rows", status );

  for( int row = 1 ; row <= nrows ; row++ )
  {
    Column *col = column;
    for ( int ci = 1 ; ci <= ncols ; ci++, col++ )
    {
#if 0
      cerr << "column = " << ci << "  type = " << col->type << 
	" col = " << col->col << " extent = " << col->extent << endl;
#endif

      fits_read_col( input.fptr, 
		     col->type, 
		     col->col,
		     row,
		     1, col->extent,
		     NULL, col->addr, NULL, &status );
      if ( status )
      {
	char err_pfx[1024];
	sprintf( err_pfx, "row %d col %d (FITS col %d)", 
		 row, ci, col->col );
	fits_error( err_pfx, status );
      }
    }
    if ( bpipe_write_dpkt( output.bp, output.data, output.bpo ) )
      tf.exit( 1, "error writing row %d to output file", row );
  }

}
