#ifndef FLATBPIPERAY_H
#define FLATBPIPERAY_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of flat
 *
 * flat is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * flat is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <limits.h>

#include <bpipe/datatypes.h>

#include "bitmask.h"

typedef struct {
    DVector3     *pos;       /* ray position                              */
    DVector3     *dir;       /* ray direction vector (dir cosines)        */
    DVector3     *norm ;     /* surf normal, most recent reflection       */
    double       *wt;        /* statistical weight of ray                 */
    double       *energy;    /* photon energy                             */
    double       *g_ang;     /* graze angle                               */
    DCVector2    *c2;        /* polarization amplitude (cosine comp)      */
    DCVector2    *s2;        /* polarization amplitude (sine comp)        */
    unsigned int *rayid;     /* ray id number                             */
    int          *raycode;   /* flat ray code                             */
    bitmask_t    *missed;    /* surface hit flags                         */
} Ray;

#endif /* FLATBPIPERAY_H */





