#ifndef MAIN_H
#define MAIN_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of flat
 *
 * flat is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * flat is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include "flatbpipe.h"

#define NAME_SIZE 255
#define DEBUG_LENGTH 255

typedef struct
{
  char input[NAME_SIZE+1];
  char output[NAME_SIZE+1];
  char debug[DEBUG_LENGTH+1];
  
  double x0,y0,z0;
  double normx,normy,normz;
  double height, width, rotation;
  int srfno;
  int help;
  int version;
  int printdebug;

} Common;

/* common.c */
/* gparam.c */
void gparam(int argc, char *argv[]);
/* main.c */
int main(int argc, char *argv[]);
/* vector_ops.c */
double dot_product(DVector3 *a, DVector3 *b);
void normalize(DVector3 *vector);
/* matrix_ops.c */
double **inverse_transformation(double **a);
double **matrix_mult(double **a, double **b);
void print_4x4_matrix(double **mat, char *str);
double **rotation(XYZaxis axis, double theta);
double **rotate(XYZaxis axis, double angle);
double **translation(double tran[ DIM3 ]);
void transform_position(double **tran, DVector3 *in, DVector3 *out);
void transform_direction(double **tran, DVector3 *in, DVector3 *out);
/* rayfuncs.c */
double calc_rayscalar(Ray *ray, DVector3 *inmirror, DVector3 *normal);
void find_intersection(Ray *ray, double rayscalar, DVector3 *inmirror);
int check_optic_hit(Ray *ray, DVector3 *inter_transf, DVector3 *normal);
/* process.c */
void process(void);
/* readbp.c */
void init_bpipe(char *input, char *output, Bpipe *pipe, Ray *photon_offset);
/* usage.c */
int usage(void);

#endif /* !MAIN_H */
