/****************************************************************************
 * Source     : rayfuncs.c
 * Author     : David Grumm
 * Description: ray functions
 */

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of flat
 *
 * flat is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * flat is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <unistd.h> 
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <mst_nrutil/mst_nrutil.h>
#include <bpipe/datatypes.h>
#include <mathconst/mathconst.h>    

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

#include "flatbpiperay.h"
#include "flatbpipe.h"
#include "bitmask.h"
#include "main.h"
#include "common.h"

double calc_rayscalar( Ray *ray, DVector3 *inmirror, DVector3* normal ) { 
  double rayscalar = 0;
  inmirror->x = ray->pos->x - src.x0;
  inmirror->y = ray->pos->y - src.y0;
  inmirror->z = ray->pos->z - src.z0;
  rayscalar = - ( dot_product( inmirror, normal ) / 
		  dot_product( ray->dir, normal ) );
  return rayscalar;
}

void find_intersection( Ray *ray, double rayscalar, DVector3 *inmirror ) {

  inmirror->x = ray->pos->x + rayscalar * ray->dir->x;
  inmirror->y = ray->pos->y + rayscalar * ray->dir->y;
  inmirror->z = ray->pos->z + rayscalar * ray->dir->z;
  return;
}

int check_optic_hit( Ray *ray, DVector3 *inter_transf, DVector3 *normal) {
  
  int surf_missed;
  double halfwidth= 0.5 * src.width;
  double halfheight= 0.5 * src.height;

  /* check if intersection of photon with plane of flat 
     is within confines of flat */
  if (( src.x0 - halfwidth )  <= inter_transf->x   &&
      inter_transf->x < ( src.x0 + halfwidth )     &&
      ( src.y0 - halfheight ) <= inter_transf->y   &&
      inter_transf->y < ( src.y0 + halfheight ))   
    {                                             /* ray hit optic */
      surf_missed = 1;
      *(ray->missed)= 0;        
    } else {                                      /* ray missed optic */
      surf_missed = 0; 
      BITSET( *(ray->missed), src.srfno);         /* set bit for output */  
    }

  return surf_missed;
}
/*   commented out for now because ray is not reflected

     void reflect_ray( DVector3* odir ) {
     odir->z = - odir->z;
     } 
     */

