/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <stdlib.h> 
#include <math.h>

#include <mathconst/mathconst.h>
#include <tparam/tparam.h>
#include <tracefct/tracefct.h>

#include "main.h"
#include "common.h"

/*---------------------------------------------------------------*/

static void gparam_circ_peri( paramfile pf );
static size_t init( void );
static int genphot( double *x, double *y );

OutputGeometry
circ_peri_geo =
{
  "circ-peri",
    gparam_circ_peri,
    init,
    genphot
};

/*---------------------------------------------------------------*/

static double radius;

/*---------------------------------------------------------------*/

static void
gparam_circ_peri( paramfile pf )
{
  tf_enter( "gparam_circ_peri" );

  radius = tpgetd( pf, "radius" );

  tf_leave();
}

static size_t
init( void )
{

  c.xmin = 0.0;
  c.xmax = M_2PI;
  return ( size_t ) ( c.phot_dens * M_2PI * radius );
}

static int
genphot( double *x, double *y )
{
  int stop = 0;
  double theta;

  while ( ! ( stop = c.distribution->pos1d( &theta ) )
	  && ( theta < 0.0 || theta > M_2PI ) );

  if ( ! stop )
  {
    *x = radius * cos( theta );
    *y = radius * sin( theta );
  }

  return stop;
}
