/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <mathconst/mathconst.h>
#include <tparam/tparam.h>
#include <tracefct/tracefct.h>

#include "main.h"
#include "common.h"

/*---------------------------------------------------------------*/

static void gparam_ellipse_fill(paramfile pf);
static size_t init(void);
static int genphot(double *x, double *y);	

/*---------------------------------------------------------------*/

OutputGeometry
ellipse_fill_geo =
{
  "ellipse-fill",
    gparam_ellipse_fill,
    init,
    genphot
};

/*---------------------------------------------------------------*/

static double a, a2;
static double b, b2;
static double rat2;

/*---------------------------------------------------------------*/

static void
gparam_ellipse_fill(paramfile pf)
{
  tf_enter("gparam_ellipse_fill");

  a = tpgetd(pf, "maj") / 2.0;
  b = tpgetd(pf, "min") / 2.0;

  tf_leave();
}

static size_t
init(void)
{
  a2 = a * a;
  b2 = b * b;
  rat2 = a2 / b2;

  c.xmax = a;
  c.xmin = -a;

  c.ymax = b;
  c.ymin = -b;

  return (size_t) (c.phot_dens * M_PI * a * b);
}

static int
genphot(double *x, double *y)
{			
  int stop = 0;
  /* get x and y values which fall inside ellipse */
  do
  {
    stop = c.distribution->pos2d(x, y);
  }
  while (!stop && (*x * *x + rat2 * *y * *y) > a2) ;

  return stop;
}
