/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

/* gen_energy.c: generate random energy from spectrum */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <tparam/tparam.h>
#include <randlib_c/randlib.h>
#include <mst_rdb/mst_rdb.h>
#include "flux.h"
#include "main.h"

static int 
key_compare( const void *e1, const void *e2 )
{
  double v = *( (double*) e1);
  double vlow = ( ( Flux * ) e2 )->precumint;
  double vhigh = ( ( Flux * ) e2 )->cumint;

  return ( v < vlow ) ? -1 : ( v >= vhigh ) ? 1 : 0;
}

double g_energy( Flux * data, size_t nlines, size_t n, EnergyDist en_dist )
{
  double energy;
  
  switch ( en_dist )
  {
  case EnDist_sample:
    {
      double key_elem = ranf();
      Flux *jfound;

      /* generate a uniform deviate from which a corresponding deviate
	 from the normalized cumulative distribution of the input
	 spectrum is found. jfound is the returned pointer to the
	 energy bin within whose bounds an energy will be generated */

      jfound = ( Flux * ) bsearch( 
	( const void * ) &key_elem, ( const void * ) data,
        nlines, sizeof( Flux ), key_compare );

      if ( NULL == jfound )
      {
	tf_enter( "g_energy" );
	tf_exit( ExitERR_misc, "internal error: prob out of range: %d\n",
		 key_elem );
      }

      /* generate random energy within bounds of selected bin */
      energy = jfound->emin + ranf() * ( jfound->emax - jfound->emin );
    }
    break;
    
  case EnDist_picket:
    {
      double key_elem = ranf();
      Flux *jfound;

      /* generate a uniform deviate from which a corresponding deviate
	 from the normalized cumulative distribution of the input
	 spectrum is found. jfound is the returned pointer to the
	 energy bin within whose bounds an energy will be generated */

      jfound = ( Flux * ) bsearch( 
	( const void * ) &key_elem, ( const void * ) data,
        nlines, sizeof( Flux ), key_compare );

      if ( NULL == jfound )
      {
	tf_enter( "g_energy" );
	tf_exit( ExitERR_misc, "internal error: prob out of range: %d\n",
		 key_elem );
      }

      /* use minimum energy */
      energy = jfound->emin;
    }
    break;

  case EnDist_ave:
    {
      if ( n >= nlines )
	tf_assert( 0 );

      energy = ( data[n].emin + data[n].emax ) / 2.0;
    }
    break;

  case EnDist_min:
    {
      if ( n >= nlines )
	tf_assert( 0 );

      energy = data[n].emin;
    }
    break;
  case EnDist_max:
    {
      if ( n >= nlines )
	tf_assert( 0 );

      energy = data[n].emax;
    }
    break;
  default:
    tf_assert( 0 );
  }

  return energy;
}
