/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <mathconst/mathconst.h>
#include <tracefct/tracefct.h>
#include <tparam/tparam.h>
#include <randlib_c/randlib.h>

#include "main.h"
#include "common.h"

/*---------------------------------------------------------------*/

static void gparam_grid( paramfile pf );
static size_t init( size_t nphot );
static int pos2d( double *x, double *y );
static int pos1d( double *x );

/*---------------------------------------------------------------*/

OutputDistribution
grid_dist =
{
  "grid",
    gparam_grid,
    init,
    pos1d,
    pos2d
};

int nx, ny, ix, iy;
double dx, dy;

/*---------------------------------------------------------------*/

static void
gparam_grid( paramfile pf )
{
  tf_enter( "gparam_grid" );

  nx = tpgeti( pf, "nx" );
  ny = tpgeti( pf, "ny" );

  tf_leave();
}

static size_t
init( size_t nphot )
{
  ix = 0;
  iy = 0;
  dx = ( c.xmax - c.xmin ) / (nx + 1);
  dy = ( c.ymax - c.ymin ) / (ny + 1);
  return nx * ny;
}

static int
pos2d( double *x, double *y )
{
  if ( iy == ny )
    return 1;

  *x = ix * dx + c.xmin + dx;
  *y = iy * dy + c.ymin + dy;
  if ( ++ix == nx ) {
    iy++;
    ix = 0;
  }

  return 0;
}

static int
pos1d( double *x )
{
  if ( ix == nx )
    return 1;

  *x = ix++ * dx + c.xmin;

  return 0;
}
