/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

/* gspectrum.c: get energy spectrum from rdb table and calculate
   cumulative distribution */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <tracefct/tracefct.h>
#include <tparam/tparam.h>
#include <randlib_c/randlib.h>
#include <mst_rdb/mst_rdb.h>

#include "flux.h"
#include "main.h" 

/* calculate cumulative distribution of spectrum */
static void 
calc_cumul_dist( Flux * data, long *nlines )
{

  int index;
  double cumulcounts = 0.0;
  double totalcounts = 0.0;

  tf_enter( "calc_cumul_dist" );

  /* calculate total number of counts */
  for ( index = 0; index < *nlines; index++ )
    totalcounts += data[ index ].counts;

  /* set lower limit for 1st bin (precumint = previous cumulative integral */
  data[ 0 ].cumint = data[ 0 ].counts / totalcounts;
  data[ 0 ].precumint = 0.0;
  
  /* calculate cumulative normalized integral of flux */
  cumulcounts = data[ 0 ].counts;
  for ( index = 1; index < *nlines; index++ ) {
    cumulcounts += data[ index ].counts;
    data[ index ].cumint = cumulcounts / totalcounts;
    data[ index ].precumint = data[ index - 1 ].cumint;
  }
  
  tf_leave();
  return;

}

/* get spectrum from rdb table */
Flux *
spectrum( char *spec_file, long *nlines, EnergyDist energy_dist )
{

  Flux *data;
  
  tf_enter( "spectrum" );
  
  data = readrdb( spec_file, nlines, energy_dist );

  checkrdb( data, nlines, energy_dist );

  if ( EnDist_sample == energy_dist || EnDist_picket == energy_dist )
    calc_cumul_dist( data, nlines );
  
  tf_leave();
  return data;

}
