#ifndef MAIN_H
#define MAIN_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>

#include <bpipe/datatypes.h>
#include <tparam/tparam.h>

#include "flux.h"

#define NAME_LENGTH 255

typedef struct
{
  char *name;
  void    (*gparam)(paramfile pf);
  size_t  (*init)(void);
  int     (*genphot)(double *x, double *y);   
} OutputGeometry;

typedef struct
{
  char  *name;
  void	 (*gparam)(paramfile pf);
  size_t (*init)(size_t nphot);
  int    (*pos1d)(double *x);
  int	 (*pos2d)(double *x, double *y); 
} OutputDistribution;

typedef struct
{
  char input[NAME_LENGTH + 1];
  char output[NAME_LENGTH + 1];
  char spectrum[NAME_LENGTH + 1];

  long seed1;
  long seed2;
  long block;
  double phot_dens;

  double x, y, z, phi;
  double xmin, xmax, ymin, ymax;

  /* tilt the bunch of rays (is) */

  double ray_phi, ray_theta; 

  double energy;
  
  int version;
  int help;

  EnergyDist en_dist;
  OutputGeometry *geometry;
  OutputDistribution *distribution;
} Common;

void gparam(int argc, char *argv[]);
int main(int argc, char *argv[]);

void setup_polarization(DCVector2 *pol_cos, DCVector2 *pol_sin, double *weight);
void process(void);

Flux *spectrum(char *spec_file, long *nlines, EnergyDist energy_dist);
int checkrdb(Flux *data, long *nlines, EnergyDist ed);
int qsort_cmp(const void *vp, const void *vq);

Flux *readrdb(const char *spectrum_file, long *nlines, EnergyDist energy_dist);
double g_energy(Flux *data, size_t nlines, size_t n, EnergyDist en_dist);

int usage(void);

#endif /* ! MAIN_H */

