/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

/* rdbcheck.c -  check limits in rdb table */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <tracefct/tracefct.h>
#include <tparam/tparam.h>
#include <randlib_c/randlib.h>
#include <mst_rdb/mst_rdb.h>

#include "flux.h"

static int 
checknegflux( Flux * data, long *nlines, EnergyDist ed )
{
  /* if we don't care about the flux, don't check it! */
  if ( EnDist_sample == ed || EnDist_picket == ed )
  {
    int index;
    for ( index = 0; index < *nlines; index++ ) {
      if ( data[index].counts < 0.0 ) {
	tf_enter( "checknegflux" );
	tf_message( "Flux is negative for bin\n");
	tf_leave();
	return EXIT_FAILURE;
      }
    }
  }

  return EXIT_SUCCESS;
}

static int 
checkminmax( Flux * data, long *nlines, EnergyDist ed )
{

  /* only do this if we need to */
  if ( EnDist_sample == ed || EnDist_ave == ed )
  {
    int index;

    for ( index = 0; index < *nlines; index++ ) {
      if ( data[index].emin >= data[index].emax ) {
	tf_enter( "checkminmax" );
	tf_message( "Minimum energy is NOT <  maximum energy for bin\n" );
	tf_leave();
	return EXIT_FAILURE;
      }
    }
  }
  return EXIT_SUCCESS;
}

static int 
checkbinoverlap( Flux * data, long *nlines, EnergyDist ed )
{

  /* only do this if we need to */
  if ( EnDist_sample == ed || EnDist_ave == ed )
  {
    int index;
    for ( index = 0; index < *nlines - 1; index++ ) {
      if ( data[index].emax > data[index + 1].emin ) {
	tf_enter( "checkbinoverlap" );
	tf_message( "Overlapping energy levels for bin\n");
	tf_leave();
	return EXIT_FAILURE;
      }
    }
  }
  return EXIT_SUCCESS;
}


int 
checkrdb( Flux * data, long *nlines, EnergyDist ed )
{

  return
    checkminmax( data, nlines, ed )     ||
    checknegflux( data, nlines, ed )    ||
    checkbinoverlap( data, nlines, ed ) ?
    EXIT_FAILURE : EXIT_SUCCESS;
}
