/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of genphot
 *
 * genphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * genphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <mathconst/mathconst.h>
#include <tparam/tparam.h>
#include <tracefct/tracefct.h>

#include "main.h"
#include "common.h"

/*---------------------------------------------------------------*/

static void gparam_rect_peri( paramfile pf );
static size_t init( void );
static int genphot( double *x, double *y );

OutputGeometry
rect_peri_geo =
{
  "rect-peri",
    gparam_rect_peri,
    init,
    genphot
};

/*---------------------------------------------------------------*/

static double height, height_2;
static double width, width_2;
static double perimeter, p1, p2, p3;

/*---------------------------------------------------------------*/

static void
gparam_rect_peri( paramfile pf )
{
  tf_enter( "gparam_rect_peri" );

  height = tpgetd( pf, "height" );
  width = tpgetd( pf, "width" );

  tf_leave();
}

static size_t
init( void )
{
  p1 = height;
  p2 = p1 + width;
  p3 = p2 + height;

  width_2 = width / 2.0;
  height_2 = height / 2.0;

  perimeter = 2.0 * ( height + width );
  c.xmax = perimeter;
  c.xmin = 0.0;

  return ( size_t ) ( c.phot_dens * perimeter );
}

static int
genphot( double *x, double *y )
{
  int stop = 0;
  double peri;

  while ( ! ( stop = c.distribution->pos1d( &peri ) )
	  && ( peri < 0.0 || peri > perimeter ) );

  if ( stop )
    return stop;

  if ( peri < p1 ) {
    *x = width_2;
    *y = -height_2 + peri;
  } else if ( peri < p2 ) {
    *y = height_2;
    *x = -width_2 + ( peri - p1 );
  } else if ( peri < p3 ) {
    *x = -width_2;
    *y = -height_2 + ( peri - p2 );
  } else {
    *y = -height_2;
    *x = -width_2 + ( peri - p3 );
  }

  return 0;
}
