/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of linklist
 *
 * linklist is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * linklist is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <assert.h>
#include <stdio.h>

#include <linklist.h>

#define LEN(list) (sizeof(list) / sizeof(int))

int cmp( const void *i1, const void *i2)
{
  return *((int *) i1) - *((int *) i2);
}

int
print_node( void *data, void *udata )
{
  fprintf( stderr, "%s: %d\n", (char *) udata, *((int *) data) ); 
  return 0;
}

int
main( int argc, char *argv[])
{
  int list[] = {1, 3, 5, 7, 9, 0, 2, 4, 6, 8};
  size_t i;

  DLinkList ll1;
  DLinkList ll2;
  DLLNode node;

  ll1 = dll_new( cmp );
  ll2 = dll_new( DLL_NULL_CMP );

  for ( i = 0 ; i < LEN(list) / 2 ; i++ )
    dll_insert( ll1, &list[i] );

  for ( ; i < LEN(list) ; i++ )
    dll_insert_tail( ll2, &list[i] );

  dll_join( ll1, ll2 );

  assert( dll_count( ll1 ) == LEN(list) );

  assert( dll_count( ll2 ) == 0 );

  node = dll_head_node( ll1 );
  for ( i = 0 ; i < LEN(list) ; i++ )
  {
    assert( *((int *) dll_node_get_data( node )) == i );
    node = dll_next_node( ll1, node );
  }

  dll_delete( ll1, DLL_NULL_DELETE );
  dll_delete( ll2, DLL_NULL_DELETE );

  ll1 = dll_new( DLL_NULL_CMP );
  ll2 = dll_new( DLL_NULL_CMP );

  for ( i = 0 ; i < LEN(list) / 2 ; i++ )
    dll_insert_tail( ll1, &list[i] );

  for ( ; i < LEN(list) ; i++ )
    dll_insert_tail( ll2, &list[i] );

  dll_join( ll1, ll2 );

  assert( dll_count( ll1 ) == LEN(list) );

  assert( dll_count( ll2 ) == 0 );

  node = dll_head_node( ll1 );
  for ( i = 0 ; i < LEN(list) ; i++ )
  {
    assert( *((int *) dll_node_get_data( node )) == list[i] );
    node = dll_next_node( ll1, node );
  }

  dll_delete( ll1, DLL_NULL_DELETE );
  dll_delete( ll2, DLL_NULL_DELETE );

  return 0;
}

