#ifndef LUASUP_H
#define LUASUP_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of luasup
 *
 * luasup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * luasup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdarg.h>

#include <lua.h>

#ifdef __cplusplus
extern "C" {
#endif

/* tags for lua types */
typedef enum
{
  SLua_STRING   = 0x000,
  SLua_NUMBER   = 0x001,
  SLua_TABLE    = 0x002,
  SLua_NIL	= 0x003,
  SLua_FUNCTION	= 0x004,
  SLua_CFUNCTION= 0x005,
  SLua_USERDATA = 0x006,
  SLua_TYPES    = 0x0ff,
  SLua_OPTIONAL = 0x100,
  SLua_DELENTRY = 0x200,
  SLua_COPY	= 0x400,
  SLua_BOGUS    = 0xfff
} SLuaArgType;

/* errors */

typedef enum
{
  SLua_ENOERR	= 0,
  SLua_ENOENT	= 1,
  SLua_ENOMEM	= 2,
  SLua_ENOTYPE	= 3,
  SLua_ENOEL	= 4,
  SLua_ENOMATCH	= 5
} SLuaError;
  

typedef void(*SLua_ErrFunc)( const char *fmt, ... );

int	slua_args( SLua_ErrFunc errfunc, const char *func, ... );
int	slua_args_n( SLua_ErrFunc errfunc, const char *func, int n, ... );
int	slua_args_va( SLua_ErrFunc errfunc, const char *func, int n, va_list ap );
char *	slua_str_arg_p(SLua_ErrFunc errfunc, const char *func, const char *arg);
int	slua_check_n_arg(SLua_ErrFunc errfunc, const char *func, int n );

int	slua_tbl_nelem( lua_Object table );
lua_Object	slua_tbl_next( lua_Object table, lua_Object *index );

SLuaError slua_tbl_el(SLua_ErrFunc errfunc, lua_Object table,
		const char *err_pfx, const char *index,
		SLuaArgType type, void *dest);

int	slua_tbl_el_s( lua_Object table, const char *index, char **value );
char *	slua_tbl_el_si(lua_Object table, const char *index);
int	slua_tbl_el_n( lua_Object table, const char *index, double *value );
lua_Object	slua_tbl_el_tbl( lua_Object table, const char *index );
void	slua_tbl_el_delete( lua_Object table, const char *index );
int	slua_tbl_el_exists(lua_Object table, const char *index);
lua_Object	slua_tbl_copy(lua_Object table );

  int slua_tbl_cmp( lua_Object table1, lua_Object table2 );

SLuaArgType slua_type( lua_Object object );
const char *slua_type_name( SLuaArgType type );

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* ! LUASUP_H */
