/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of luasup
 *
 * luasup is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * luasup is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <lua.h>

#include "luasup.h"

/*
  PROCEDURE
  slua_next - an implementation of the Lua @code{next} function

  DESCRIPTION
  This routine is a wrapper around the Lua @code{next} function, for
  which there are no default bindings for C.  It is passed a table and
  an index.  The index should be set to @code{LUA_NOOBJECT} to start
  the scan in the table.  @code{slua_next} will update the index to
  point to the next element in the table, and will return the value
  in the table at that index.  Note that the @emph{address} of the
  index is passed to @code{slua_next}.

  If there are no more elements in the table, the returned value and
  the index are set to @code{LUA_NOOBJECT}.

  RETURNS
  It returns the value of the table at the next index, or
  @code{LUA_NOOBJECT} if there are no more elements.

  AUTHOR
  Diab Jerius

*/

lua_Object
slua_next(
  lua_Object table, 		/* the table to iterate over */
  lua_Object *index		/* A pointer to the last index used; set the
				   index's value to LUA_NOOBJECT on
				   the first pass */
  )
{
  lua_Object lua_next_func = lua_getglobal( "next" );

  lua_pushobject( table );
  if ( LUA_NOOBJECT == *index )
    lua_pushnil();
  else
    lua_pushobject( *index );
  lua_callfunction( lua_next_func );

  *index = lua_getparam( 1 );
  return lua_getparam( 2 );
}
