package MakeWorld::Interpolate;

use strict;
use warnings;

use POSIX qw[];
use MakeWorld::Global;

use base qw[ Exporter ];

our @EXPORT = qw[
		    interpolate
	       ];

our @EXPORT_OK = qw[
		       interpolate_vars
		       InterpolateVars
		  ];


# options that can be interpolated
my @InterpolateOk = qw{ builddir cache exec_prefix prefix make_cmd};


# various other variables that may be interpolated
our %InterpolateVars;

INIT {
    @InterpolateVars{qw(uname_sysname
			uname_nodename
			uname_release
			uname_version
			uname_machine)}
      = POSIX::uname();
}


sub interpolate
{
  my ( $text, @vars ) = @_;


  return _interpolate( $text, interpolate_vars( @vars ) );
}

sub interpolate_vars {

    my ( @vars ) = @_;

    my %var = ( ( map { $_ => $opt{$_} } @InterpolateOk ),
		( map { %{$_} } @vars ), %InterpolateVars
	      );

    return \%var;
}


sub _interpolate {

    my ( $text, $var ) = @_;

    if ( 'HASH' eq ref $text )
    {

	while( my ( $key, $val ) = each %$text )
	{

	    $text->{$key} = _interpolate( $val, $var )
	}
    }

    elsif( 'ARRAY' eq ref $text )
    {
	$text = [ map { _interpolate( $_, $var ) } @{$text} ];
    }

    else
    {
	$text = _interpolate_scalar( $text, $var );
    }


    return $text;
}



sub _interpolate_scalar
{
  my ( $text, $var ) = @_;

  $text =~ s{
       \$                         # find a literal dollar sign
      (?:{(\w+)}|(\w+))           # find a "word" and store it in $1 or $2
  }{
      my $t = defined $2 ? $2 : $1;
      defined $var->{$t} ? $var->{$t} : exists $ENV{$t} ? $ENV{$t} : '';
  }egx;

  $text =~ s{
       \$\(             # find a literal dollar sign and a literal open paren
      (?:([^\)]+))
       \)		# grab everything up until a closing paren
  }{
      my @r = qx( $1 );
      chomp @r;
      join(' ', @r );
  }egx;

  return $text;
}


1;
