#!@PERL@

# --8<--8<--8<--8<--
#
# Copyright (C) 2010 Smithsonian Astrophysical Observatory
#
# This file is part of make-world
#
# make-world is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# make-world is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

package MakeWorld::Package::AutoConf;

use strict;
use warnings;

use Carp;
use File::Spec::Functions qw/ catdir /;

use base 'MakeWorld::Package';

use MakeWorld::Global;
use MakeWorld::Run;

sub identify
{
    my ( $package, $self ) = @_;

    return bless $self, $package
      if -f 'configure';

    return undef;
}

sub configure
{
    my $self = shift;

    my $rd = $self->logredir(  'configure' );

    $self->SUPER::configure && return;

    my $config_args = $self->{option}{configure}{arg};

    # just in case
    local %ENV = %ENV;

    # check for configure file. if its a dryrun, it's probable that the
    # source hasn't been unarchived and we won't see the configure file.
    # just assume its there. can't hurt!
    if ( -f 'configure' || $opt{dryrun} )
    {
	my @args = "--prefix=$opt{prefix}";

	my $use_cache = 
	  defined $self->{option}{use_cache} ? $self->{option}{use_cache} 
	    : defined $opt{cache};

	push @args, "--exec_prefix=$opt{exec_prefix}";

	push @args, "--cache=$opt{cache}" if $use_cache;
	push @args,  $config_args         if $config_args;

	run( './configure', @args )
	  or die( "error configuring package\n" );
    }
}

sub build
{
    my $self = shift;

    my $rd = $self->logredir(  'build' );

    $self->SUPER::build && return;

    my @cmd = ( $opt{make_cmd} );

    push @cmd, "LIBTOOL=" . $self->{option}{libtool}
      if exists $self->{option}{libtool};

    push @cmd, ( $self->{option}{build}{arg} || () );

    run( @cmd  )
	or die( "error building package\n" );
}

sub test
{
    my $self = shift;

    my $rd = $self->logredir(  'test' );

    $self->SUPER::test && return;

    my @cmd = ( $opt{make_cmd}, 'check' );

    push @cmd, "LIBTOOL=" . $self->{option}{libtool}
      if exists $self->{option}{libtool};

    push @cmd, ( $self->{option}{test}{arg} || () );

    run( @cmd )
      or die( "error testing package\n" );
}


sub install
{
    my $self = shift;

    my $rd = $self->logredir(  'install' );

    $self->SUPER::install && return;

    my @cmd = ( $opt{make_cmd}, 'install' );

    push @cmd, "LIBTOOL=" . $self->{option}{libtool}
      if exists $self->{option}{libtool};

    push @cmd, ( $self->{option}{install}{arg} || () );

    run( @cmd )
      or die( "error installing package\n" );
}


1;
