
#include <float.h>
#include <stdio.h>

#ifndef MATTYPE
#define MATTYPE double
#endif

#define PI 3.14159265358979323846
#define DEG2RAD(angle) ( (angle) * PI / 180.0 )

#define COS30 (sqrt(3.0)/2.0)
#define SIN30 0.5

#define TVECPRINT(vector) TVECPRINT_COMP(vector)
#define TVECPRINT_COMP(v0,v1,v2) \
  do { printf( "% f % f % f\n", v0, v1, v2 ); } while( 0 )

#define T3X3PRINT(matrix) T3X3PRINT_COMP(matrix)
#define T3X3PRINT_COMP(						\
  a00, a01, a02,						\
  a10, a11, a12,						\
  a20, a21, a22							\
  )								\
  do{								\
  printf( "% f % f % f\n% f % f % f\n% f % f % f\n",	\
	  a00, a01, a02,					\
	  a10, a11, a12,					\
	  a20, a21, a22						\
	  );							\
  } while( 0 )

#define T4X4PRINT(matrix) T4X4PRINT_COMP(matrix)

#define T4X4PRINT_COMP(						\
  a00, a01, a02, a03,						\
  a10, a11, a12, a13,						\
  a20, a21, a22, a23,						\
  a30, a31, a32, a33						\
  )								\
  do{								\
  printf( "%f %f %f %f\n%f %f %f %f\n%f %f %f %f\n%f %f %f %f\n",	\
	  a00, a01, a02, a03,					\
	  a10, a11, a12, a13,					\
	  a20, a21, a22, a23,					\
	  a30, a31, a32, a33					\
	  );							\
  } while( 0 )

#define T3X3EQ(					\
  a00, a01, a02,				\
  a10, a11, a12,				\
  a20, a21, a22,				\
  matb						\
  )						\
  T3X3EQ_COMP(					\
    a00, a01, a02,				\
    a10, a11, a12,				\
    a20, a21, a22,				\
    matb )

#define T3X3EQ_COMP(	\
  a00, a01, a02,	\
  a10, a11, a12,	\
  a20, a21, a22,	\
  b00, b01, b02,	\
  b10, b11, b12,	\
  b20, b21, b22		\
)			\
  (    (a00) == (b00) && (a01) == (b01) && (a02) == (b02) \
    && (a10) == (b10) && (a11) == (b11) && (a12) == (b12) \
    && (a20) == (b20) && (a21) == (b21) && (a22) == (b22) )


#define T4X4EQ(					\
  a00, a01, a02, a03,				\
  a10, a11, a12, a13,				\
  a20, a21, a22, a23,				\
  a30, a31, a32, a33,				\
  matb						\
  )						\
  T4X4EQ_COMP(					\
    a00, a01, a02, a03,				\
    a10, a11, a12, a13,				\
    a20, a21, a22, a23,				\
    a30, a31, a32, a33,				\
    matb )

#define T4X4EQ_COMP(	\
  a00, a01, a02, a03,	\
  a10, a11, a12, a13,	\
  a20, a21, a22, a23,	\
  a30, a31, a32, a33,	\
  b00, b01, b02, b03,	\
  b10, b11, b12, b13,	\
  b20, b21, b22, b23,	\
  b30, b31, b32, b33	\
)			\
  (    (a00) == (b00) && (a01) == (b01) && (a02) == (b02) && (a03) == (b03) \
    && (a10) == (b10) && (a11) == (b11) && (a12) == (b12) && (a13) == (b13) \
    && (a20) == (b20) && (a21) == (b21) && (a22) == (b22) && (a23) == (b23) \
    && (a30) == (b30) && (a31) == (b31) && (a32) == (b32) && (a33) == (b33) \
)


#define TVECEQ(		\
  a0, a1, a2,		\
  vecb			\
  )			\
  TVECEQ_COMP( a0, a1, a2, vecb)


#define TVECEQ_COMP(	\
  a0, a1, a2,		\
  b0, b1, b2		\
  )			\
  ( (a0) == (b0) && (a1) == (b1) && (a2) == (b2) ) 


#define def_MAT3X3_AS_ARRAY			\
  const MATTYPE mat[9] = { 00, 01, 02,		\
			   10, 11, 12,		\
			   20, 21, 22  }

#define def_MAT3X3_AS_MDARRAY				\
  const MATTYPE mat[3][3] = { { 00, 01, 02 },		\
			      { 10, 11, 12 },		\
			      { 20, 21, 22 }		\
  }


#define def_MAT4X4_AS_ARRAY			\
  const MATTYPE mat[16] = { 00, 01, 02, 03,	\
			    10, 11, 12, 13,	\
			    20, 21, 22, 23,	\
			    30, 31, 32, 33 }


#define def_MAT4X4_AS_MDARRAY				\
  const MATTYPE mat[4][4] = { { 00, 01, 02, 03 },	\
			      { 10, 11, 12, 13 },	\
			      { 20, 21, 22, 23 },	\
			      { 30, 31, 32, 33 },	\
  }


#define T3X3AR_EQ(mata, matb) mat_array_eq( 9, mata, matb)
#define T4X4AR_EQ(mata, matb) mat_array_eq( 16, mata, matb)


int mat_array_eq( int nelem, const MATTYPE* mata, const MATTYPE* matb )
{
  size_t i    = 0;
  size_t n_eq = 0;
  
  for ( i = 0 ; i < nelem ; i++ )
    n_eq += mata[i] == matb[i];

  return n_eq == nelem;
}

#define TVECAR_FPEQ(veca, vecb) mat_array_fpeq( 3, veca, vecb)
#define T3X3AR_FPEQ(mata, matb) mat_array_fpeq( 9, mata, matb)
#define T4X4AR_FPEQ(mata, matb) mat_array_fpeq( 16, mata, matb)

int mat_array_fpeq( int nelem, const MATTYPE* mata, const MATTYPE* matb )
{
  size_t i    = 0;
  size_t n_eq = 0;
  
  for ( i = 0 ; i < nelem ; i++ )
  {
    n_eq += fabs(mata[i] - matb[i]) <= (10 * DBL_EPSILON);
  }

  return n_eq == nelem;
}
