/* Test of vector operations */

#include <stdlib.h>
#include <math.h>

#include <testlib/testlib.h>

#include "matvec/macros.h"

#include "macros.h"

#define VEC(vector) MATVEC_VEC_AS_ARRAY(vector,1)

void test_MATVEC_VEC_ASSIGN( void )
{
  const MATTYPE src[3] = { 2, 1, 9 };
  MATTYPE dst[3];

  MATVEC_VEC_ASSIGN( VEC(dst), VEC(src) );
  TEST( "MATVEC_VEC_ASSIGN", 1,
	TVECEQ(2, 1, 9, VEC(dst) ));
}

void test_MATVEC_VEC_SCALE( void )
{
  const MATTYPE src[3] = { 2, 1, 9 };
  MATTYPE dst[3];

  MATVEC_VEC_SCALE( VEC(dst), VEC(src), 8 );
  TEST( "MATVEC_VEC_SCALE", 1,
	TVECEQ(16, 8, 72, VEC(dst) ));
}

void test_MATVEC_VEC_ADD( void )
{
  MATTYPE veca[3] = { 0.2, 1.3, 9.8 };
  MATTYPE vecb[3] = { 0.5, 0.5, 0.1 };
  MATTYPE vexp[3] = { 0.7, 1.8, 9.9 };
  MATTYPE sum[3];

  MATVEC_VEC_ADD( VEC(sum), VEC(veca), VEC(vecb) );
  TEST( "MATVEC_VEC_ADD", 1, TVECAR_FPEQ(vexp, sum ));
}

void test_MATVEC_VEC_SUB( void )
{
  MATTYPE veca[3] = {  0.2, 1.3, 9.8 };
  MATTYPE vecb[3] = {  0.5, 0.5, 0.1 };
  MATTYPE vexp[3] = { -0.3, 0.8, 9.7 };
  MATTYPE diff[3];

  MATVEC_VEC_SUB( VEC(diff), VEC(veca), VEC(vecb) );
  TEST( "MATVEC_VEC_SUB", 1, TVECAR_FPEQ(vexp, diff ));
}

void test_MATVEC_VEC_PROJECT( void )
{
  MATTYPE pos[3] = { 0, 0, 0 };
  const MATTYPE dir[3] = { 0.5, 0.5, 0.1 };
  MATTYPE res[3] = { 1, 1, 0.2 };

  MATVEC_VEC_PROJECT(VEC(pos), VEC(pos), VEC(dir), 2);
  TEST( "MATVEC_VEC_PROJECT", 1,
	mat_array_eq( 3, pos, res )
	);
}

void test_MATVEC_VEC_DOTPROD( void )
{
  const MATTYPE veca[3] = { 1, 2, 3 };
  const MATTYPE vecb[3] = { 8, 22, 44 };

  TEST( "MATVEC_VEC_DOTPROD", 184,
	MATVEC_VEC_DOTPROD( VEC(veca), VEC(vecb) )
	);
}

void test_MATVEC_VEC_CROSSPROD( void )
{
  const MATTYPE veca[3] = {  2,  5, 7 };
  const MATTYPE vecb[3] = {  3,  4, 5 };
  MATTYPE  res[3];

  MATVEC_VEC_CROSSPROD( VEC(res), VEC(veca), VEC(vecb) );

  TEST( "MATVEC_VEC_CROSSPROD", 1,
	TVECEQ( -3, 11, -7, VEC(res) )
	);
}

void test_MATVEC_VEC_NORM( void )
{
  MATTYPE veca[3] = { 8, 22, 44 };
  MATTYPE vecb[3];
  int i;

  double veclen = sqrt( veca[0]*veca[0] + veca[1]*veca[1] + veca[2]*veca[2] );

  for ( i = 0 ; i < 3 ; i++ )
    vecb[i] = veca[i] / veclen;
  
  MATVEC_VEC_NORM( VEC(veca) );

  TEST( "MATVEC_VEC_NORM", 1,
	mat_array_fpeq( 3, veca, vecb )
	);
  

}


int main( void )
{
  START( "Vector Operations" );

  test_MATVEC_VEC_ASSIGN();
  test_MATVEC_VEC_SCALE();
  test_MATVEC_VEC_ADD();
  test_MATVEC_VEC_SUB();
  test_MATVEC_VEC_PROJECT();
  test_MATVEC_VEC_DOTPROD();
  test_MATVEC_VEC_CROSSPROD();
  test_MATVEC_VEC_NORM();

  SUMMARY();

  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
