#ifndef BPIPEINPUT_H
#define BPIPEINPUT_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <string>
#include <vector>
#include <cstdlib>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>
#include <linklist/linklist.h>

using namespace std;

//
//: A Bpipe Input class to open a list of input bpipe.
//  The class is responsible for tranferring all header fields
//  to the output stream unless they are requested to be transferred to
//  the data field.
//
class BpipeInput {

  friend ostream& operator << ( ostream& os, BpipeInput& a ) {
    a.print( os );
    return os;
  }

  friend ostream& operator << ( ostream& os, BpipeInput* a ) {
    os << *a;
    return os;
  }

public:

  virtual ~BpipeInput( );

  BpipeInput( ) : num( 0 ), bpipe( NULL ), n_pkts_valid( 0 ), n_pkts( NULL )
    { }

  BpipeInput( string& fnames, string& xfer, int& status, ostream& os );

  size_t get_n_dpkts( size_t i ) {
    if ( NULL == n_pkts )
      return 0;
    if ( i >= num )
      return 0;
    return n_pkts[ i ];
  }

  size_t get_num( ) { return num; }
  // return the number of input streams.

  BPipe* get_bpipe( size_t i, ostream& os ) {
    if ( i < 0 || i >= num ) {
      os << "BpipeInput::get_bpipe( " << i << ", ostream& )" << endl;
      return NULL;
    } else
      return bpipe[ i ];
  }
  // return the ith input stream.

  int have_valid_npkts( void ) { 
    return (num == n_pkts_valid ? EXIT_SUCCESS : EXIT_FAILURE); }

  void print( ostream& os ) { os << ""; }

  int resize_core( SLinkList data_fields, string& xfer, ostream& os );

  size_t read_dpkts( size_t i, ostream& os ) {
    if ( i < 0 || i >= num ) {
      os << "BpipeInput::read_dpkts( " << i << ", ostream& )" << endl;
      return 0;
    } else
      return bpipe_read_dpkts( bpipe[ i ], data[ i ], 1 ); 
  }
  //: Read from the ith input stream.

protected:

  size_t       num;
  //: The number of input bpipe.

  BPipe**      bpipe;
  //: The input bpipe stream.

  void**       data;
  //: The input data core.

  size_t       n_pkts_valid;
  //: The number of input streams containing the the n_pkts field.

  size_t*      n_pkts;
  //: n_pkts[ 0 ] .... npkts[ num - 1 ] holds
  //: the number of data packet in each stream.

private:

  int allocate_bpipe( vector<string>& filenames, ostream& os );
  int get_num_dpkts( ostream& os );
  int header_to_datafield( string& xfer, ostream& os );

  //: declare but (purposedly) do not define
  BpipeInput( const BpipeInput& tmp );
  BpipeInput& operator = ( const BpipeInput& rhs );

};

#endif
