#ifndef BPIPEOUTPUT_H
#define BPIPEOUTPUT_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

#include <linklist/linklist.h>

#include <paramxx/ParFile.h>

#include "BpipeInput.h"
#include "DataField.h"

using namespace std;

//
//: The base class for the Bpipe Output.  The class is responsible for
//: determining the common output data field.
//  The class loops over the input streams to determine the data fields
//  which have the same name, data type and the greatest common denominator
//  matrix size for all the data fields.
//
class BpipeOutput {

  friend ostream& operator << ( ostream& os, BpipeOutput& a ) {
    a.print( os );
    return os;
  }

  friend ostream& operator << ( ostream& os, BpipeOutput* a ) {
    os << *a;
    return os;
  }

public:

  virtual ~BpipeOutput( );

  BpipeOutput( ) : bpipe( NULL ), bpo( NULL ), data( NULL ), input( NULL ) { }

  BpipeOutput( BpipeInput* in, ParFile& parfile, string& xfer, int& status,
	       ostream& os );

  virtual int write_dpkts( ostream& os );

  void print( ostream& os ) { os << ""; }
  
protected:
  
  BPipe*       bpipe;
  BPipeOutput* bpo;
  void*        data;

  DataField** output_data_fields;
  //: The output data fields.

  string filename;
  //: The output filename

  BpipeInput* input;
  //: An association to the input streams.

  virtual size_t copy_data( size_t i, ostream& os );
  //: loop through the output_data_fields array to copy
  //: the data for the datafields from the ith stream.

private:

  int add_header_fields( BPipe* input, string& xfer, SLinkList data_fields, 
			 ostream& os );
  //: Get the header from the input bpipe, make sure that the fields
  //: specified by the xfer parameter are not written to the output header.

  int get_data_field_ptr( SLinkList data_fields, ostream& os );

  int init_bpipe_output( SLinkList data_fields, ostream& os );

  int select_data_fields( BPipe* bpipe_input, string& xfer,
			  SLinkList data_fields, ostream& os );

  int select_data_fields( char* n, BPDataType t, BPMatrix* matrix,
			  SLinkList data_fields, ostream& os );

  int set_output_data_fields( SLinkList data_fields, ostream& os );
  //: Transfer the output data fields from a linklist to an array of
  //: DataField to speed things up.
};

#endif
