#ifndef DATAPACKET_H
#define DATAPACKET_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <bpipe.h>
#include <bpipe_axaf.h>
#include <string>

class DataPacket {

  friend ostream& operator << ( ostream& os, DataPacket& a ) {
    a.print( os );
    return os;
  }

  friend ostream& operator << ( ostream& os, DataPacket* a ) {
    os << *a;
    return os;
  }

public:

  ~DataPacket( ) { 
    if ( input_data )
      delete [] input_data;
    matrix = NULL;
  }

  DataPacket( ) : count( 1 ), type( _BPDType_NOTYPE ), matrix( NULL ), 
    input_data( NULL ), output_data( NULL ), header_data( NULL ) { }

  DataPacket( char* n, BPDataType t, BPMatrix* mat, int nbpipe, int& status, 
	      ostream& os ) : count( 1 ), type( t ), name( n ), 
    matrix( mat ), input_data( NULL ), output_data( NULL ),
    header_data( NULL ) {
      
    status = init_DataPacket( n, t, nbpipe, os );

  }

  int copy_data( int i ) {
    bpipe_data_copy( output_data, input_data[ i ], matrix, type );
    return EXIT_SUCCESS;
  }

  size_t     get_count( ) { return count; }

  BPMatrix*  get_matrix( ) { return matrix; }

  char*      get_name( ) { return (char*) name; }

  void*      get_input_data( int i ) { return input_data[ i ]; }

  BPDataType get_type( ) { return type; }

  void       set_input_data( int i, void* ptr ) { input_data[ i ] = ptr; }

  void       set_output_data( void* ptr ) { output_data = ptr; }

  int init_DataPacket( char* nom, BPDataType t, int nbpipe, ostream& os ) {

    name = nom;
    type = t;

    input_data = new void*[ nbpipe ];
    if ( NULL == input_data ) {
      os << "DataPacket::init_DataPacket( " << nom << ", " << t;
      os << "BPMatrix*, " << nbpipe << ", ostream& ) : Unable to ";
      os << "allocate inpu_data" << endl;
      return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;

  }

  void print( ostream& os ) {
    os << "name = " << (char*) name;
    os << "\ncount = " << count;
    os << "\ntype = " << type;
    os << "\nne = " << matrix->ne;
    os << "\nnd = " << matrix->nd << endl;
    for ( int i = 0; i < matrix->nd; i++ ) {
      os << "extent[" << i << "] = " << matrix->extent[ i ];
      if ( i == matrix->nd - 1 )
	os << endl;
      else
	os << '\t';
    }
  }

  int update( DataPacket* datapacket, ostream& os ) { 

    if ( 0 != ::strcmp( name, datapacket->get_name( ) ) &&
	 type != datapacket->get_type( ) )
	 return EXIT_SUCCESS;

    count++;

    BPMatrix* tmp_matrix = matrix;

    BPMatrix* tmp_datapacket = datapacket->get_matrix( );

    matrix = bpipe_matrix_min( tmp_matrix, tmp_datapacket );
    if ( NULL == matrix ) {
      os << "DataPacket::update( DataPacket*, ostream& ) : ";
      os << "Unable to allocate matrix matrix" << endl;
      return EXIT_FAILURE;
    }

    // bpipe_matrix_delete( tmp_matrix );

    return EXIT_SUCCESS;

  }

protected:

  size_t     count;     
  //: The number of streams containing data packet with the 
  //: following properties:

  BPDataType type;
  //: The datatype for the data packet.

  string       name;
  //: The name of the data packet

  BPMatrix*  matrix;

  void**     input_data;
  //: The pointer to the input data

  void*      output_data;
  //: The pointer to the output data

  void*      header_data;
  //: The pointer to the header data

};

#endif
