// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <randlib_c/randlib.h>

#include "SampleBpipeOutput.h"

SampleBpipeOutput::SampleBpipeOutput( BpipeInput* in, ParFile& parfile,
				      string& xfer, int& status, ostream& os ) :
  BpipeOutput( in, parfile, xfer, status, os ) {

    if ( EXIT_FAILURE == status )
      return;

    status = EXIT_FAILURE;

    // All the input streams must have the flag
    // BPHN_n_dpkts_valid set to a non-zero value.
    if ( EXIT_FAILURE == input->have_valid_npkts( ) ) {
      os << "Not all the input streams have non-zero value for the `";
      os << BPHN_n_dpkts_valid << "' flag" << endl;
      return;
    }

    if ( EXIT_FAILURE == seed_block( parfile, os ) )
      return;

    status = EXIT_SUCCESS;

    return;

}

int SampleBpipeOutput::write_dpkts( ostream& os ) {

  size_t num_input_streams = input->get_num( );
  size_t num = 0;
  for ( size_t i = 0; i < num_input_streams; i++ )
    num += input->get_n_dpkts( i );

  double* distribution = new double[ num_input_streams ];

  distribution[ 0 ] = input->get_n_dpkts( 0 ) / double( num );

  for ( size_t j = 1; j < num_input_streams; j++ )
    distribution[ j ] = distribution[ j - 1 ] + 
      input->get_n_dpkts( j ) / double( num );

  size_t k = 0;

  while ( k < num ) {

    double random = ranf( );
    size_t stream_num = 0;

    for ( stream_num = 0; stream_num < num_input_streams; stream_num++ )
      if ( random < distribution[ stream_num ] )
	break;

    if ( 0 == copy_data( stream_num, os ) )
      continue;

    size_t kk = 0;
    DataField* datafield = NULL;
    while( ( datafield = output_data_fields[ kk++ ] ) )
      datafield->copy_data( stream_num );
    
    bpipe_write_dpkt( bpipe, data, bpo );

    k++;

  }

  delete [] distribution;
  distribution = NULL;

  return EXIT_SUCCESS;

}

int SampleBpipeOutput::seed_block( ParFile& pf, ostream& os ) {

  try {

    long seed1;
    pf.pget( "seed1", seed1 );

    long seed2;
    pf.pget( "seed2", seed2 );

    long block;
    pf.pget( "block", block );

    /* initialize random number generator and get to correct block */
    setall( seed1, seed2 );
    for ( long n = 0 ; n < block ; n++ )
      initgn(1);

    return EXIT_SUCCESS;

  } catch( ParFileException& pfe ) {

    os << pfe << endl;
    return EXIT_FAILURE;

  }

}
