// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "TimeBpipeOutput.h"

TimeBpipeOutput::TimeBpipeOutput( BpipeInput* in, ParFile& parfile, string& xfer,
				  int& status, ostream& os ) :
  BpipeOutput( in, parfile, xfer, status, os ), time_data( NULL ), num( 0 ),
  time_datafield( NULL ) {

    if ( EXIT_FAILURE == status )
      return;

    status = is_time_present( os );
    if ( EXIT_FAILURE == status )
      return;

    num = input->get_num( );

    time_data = new void* [ num ];
  
    if ( NULL == time_data ) {
      os << "TimeBpipeOutput::init_time_data( " << num << ", ostream& ) : ";
      os << "Unable to allocate memory for time_data" << endl;
      status = EXIT_FAILURE;
      return;
    }

    for ( size_t i = 0; i < num; i++ )
      time_data[ i ] = NULL;

}

size_t TimeBpipeOutput::copy_data( size_t i, ostream& os ) {

  size_t n = input->read_dpkts( i, os );
  if ( 0 == n )
    return 0;

  if ( 0 == n ) {
    time_data[ i ] = NULL;
    return 0;
  }

  time_data[ i ] = data;
  return n;

}

//
//  The method returns EXIT_SUCCESS or EXIT_FAILURE if the field is
//  present in all of the input streams or if the time field is missing
//  in one of the input streams, respectively.
//
int TimeBpipeOutput::is_time_present( ostream& os ) {

  size_t j = 0;
  DataField* datafield = NULL;
  while( ( datafield = output_data_fields[ j ] ) ) {
    if ( 0 == ::strcmp( BP_AXAF_time, datafield->get_name( ) ) ) {
      time_datafield = output_data_fields[ j ];
      return EXIT_SUCCESS;
    }
    j++;
  }
  
  os << "TimeBpipeOutput::is_time_present( ostream& ) : ";
  os << "The field `" << BP_AXAF_time << "' is not present in all of the ";
  os << "input stream" << endl;

  return EXIT_FAILURE;

}

int TimeBpipeOutput::write_dpkts( ostream& os ) {

  size_t num_empty_stream = 0;
  size_t* empty_stream = new size_t[ num ];

  {
    for ( size_t i = 0; i < num; i++ )
      if ( 0 == copy_data( i, os ) ) {
	empty_stream[ i ] = 1;
	++num_empty_stream;
      } else
	empty_stream[ i ] = 0;
  }

  while( num_empty_stream < num ) {

    size_t stream_num = time_datafield->min_time( empty_stream );

    // os << PRINTVAL( stream_num, endl );
    // os << time_datafield << endl;

    if ( time_data[ stream_num ] ) {

      size_t j = 0;
      DataField* datafield = NULL;
      while( ( datafield = output_data_fields[ j++ ] ) )
	datafield->copy_data( stream_num );

      bpipe_write_dpkt( bpipe, time_data[ stream_num ], bpo );

    }

    if ( 0 == copy_data( stream_num, os ) ) {
      empty_stream[ stream_num ] = 1;
      ++num_empty_stream;
    }

  }

  delete [] empty_stream;
  empty_stream = NULL;
  num_empty_stream = 0;

  return EXIT_SUCCESS;

}
