// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of mergerays
//
// mergerays is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// mergerays is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>

#include <paramxx/ParFile.h>

#include "BpipeInput.h"
#include "TimeBpipeOutput.h"
#include "SampleBpipeOutput.h"

#include "config.h"

using namespace std;

int mergeray( ParFile& parfile, ostream& os ) {

  int status = EXIT_FAILURE;

  try {

    string xfer;
    parfile.pget( "xfer", xfer );
  
    /*
    if ( string::OK != xfer.parse( "," ) ) {
      cerr << "mergeray( ParFile& ) : Unable to parse xfer" << endl;
      return EXIT_FAILURE;
    }
    */

    string filenames;
    parfile.pget( "input", filenames );

    BpipeInput input( filenames, xfer, status, os );
    if ( EXIT_FAILURE == status )
      return EXIT_FAILURE;
    // os << input << endl;

    string merge_method;
    parfile.pget( "method", merge_method );

    BpipeOutput* output = NULL;

    if ( "cat" == merge_method )
      output = new BpipeOutput( &input, parfile, xfer, status, os );
    else if ( "time" == merge_method )
      output = new TimeBpipeOutput( &input, parfile, xfer, status, os );
    else if ( "sample" == merge_method )
      output = new SampleBpipeOutput( &input, parfile, xfer, status, os );

    if ( EXIT_FAILURE == status ) {
      delete output;
      os << "mergeray( ParFile&, ostream& ) : EXIT_FAILURE == status" << endl;
      return EXIT_FAILURE;
    }

    output->write_dpkts( os );

    delete output;

  } catch ( ParFileException& pfe ) {

    os << pfe << endl;

    return EXIT_FAILURE;

  }

  return EXIT_SUCCESS;

}

int get_help_version( char*, ParFile&, bool&, bool& );
int usage();

int main( int argc, char** argv ) {

  try {

    int status = EXIT_FAILURE;
    ParFile parfile( argc, argv );

    bool version = true, help = true;
    if ( EXIT_FAILURE ==
	 get_help_version( argv[ 0 ], parfile, help, version ) )
      return EXIT_FAILURE;

    //
    // If the user wants to know how to run the program or which version
    // number it is then print requested info then exit.
    //
    if ( help || version )
      return EXIT_SUCCESS;

    status = mergeray( parfile, cerr );

    return status;

  } catch ( ParFileException& pfe ) {

    cerr << pfe << endl;
    return EXIT_FAILURE;

  }

}

int get_help_version( char* progname, ParFile& parfile, bool& help, 
		      bool& version ) {

  try {

    // determine if the user wants to see a help menu or not
    parfile.pget( "help", help );

    // print the usage documentation then exit
    if ( help ) {
      usage();
      return EXIT_SUCCESS;
    }

    // determine if the user wants to see a version number or not
    parfile.pget( "version", version );

    // print the version number then exit
    if ( version ) {
      cout << progname << "\t" << VERSION << endl;
      return EXIT_SUCCESS;
    }

    return EXIT_SUCCESS;

  } catch ( ParFileException& pfe ) {

    cerr << pfe << endl;

    return EXIT_FAILURE;

  }

}

