#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

/* simple print error and exit routine */
#define error(string)                                                   \
  do {                                                                  \
      fprintf( stderr, __FILE__ " %d: %s\n", __LINE__, string );        \
      exit( EXIT_FAILURE );                                             \
     } while (0)

/*
  invoke as

  create

*/



int 
main (int argc, char *argv[])
{
  BPipe *bpipe;
  BPipeOutput *bpo;

  void *data;
  unsigned int *id;
  double *time = NULL;

  size_t i;
  size_t nphots = 10;

  size_t id_start = 1;
  double time_start = 1;
  

  if ( argc < 3 )
    error( "usage: create <id start> <time start>" );

  id_start = atoi(argv[1]);
  time_start = atof( argv[2] );


  if ( NULL == ( bpipe = bpipe_new( ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* create new data packet fields */

  if ( bpipe_dpktf_add( bpipe,  BP_AXAF_phot_id,     BPDType_uint,   NULL ) )
    error( bpipe_strerror( bpipe_errno ) );

  if ( time_start >= 0 )
  {
    if ( bpipe_dpktf_add( bpipe,  BP_AXAF_time,     BPDType_double,   NULL ) )
      error( bpipe_strerror( bpipe_errno ) );
  }

  if ( NULL == ( bpo = bpipe_output( bpipe, "stdout" ) ) )
    error( bpipe_strerror( bpipe_errno ) );
    
  /* map data packet fields */
  if ( NULL == ( data = bpipe_map_alloc( bpipe, 1, NULL ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* output header */
  if ( bpipe_write_hdr(bpipe) )
    error( bpipe_strerror( bpipe_errno ) );

  /* get handles to data packet fields */
  id  = bpipe_dpktf_datap( bpipe, data, BP_AXAF_phot_id, BPDType_uint );
  *id = id_start;

  if ( time_start >= 0)
  {
    time  = bpipe_dpktf_datap( bpipe, data, BP_AXAF_time, BPDType_double );
    *time = time_start;
  }


  for (i = 1; i <= nphots ; i++)
  {
    
    if ( bpipe_write_dpkt( bpipe, data, bpo ) )
      error( "error writing photon" );

    (*id)++;
    if ( time ) (*time)++;
  }

  bpipe_delete(bpipe);

  return EXIT_SUCCESS;
}

