#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>

#include "mst_nrutil.h"

/*
 allocate a float vector with subscript range v[nl..nh]

 returns NULL if out of memory.
*/
float *
vector_r(long nl, long nh)
{
  float *v;

  v = malloc((size_t) ((nh - nl + 1 + NR_END) * sizeof(float)));

  return v ? v - nl + NR_END : NULL;

}

/*
 allocate an int vector with subscript range v[nl..nh]

 returns NULL if out of memory.
*/
int *
ivector_r(long nl, long nh)
{
  int *v;

  v = malloc((size_t) ((nh - nl + 1 + NR_END) * sizeof(int)));

  return v ? v - nl + NR_END : NULL;
}

/*
 allocate an unsigned char vector with subscript range v[nl..nh]

 returns NULL if out of memory.
*/
unsigned char *
ucvector_r(long nl, long nh)
{
  unsigned char *v;

  v = malloc((size_t) ((nh - nl + 1 + NR_END) * sizeof(unsigned char)));

  return v ? v - nl + NR_END : NULL;
}

/*
 allocate an char vector with subscript range v[nl..nh]

 returns NULL if out of memory.
*/
char *
cvector_r(long nl, long nh)
{
  char *v;

  v = malloc((size_t) ((nh - nl + 1 + NR_END) * sizeof(char)));

  return v ? v - nl + NR_END : NULL;
}

/*
 allocate an long vector with subscript range v[nl..nh]

 returns NULL if out of memory.
*/
long *
lvector_r(long nl, long nh)
{
  long *v;

  v = malloc((size_t) ((nh - nl + 1 + NR_END) * sizeof(long)));

  return v ? v - nl + NR_END : NULL;
}

/*
 allocate an unsigned long vector with subscript range v[nl..nh]

 returns NULL if out of memory.
*/
unsigned long *
ulvector_r(long nl, long nh)
{
  unsigned long *v;

  v = malloc((size_t) ((nh - nl + 1 + NR_END) * sizeof(long)));

  return v ? v - nl + NR_END : NULL;
}

/*
 allocate a double vector with subscript range v[nl..nh]

 returns NULL if out of memory.
*/
double *
dvector_r(long nl, long nh)
{
  double *v;

  v = malloc((size_t) ((nh - nl + 1 + NR_END) * sizeof(double)));

  return v ? v - nl + NR_END : NULL;
}

/*
 allocate a float matrix with subscript range m[nrl..nrh][ncl..nch]

 returns NULL if out of memory.
*/
float **
matrix_r(long nrl, long nrh, long ncl, long nch)
{
  long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  float **m, **m_orig;

/* allocate pointers to rows */
  m_orig = m = malloc((size_t) ((nrow + NR_END) * sizeof(float *)));

  if (!m)
    return NULL;

  m += NR_END;
  m -= nrl;

/* allocate rows and set pointers to them */
  m[nrl] = malloc((size_t) ((nrow * ncol + NR_END) * sizeof(float)));

  if (!m[nrl])
  {
    free(m_orig);
    return NULL;
  }

  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i = nrl + 1; i <= nrh; i++)
    m[i] = m[i - 1] + ncol;

/* return pointer to array of pointers to rows */
  return m;
}

/*
 allocate a double matrix with subscript range m[nrl..nrh][ncl..nch]

 returns NULL if out of memory.
*/
double **
dmatrix_r(long nrl, long nrh, long ncl, long nch)
{
  long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  double **m, **m_orig;

/* allocate pointers to rows */
  m_orig = m = malloc((size_t) ((nrow + NR_END) * sizeof(double *)));

  if (!m)
    return NULL;

  m += NR_END;
  m -= nrl;

/* allocate rows and set pointers to them */
  m[nrl] = malloc((size_t) ((nrow * ncol + NR_END) * sizeof(double)));

  if (!m[nrl])
  {
    free(m_orig);
    return NULL;
  }

  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i = nrl + 1; i <= nrh; i++)
    m[i] = m[i - 1] + ncol;

/* return pointer to array of pointers to rows */
  return m;
}

/*
 allocate a int matrix with subscript range m[nrl..nrh][ncl..nch]

 returns NULL if out of memory.
*/
int **
imatrix_r(long nrl, long nrh, long ncl, long nch)
{
  long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  int **m, **m_orig;

/* allocate pointers to rows */
  m_orig = m = malloc((size_t) ((nrow + NR_END) * sizeof(int *)));

  if (!m)
    return NULL;

  m += NR_END;
  m -= nrl;


/* allocate rows and set pointers to them */
  m[nrl] = malloc((size_t) ((nrow * ncol + NR_END) * sizeof(int)));

  if (!m[nrl])
  {
    free(m_orig);
    return NULL;
  }

  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i = nrl + 1; i <= nrh; i++)
    m[i] = m[i - 1] + ncol;

/* return pointer to array of pointers to rows */
  return m;
}

/*
 allocate a long matrix with subscript range m[nrl..nrh][ncl..nch]

 returns NULL if out of memory.
*/
long **
lmatrix_r(long nrl, long nrh, long ncl, long nch)
{
  long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  long **m, **m_orig;

/* allocate pointers to rows */
  m_orig = m = malloc((size_t) ((nrow + NR_END) * sizeof(long *)));

  if (!m)
    return NULL;

  m += NR_END;
  m -= nrl;


/* allocate rows and set pointers to them */
  m[nrl] = malloc((size_t) ((nrow * ncol + NR_END) * sizeof(long)));

  if (!m[nrl])
  {
    free(m_orig);
    return NULL;
  }

  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i = nrl + 1; i <= nrh; i++)
    m[i] = m[i - 1] + ncol;

/* return pointer to array of pointers to rows */
  return m;
}

/*
 point a submatrix [newrl..][newcl..] to a[oldrl..oldrh][oldcl..oldch]

 returns NULL if out of memory.
*/
float **
submatrix_r(float **a, long oldrl, long oldrh, long oldcl, long oldch,
	    long newrl, long newcl)
{
  long i, j, nrow = oldrh - oldrl + 1, ncol = oldcl - newcl;
  float **m;

/* allocate array of pointers to rows */
  m = malloc((size_t) ((nrow + NR_END) * sizeof(float *)));

  if (!m)
    return NULL;

  m += NR_END;
  m -= newrl;

/* set pointers to rows */
  for (i = oldrl, j = newrl; i <= oldrh; i++, j++)
    m[j] = a[i] + ncol;

/* return pointer to array of pointers to rows */
  return m;
}

/*

 allocate a float matrix m[nrl..nrh][ncl..nch] that points to the
 matrix declared in the standard C manner as a[nrow][ncol], where
 nrow=nrh-nrl+1 and ncol=nch-ncl+1. The routine should be called
 with the address &a[0][0] as the first argument.

 returns NULL if out of memory.
*/
float **
convert_matrix_r(float *a, long nrl, long nrh, long ncl, long nch)
{
  long i, j, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  float **m;

/* allocate pointers to rows */
  m = malloc((size_t) ((nrow + NR_END) * sizeof(float *)));

  if (!m)
    return NULL;

  m += NR_END;
  m -= nrl;

/* set pointers to rows */
  m[nrl] = a - ncl;
  for (i = 1, j = nrl + 1; i < nrow; i++, j++)
    m[j] = m[j - 1] + ncol;
/* return pointer to array of pointers to rows */
  return m;
}


/*
 allocate an int 3tensor with range t[nrl..nrh][ncl..nch][ndl..ndh]

 returns NULL if out of memory.
*/
int ***
i3tensor_r(long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
{
  long i, j, nrow = nrh - nrl + 1, ncol = nch - ncl + 1, ndep = ndh - ndl + 1;
  int ***t, ***t_orig, **t2_orig;

/* allocate pointers to pointers to rows */
  t_orig = t = malloc((size_t) ((nrow + NR_END) * sizeof(int **)));

  if (!t)
    return NULL;

  t += NR_END;
  t -= nrl;

/* allocate pointers to rows and set pointers to them */
  t2_orig = t[nrl] =
    malloc((size_t) ((nrow * ncol + NR_END) * sizeof(int *)));

  if (!t[nrl])
  {
    free(t_orig);
    return NULL;
  }

  t[nrl] += NR_END;
  t[nrl] -= ncl;

/* allocate rows and set pointers to them */
  t[nrl][ncl] = malloc((size_t) ((nrow * ncol * ndep + NR_END) * sizeof(int)));

  if (!t[nrl][ncl])
  {
    free(t_orig);
    free(t2_orig);
    return NULL;
  }

  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;

  for (j = ncl + 1; j <= nch; j++)
    t[nrl][j] = t[nrl][j - 1] + ndep;
  for (i = nrl + 1; i <= nrh; i++)
  {
    t[i] = t[i - 1] + ncol;
    t[i][ncl] = t[i - 1][ncl] + ncol * ndep;
    for (j = ncl + 1; j <= nch; j++)
      t[i][j] = t[i][j - 1] + ndep;
  }

/* return pointer to array of pointers to rows */
  return t;
}

/*
 allocate a float 3tensor with range t[nrl..nrh][ncl..nch][ndl..ndh]

 returns NULL if out of memory.
*/
float ***
f3tensor_r(long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
{
  long i, j, nrow = nrh - nrl + 1, ncol = nch - ncl + 1, ndep = ndh - ndl + 1;
  float ***t, ***t_orig, **t2_orig;

/* allocate pointers to pointers to rows */
  t_orig = t = malloc((size_t) ((nrow + NR_END) * sizeof(float **)));

  if (!t)
    return NULL;

  t += NR_END;
  t -= nrl;

/* allocate pointers to rows and set pointers to them */
  t2_orig = t[nrl] =
    malloc((size_t) ((nrow * ncol + NR_END) * sizeof(float *)));

  if (!t[nrl])
  {
    free(t_orig);
    return NULL;
  }

  t[nrl] += NR_END;
  t[nrl] -= ncl;

/* allocate rows and set pointers to them */
  t[nrl][ncl] = malloc((size_t) ((nrow * ncol * ndep + NR_END) * sizeof(float)));

  if (!t[nrl][ncl])
  {
    free(t_orig);
    free(t2_orig);
    return NULL;
  }

  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;

  for (j = ncl + 1; j <= nch; j++)
    t[nrl][j] = t[nrl][j - 1] + ndep;
  for (i = nrl + 1; i <= nrh; i++)
  {
    t[i] = t[i - 1] + ncol;
    t[i][ncl] = t[i - 1][ncl] + ncol * ndep;
    for (j = ncl + 1; j <= nch; j++)
      t[i][j] = t[i][j - 1] + ndep;
  }

/* return pointer to array of pointers to rows */
  return t;
}

double ***
d3tensor_r(long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
/* allocate a double 3tensor with range t[nrl..nrh][ncl..nch][ndl..ndh] */
{
  long i, j, nrow = nrh - nrl + 1, ncol = nch - ncl + 1, ndep = ndh - ndl + 1;
  double ***t, ***t_orig, **t2_orig;

/* allocate pointers to pointers to rows */
  t_orig = t = malloc((size_t) ((nrow + NR_END) * sizeof(double **)));

  if (!t)
    return NULL;

  t += NR_END;
  t -= nrl;

/* allocate pointers to rows and set pointers to them */
  t2_orig = t[nrl] =
    malloc((size_t) ((nrow * ncol + NR_END) * sizeof(double *)));

  if (!t[nrl])
  {
    free(t_orig);
    return NULL;
  }

  t[nrl] += NR_END;
  t[nrl] -= ncl;

/* allocate rows and set pointers to them */
  t[nrl][ncl] = malloc((size_t) ((nrow * ncol * ndep + NR_END) * sizeof(double)));

  if (!t[nrl][ncl])
  {
    free(t_orig);
    free(t2_orig);
    return NULL;
  }

  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;

  for (j = ncl + 1; j <= nch; j++)
    t[nrl][j] = t[nrl][j - 1] + ndep;
  for (i = nrl + 1; i <= nrh; i++)
  {
    t[i] = t[i - 1] + ncol;
    t[i][ncl] = t[i - 1][ncl] + ncol * ndep;
    for (j = ncl + 1; j <= nch; j++)
      t[i][j] = t[i][j - 1] + ndep;
  }

/* return pointer to array of pointers to rows */
  return t;
}

/* free a float vector allocated with vector() */
void
free_vector(float *v, long nl, long nh)
{
  free(v + nl - NR_END);
}

/* free an int vector allocated with ivector() */
void
free_ivector(int *v, long nl, long nh)
{
  free(v + nl - NR_END);
}

/* free an char vector allocated with cvector() */
void
free_cvector(char *v, long nl, long nh)
{
  free(v + nl - NR_END);
}

/* free an unsigned char vector allocated with cvector() */
void
free_ucvector(unsigned char *v, long nl, long nh)
{
  free(v + nl - NR_END);
}

/* free an long vector allocated with lvector() */
void
free_lvector(long *v, long nl, long nh)
{
  free(v + nl - NR_END);
}

/* free an unsigned long vector allocated with lvector() */
void
free_ulvector(unsigned long *v, long nl, long nh)
{
  free(v + nl - NR_END);
}

/* free a double vector allocated with dvector() */
void
free_dvector(double *v, long nl, long nh)
{
  free(v + nl - NR_END);
}

/* free a float matrix allocated by matrix() */
void
free_matrix(float **m, long nrl, long nrh, long ncl, long nch)
{
  free(m[nrl] + ncl - NR_END);
  free(m + nrl - NR_END);
}

/* free a double matrix allocated by dmatrix() */
void
free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch)
{
  free(m[nrl] + ncl - NR_END);
  free(m + nrl - NR_END);
}

/* free an int matrix allocated by imatrix() */
void
free_imatrix(int **m, long nrl, long nrh, long ncl, long nch)
{
  free(m[nrl] + ncl - NR_END);
  free(m + nrl - NR_END);
}

/* free an long matrix allocated by lmatrix() */
void
free_lmatrix(long **m, long nrl, long nrh, long ncl, long nch)
{
  free(m[nrl] + ncl - NR_END);
  free(m + nrl - NR_END);
}

/* free a submatrix allocated by submatrix() */
void
free_submatrix(float **b, long nrl, long nrh, long ncl, long nch)
{
  free(b + nrl - NR_END);
}

/* free a matrix allocated by convert_matrix() */
void
free_convert_matrix(float **b, long nrl, long nrh, long ncl, long nch)
{
  free(b + nrl - NR_END);
}

/* free a int i3tensor allocated by i3tensor() */
void
free_i3tensor(int ***t, long nrl, long nrh, long ncl, long nch,
	      long ndl, long ndh)
{
  free(t[nrl][ncl] + ndl - NR_END);
  free(t[nrl] + ncl - NR_END);
  free(t + nrl - NR_END);
}

/* free a float f3tensor allocated by f3tensor() */
void
free_f3tensor(float ***t, long nrl, long nrh, long ncl, long nch,
	      long ndl, long ndh)
{
  free(t[nrl][ncl] + ndl - NR_END);
  free(t[nrl] + ncl - NR_END);
  free(t + nrl - NR_END);
}

/* free a double d3tensor allocated by d3tensor() */
void
free_d3tensor(double ***t, long nrl, long nrh, long ncl, long nch,
	      long ndl, long ndh)
{
  free(t[nrl][ncl] + ndl - NR_END);
  free(t[nrl] + ncl - NR_END);
  free(t + nrl - NR_END);
}
