// main.cc

/****************************************************************************
 * Author     : Terry Gaetz
 */

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of multilayer_reflect
//
// multilayer_reflect is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// multilayer_reflect is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <libgen.h>

#include <Exception/Exception.h>
#include <paramxx/ParFile.h>
#include <suplib/debug.h>
#include <tracefctxx/TraceFct.h>

#include "main.h"
#include "usage.h"

using namespace std;

int
main( int argc, char *argv[] )
{
  TraceFct tf( basename( argv[0] ), 0, 0 );

  try {
    ParFile pf( argc, argv );

    if ( pf.pgetb( "help" ) )
    {
      usage( );
      return EXIT_SUCCESS;
    }
    
    if ( pf.pgetb( "version" ) )
    {
      cerr << basename( argv[0] ) << PACKAGE_VERSION << endl;
      return EXIT_SUCCESS;
    }

    // get debug flags
    debug_init( pf.pgetstring( "debug" ).c_str( ) );
    
    process( pf );

  } catch ( Exception& E ) {

    tf.exit( 1, E );

  } catch ( exception& e ) {

    tf.exit( 1, e.what( ) );

  } catch ( ... ) {

    tf.exit( 1, "unexpected exception caught" );

  }

  return EXIT_SUCCESS;
}

