/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of mxwldmn
 *
 * mxwldmn is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * mxwldmn is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */


#include <tparam/tparam.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <suplib/debug.h>

#include "main.h"
#include "common.h"

void
gparam(int argc, char *argv[])
{
  paramfile pf;
  char dummy[NAME_LENGTH+1];

  tf_enter("gparam");

  pf = tparamopen(NULL, argv, argc, "r");

  /* get debug flags */
  tpgetstr(pf, "debug", dummy, NAME_LENGTH, 0);
  debug_init(dummy);

  c.version = tpgetb(pf, "version");
  c.help = tpgetb(pf, "help" );

  if ( ! c.version && ! c.help)
  {
    /* input files */
    tpgetstr(pf, "input", c.input, NAME_LENGTH, 1);

    /* output file */
    tpgetstr(pf, "output", c.output, NAME_LENGTH, 1);

    c.seed1 = tpgeti(pf, "seed1");
    if (c.seed1 < 1 || c.seed1 > 2147483562)
      tf_exit(ExitERR_illegalArg, "seed1 must have range [1,2147483562]");

    c.seed2 = tpgeti(pf, "seed2");
    if (c.seed2 < 1 || c.seed2 > 2147483398)
      tf_exit(ExitERR_illegalArg, "seed2 must have range [1,2147483398]");

    c.block = tpgeti(pf, "block");
    if (c.block < 0 || c.block > 1048575)
      tf_exit(ExitERR_illegalArg, "block must have range [0,1048575]");
  }

  paramclose(pf);
  tf_leave();
}
