/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of mxwldmn
 *
 * mxwldmn is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * mxwldmn is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>

#include <randlib_c/randlib.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "common.h"

void
process(void)
{
  unsigned long n;
  BPipe *bpipe;
  BPipeOutput *bpo;
  void *data;
  double *weight;

  tf_enter( "process" );

  /* initialize random number generator and get to correct block */
  setall(c.seed1, c.seed2);
  for ( n = 0 ; n < c.block ; n++ )
    initgn(1);

  bpipe_errno = BPNOERROR;

  if ( NULL == ( bpipe = bpipe_new( ) ) )
    tf_exit( ExitERR_misc, "error creating bpipe: %s",
	     bpipe_strerror( bpipe_errno ) );

  if ( bpipe_input( bpipe, c.input ) )
    tf_exit( ExitERR_misc, "error opening %s for input: %s",
	     c.input, bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( bpo = bpipe_output( bpipe, c.output ) ) )
    tf_exit( ExitERR_misc, "error opening %s for output: %s",
	     c.output, bpipe_strerror( bpipe_errno ) );

  /* kill off the polarization, if it exists */
  if ( -1 == bpipe_dpktf_delete( bpipe, BP_AXAF_pol_sin, BPDSite_CORE, bpo ) )
    tf_exit( ExitERR_misc, "error deleting %s from stream: %s",
	     BP_AXAF_pol_sin, bpipe_strerror( bpipe_errno ) );
       
  if ( -1 == bpipe_dpktf_delete( bpipe, BP_AXAF_pol_cos, BPDSite_CORE, bpo ) )
    tf_exit( ExitERR_misc, "error deleting %s from stream: %s",
	     BP_AXAF_pol_cos, bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( data = ( bpipe_map_alloc( bpipe, 1, NULL ) ) ) )
    tf_exit( ExitERR_misc, "error mapping bpipe: %s",
	     bpipe_strerror( bpipe_errno ) );

  if ( bpipe_write_hdr(bpipe) )
    tf_exit( ExitERR_misc, "error writing header: %s",
	     bpipe_strerror( bpipe_errno ) );

  if ( NULL ==
       (weight = bpipe_dpktf_datap( bpipe, data, BP_AXAF_phot_wt,
				    BPDType_double )) )
    tf_exit( ExitERR_misc, "unable to get pointer to weight data: %s",
	     bpipe_strerror( bpipe_errno ) );

  for( n = 0 ; bpipe_read_dpkts( bpipe, data, 1 ) ; n++ )
  {
    if ( genunf( 0.0, 1.0 ) > *weight )
      continue;

    *weight = 1.0;
    bpipe_write_dpkt( bpipe, data, bpo );
  }

  if ( bpipe_errno != BPNOERROR )
    tf_exit( ExitERR_misc, "error whilst reading photon %d: %s", n,
	     bpipe_strerror( bpipe_errno ) );


  bpipe_delete( bpipe );

  tf_leave();
}
