// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <memory>
#include <cstring>

using namespace std;

#include <suplib/str.h>

#include "NameAttributeValue.h"

bool NameAttributeValue::check_boolean( char* str ) {

  // cout << "NameAttributeValue::check_boolean( " << str << " )\n";

  if ( 0 == str || 0 == *str )
    return false;

  size_t last_char_pos = strlen( str ) - 1;
  char   last_char = str[ last_char_pos ];

  if ( '+' == last_char ) {

    str[ last_char_pos ] = '\0';
    name = str;
    value = "1";
    return true;

  } else if ( '-' == last_char ) {

    str[ last_char_pos ] = '\0';
    name = str;
    value = "0";
    return true;

  } else if ( 0 == strncmp( str, "--", 2 ) ) {

    name = str + 2;
    value = "1";
    return true;

  }
   
  return false;

}

void NameAttributeValue::init_NameAttributeValue( const char* txt ) {

  // os << "NameAttributeValue::init_NameAttributeValue( " << str << " ) " << endl;

  name ="";
  attribute="";
  value="";

  if ( 0 == txt || 0 == *txt )
    return;

  auto_ptr< char > tmp( new char[ strlen( txt ) + 1 ] );
  char* str = tmp.get( );
  strcpy( str, txt );

  // Must call unquote before str_prune since str coud be ` " foo=bar " '
  (void) ::unquote( str );
  (void) str_prune( str );

  // str could be of the form: foo+, foo- or --foo
  if ( check_boolean( str ) )
    return;

  // Get the value.
  seperate_keyword_value( str );

  strcpy( str, name.c_str( ) );
  seperate_keyword_attribute( str );

  return;

}

void NameAttributeValue::print( ostream& os ) {

  os << name;

  if ( "" != attribute )
    os << "." << attribute;

  os << " = " << value;

}


//
// The current state of the txt_argv is:
//
//    txt_argv[ NameAttributeValue::name ] = name.attribute
//    txt_argv[ NameAttributeValue::val ] = value
//
// where .attribute may or may not be present
//
void NameAttributeValue::seperate_keyword_attribute( char* str ) {

  // There is no attribute in the original string. So return
  if ( NULL == strstr( str, "." ) )
    return;

  // the attribute starts immediately after the period: name.attribute = value
  char* period = strchr( str, '.' );
  // increment to get to the start of the attribute.
  period++;

  attribute = period;

  // decrememnt to back to where the period is
  period--;

  // set it to zero
  *period = '\0';
  name = str;

  return;

}

void NameAttributeValue::seperate_keyword_value( char* txt ) {

  //
  // Tokenize on '=' to seperate parameter name, attribute and value.
  // Do not tokenize on '.' yet since a '.' can appear multiple times.
  // That is, do not make the call as str_tokenize( txt, ".=" )
  // since it will fail on foo.min=12.3
  //
  // strstr is used instead of parse or tokenize since the following
  // example is a legitimate entry (the optimizer program):
  //    optimizer override='a0.fitme=1 a1.value=3'
  // The parse or tokenize command produces too many tokens to be
  // meaningful.
  //

  char* ptr = strstr( txt, "=" );
  if ( NULL == ptr ) {
    name = txt;
    return;
  }

  str_prune( ptr + 1 );
  value = ptr + 1;

  char tmp[ 256 ];
  strncpy( tmp, txt, ptr - txt );
  tmp[ ptr - txt ] = '\0';

  name = str_prune( tmp );

  return;

}
