// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <stdlib.h>

#include <suplibxx/str.h>

#include "RealPar.h"
#include "ParFile.h"

RealPar::RealPar( ParTxt& par ) throw ( ParFileException, Exception ) :
  Par( par ) {

    try {

      validate( parameter[ PARMINIMUM ] );
  
      validate( parameter[ PARMAXIMUM ] );
  
      validate( parameter[ PARVALUE ] );

      between_limits( parameter[ PARVALUE ] );

    } catch( ParFileException& pfe ) {

      throw;

    } catch( Exception& e ) {

      throw;

    }

}

void RealPar::between_limits( const string& str ) const
  throw ( ParFileException, Exception ) {

    try {

      // if the value field has an indirrection then do not check for limits
      if ( is_indirrect( str ) )
	return;

      char msg[ 256 ];

      // if a minimum value was entered by user then make sure that min <= val
      if ( "" != parameter[ PARMINIMUM ] ) {
	double min =
	  suplib::str2d( (const char*) parameter[ PARMINIMUM ].c_str( ) );
	double val = suplib::str2d( (const char*) str.c_str( ) );
	if ( val < min ) {
	  not_between_limits( msg, "min", parameter[ PARMINIMUM ], "val",
			      str );
	  throw ParFileException( msg );
	}
      }

      // if a minimum value was entered by user then make sure that val <= max
      if ( "" !=  parameter[ PARMAXIMUM ] ) {
	double max =
	  suplib::str2d( (const char*) parameter[ PARMAXIMUM ].c_str( ) );
	double val = suplib::str2d( (const char*) str.c_str( ) );
	if ( val > max ) {
	  not_between_limits( msg, "val", str, "max",
			      parameter[ PARMAXIMUM ] );
	  throw ParFileException( msg );
	}
      }

    } catch( ParFileException& pfe ) {
      throw;
    } catch( Exception& e ) {
      throw;
    }

}

void RealPar::set_val( const string& str )
  throw ( ParFileException, Exception ) {

    try {

      string tmp( str );

      validate( tmp );

      between_limits( tmp );

      parameter[ PARVALUE ] = str;

    } catch ( ParFileException& pfe ) {

      throw;

    } catch( Exception& e ) {

      throw;

    }

}

void RealPar::validate( const string& str ) const throw ( ParFileException ) {

  try {

    if ( "" == str )
      return;

    if ( Par::is_indirrect( str ) )
      return;

    const char* ptr = (const char*) str.c_str( );

    double result = suplib::str2d( ptr );

  } catch( Exception& e ) {

    string msg = e.get_message( );
    throw ParFileException( msg );

  }

}
