#ifndef STLPTRWRAPPER_H
#define STLPTRWRAPPER_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of paramxx
//
// paramxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// paramxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <algorithm>

using namespace std;

/**
 * A simple wrapper class to put pointers into STL containers.
 */
template <class Type>
class stlPtrWrapper {

  friend ostream& operator << ( ostream& os, stlPtrWrapper<Type>& a ) {
    os << a( );
    return os;
  }

  friend ostream& operator << ( ostream& os, stlPtrWrapper<Type>* a ) {
    os << (*a)( );
    return os;
  }

  friend bool operator == (const stlPtrWrapper<Type>& xw1,
			   const stlPtrWrapper<Type>& xw2) {
    return (xw1.operator()() && xw2.operator()()) ? xw1() == xw2() : false;
  }

  friend bool operator < (const stlPtrWrapper<Type>& xw1,
			  const stlPtrWrapper<Type>& xw2) {
    return (xw1() && xw2()) ? xw1() < xw2() : false;
  }

public:

  stlPtrWrapper( Type x = 0 ) : val( x ) { }

  stlPtrWrapper( const stlPtrWrapper<Type>& xxx ) : val( xxx.val ) {  }

  // purposely left blank
  ~stlPtrWrapper() { }

  stlPtrWrapper& operator = ( const stlPtrWrapper<Type>& rhs ) {

    if ( this == &rhs ) 
      return *this;

    val = rhs.val;
    return *this;

  }
  
  const Type operator( ) () const { return val; }

  Type operator( ) () { return val; }

private:

  Type val;

};

#endif
