#include <iostream>
#include <cstdlib>

#include <paramxx/ParFile.h>

using namespace std;

/*
  PROCEDURE
     getp - get the value of one or more parameters in  an  IRAF-
     style parameter file

  SYNOPSIS
     getp paramfile [ param1...  ]

  DESCRIPTION
     getp gets the values of parameters in the  specified  param-
     file  and  outputs  them on the standard output.  The values
     are separated by new-lines.  Thus,

          example% getp src region qpoe

     will return:

          /pros/xray_old/data/snr.qp
          c 512 512 50

     The getp program can be used, for  example,  in  conjunction
     with  the  unix back-quote facility to control the action of
     scripts.  For example:

          if (`getp scriptpars doit` == "yes") then
               echo "doing it ..."
               doit
          else
               echo "not doing it ..."
          endif

     In searching for the parameter file, .par extension will  be
     added  to  the paramfile name, if no extension is specified.
     If the file is not located in the directory specified by the
     paramfile, the directory pointed to by the UPARM environment
     variable will be searched.

  OPTIONS
     none

  ENVIRONMENT
    The environment variable PFILES specifies the list of directories,
    delimited by colon( :), in which to look for the parameter file.

    The environment variable UPARM specifies  the  directory  in
    which to look for the parameter file, if the pathname of the
    file is not part of the paramfile specification.
*/

static int parget( int argc, char** argv ) {

  try {

    ParFile pfile( argc, argv, argv[ 1 ] );

    for ( int i = 2; i < argc; i++ ) {
      Par* par = pfile.get_par( argv[ i ] );
      cout << par->get_value( ) << endl;
    }
  
    return EXIT_SUCCESS;

  } catch( ParFileException& pfe ) {

    cerr << pfe << endl;
    return EXIT_FAILURE;

  } catch ( Exception& e ) {

    cerr << e << endl;
    return EXIT_FAILURE;

  }

}

int main( int argc, char** argv ) {

  if ( argc < 3 ) {
    cerr << "Usage:\t" << argv[0] << " pfile param\n";
    return !EXIT_SUCCESS;
  }

  return parget( argc, argv );

}

