#include <iostream>
#include <vector>
#include <cstdlib>
#include <cstring>

using namespace std;

#include <paramxx/stlPtrWrapper.h>

#include "XXX.h"


int XXX::num_copy=0;
int XXX::num_assign=0;

/**
 * An ineffecient way of storing elements.
 * note the number of times the copy constructor gets called, num.
 */
void testXXX( int num ) {

  cout << "\n\ntestXXX( )\n";

  vector< XXX > v;
  for ( int i = 0; i < num; i++ ) {
    XXX x( i );
    v.push_back( x );
  }

  cout << "v.size( ) = " << v.size( ) << endl;
  v.clear( );
  cout << "v.size( ) = " << v.size( ) << endl;

  cout << "XXX::num_copy = " << XXX::num_copy << endl;
  cout << "XXX::num_assign = " << XXX::num_assign << endl;

}

void myDelete( stlPtrWrapper< XXX* > a ) {
  XXX* ptr = a();
  delete ptr;
}

/**
 * A more effecient approach.
 * Note the number of times the copy constructor gets called, 0.
 */
void testXXXptr( int num ) {

  cout << "\n\ntestXptr( )\n";

  XXX::num_copy = 0;
  XXX::num_assign = 0;

  vector< stlPtrWrapper< XXX* > > v;
  for ( int i = 0; i < num; i++ )
    v.push_back( stlPtrWrapper< XXX* >( new XXX( i ) ) );

  /**
   * Since the content of the vector is a pointer,
   * one must explicitly delete its contents.
   */
  for_each( v.begin( ), v.end( ), myDelete );

  cout << "v.size( ) = " << v.size( ) << endl;
  v.erase( v.begin( ), v.end( ) );
  cout << "v.size( ) = " << v.size( ) << endl;

  cout << "X::num_copy = " << XXX::num_copy << endl;
  cout << "X::num_assign = " << XXX::num_assign << endl;

}

int main( int argc, char** argv ) {

  int num = 4;

  // To show that it is inefficient to store as non-pointers
  testXXX( num );

  // A more efficient to use an stl data structure
  testXXXptr( num );

  return 0;

}
