#include <stdlib.h>
#include <testlib/testlib.h>

#include <paramxx/StringPar.h>

#define MYLENGTH 128
static char mystr[ MYLENGTH ];

static int testString( bool printIt ) {

  try {

    strcpy( mystr, "input,s,a,stdin,,,\"input photon stream\"" );
    ParTxt parTxt( mystr );
    StringPar stringPar( parTxt );

    if ( printIt )
      cout << stringPar << endl;

  } catch( ParFileException& pfe ) {

    cerr << pfe;
    return EXIT_FAILURE;
    exit( 1 );

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  return EXIT_SUCCESS;

}

static int not_bool_type( bool printIt ) {

  try {

    strcpy( mystr, "input,s,a,stdin,,,\"input photon stream\"" );
    ParTxt parTxt( mystr );
    StringPar stringPar( parTxt );

    bool foo = stringPar.pgetb( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "StringPar not_bool_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "StringPar not_bool_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_int_type( bool printIt ) {

  try {

    strcpy( mystr, "input,s,a,stdin,,,\"input photon stream\"" );
    ParTxt parTxt( mystr );
    StringPar stringPar( parTxt );

    int foo = stringPar.pgeti( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "StringPar not_int_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "StringPar not_int_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_long_type( bool printIt ) {

  try {

    strcpy( mystr, "input,s,a,stdin,,,\"input photon stream\"" );
    ParTxt parTxt( mystr );
    StringPar stringPar( parTxt );

    long foo = stringPar.pgetb( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "StringPar not_long_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "StringPar not_long_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_double_type( bool printIt ) {

  try {

    strcpy( mystr, "input,s,a,stdin,,,\"input photon stream\"" );
    ParTxt parTxt( mystr );
    StringPar stringPar( parTxt );

    double foo = stringPar.pgetd( );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "StringPar not_double_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "StringPar not_double_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_str_type( bool printIt ) {

  try {

    strcpy( mystr, "input,s,a,stdin,,,\"input photon stream\"" );
    ParTxt parTxt( mystr );
    StringPar stringPar( parTxt );

    stringPar.pgetstr( mystr, MYLENGTH );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "StringPar not_str_type( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "StringPar not_str_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

static int not_within_min( bool printIt ) {

  try {

    strcpy( mystr,
	    "input,s,a,stdin,stdin|stdout|stderr,,\"input photon stream\"" );
    ParTxt parTxt( mystr );
    StringPar stringPar( parTxt );

    stringPar.set_val( "foo" );

  } catch( ParFileException& pfe ) {

    // right on! Caught the exception
    TEST( "StringPar not_within_min( )", EXIT_SUCCESS, EXIT_SUCCESS );
    if ( printIt )
      cout << pfe;
    return EXIT_SUCCESS;

  } catch ( Exception & e ) {

    cerr << e;
    return EXIT_FAILURE;

  } catch( ... ) {

    cerr << "testParFilename( ) : Unknown exception thrown\n";
    return EXIT_FAILURE;

  }

  // should not be here
  TEST( "StringPar not_string_type( )", EXIT_SUCCESS, EXIT_FAILURE );
  return EXIT_FAILURE;

}

int testStringPar( bool printIt ) {

  if ( EXIT_FAILURE == testString( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_bool_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_int_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_long_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_double_type( printIt ) )
    return EXIT_FAILURE;

  if ( EXIT_FAILURE == not_within_min( printIt ) )
    return EXIT_FAILURE;

  return EXIT_SUCCESS;

}
