/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of photz
 *
 * photz is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * photz is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>


#include "main.h"
#include "common.h"

/* Routine to get the data pointer for a named data packet field */

static char *
dpktf_data( BPipe *bpipe, char *name, void *data )
{
  DpktField *field = bpipe_dpktf( bpipe, name );

  if ( field == NULL )
    tf_exit( ExitERR_misc, "%s isn't in input data stream", name);

  return bpipe_dpktf_data( field, data );

}

void
process(void)
{
  BPipe *bpipe;
  BPipeOutput *bpo;

  void *data;

  DVector3 *position;

  size_t nphot;

  tf_enter("process");

/* Setup bpipe input and output streams */

  if ( ( bpipe = bpipe_new() ) == NULL)
    tf_exit(ExitERR_misc, "couldn't open bpipe: %s",
	    bpipe_strerror(bpipe_errno));

  if ( ( bpipe_input(bpipe, c.input) ) != NULL)
    tf_exit(ExitERR_misc, "couldn't open input file `%s': %s", c.input,
	    bpipe_strerror(bpipe_errno));

  if ( NULL == ( bpo = bpipe_output( bpipe, c.output ) ) )
    tf_exit(ExitERR_misc, "couldn't open input file `%s': %s", c.output,
	    bpipe_strerror(bpipe_errno));

  if (NULL == (data = bpipe_map_alloc(bpipe,1, NULL)))
    tf_exit( ExitERR_alloc, "unable to allocate memory for data packet: %s",
	    bpipe_strerror(bpipe_errno));

  position   = (DVector3 *) dpktf_data( bpipe, BP_AXAF_phot_pos, data);

  if ( bpipe_write_hdr(bpipe) )
    tf_exit(ExitERR_misc,"bpipe_write_hdr: %s",bpipe_strerror(bpipe_errno));
  
  nphot = 0;
  bpipe_errno = BPNOERROR;

  while( bpipe_read_dpkts( bpipe, data, 1 )   == 1) 
  {
    nphot++;

    position->z = c.z;

    /* Write the data packet out  */

    if(bpipe_write_dpkt( bpipe, data, bpo))
       tf_exit( ExitERR_misc, "error writing photon in projray %d", nphot); 
  }

  if ( bpipe_errno != BPNOERROR )
    tf_exit( ExitERR_misc, "error reading photon %d: %s", nphot + 1,
	     bpipe_strerror( bpipe_errno) );

  bpipe_delete( bpipe );

  tf_leave();
}

