/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of quef
 *
 * quef is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * quef is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "quef_def.h"
#include "diagnostic.h"

/*
  PROCEDURE
    Print out an entry of the quantum efficiency of the lookup table.

  RETURN
    void
*/
void print_qe( QE* lookuptable, const char* title )
{
  QuantumEfficiency *table = lookuptable->qe;
  size_t num = lookuptable->num;
  size_t i;

  tf_enter( "qe_print" );

  tf_message( "\n" );
  tf_message( "# %s \n", title);
  tf_message( "idx\temin\temax\tqemin\tqemid\tqemax\n" );
  tf_message( "N\tN\tN\tN\tN\tN\n" );
  for ( i = 0; i < num; i++ )
    tf_message( "%d\t%g\t%g\t%g\t%g\t%g\n",
		i, table[i].emin, table[i].emax,
		table[i].qemin, table[i].qemid, table[i].qemax );
  tf_leave();
}
