/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of quef
 *
 * quef is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * quef is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <stdlib.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <tparam/tparam.h>

#include <suplib/debug.h>
#include <suplib/str.h>

#include <mst_rdb/mst_rdb.h>

#include "quef_def.h"
#include "util.h"
#include "read_qe.h"

#include "param_utils.h"
#include "gparam_def.h"
#include "gparam.h"

#include "config.h"


/*
  PROCEDURE
    Get the all the requested parameters.
    
  RETURN 
    void
 */
void get_parameters( int argc,
		     char** argv,
		     QE* lookuptable,
		     InOut* io,
		     int *version,
		     int *verbose,
		     int *help
		     )
{
  char dummy[MAX_LENGTH];

  paramfile pf;

  tf_enter( "get_parameters" );

  pf = tparamopen( NULL, argv, argc, "r" );

  if ( (*version = tpgetb( pf, "version" )) ||
       (*help    = tpgetb( pf, "help"    ))    )
  {
    paramclose(pf);
    return;
  }

  tpgetstr( pf, DEBUG, dummy, MAX_LENGTH, 0 );
  debug_init( dummy );
  
  *verbose = tpgetb( pf, VERBOSE );

  /* where to get the input output stream */
  io->in = strpget( pf, INPUT );
  io->out = strpget( pf, OUTPUT );

  lookuptable->qemin = tpgetd( pf, "qemin" );
  lookuptable->qemax = tpgetd( pf, "qemax" );
  lookuptable->qeclip = tpgetb( pf, "qeclip" );

  get_lookup_table( pf, lookuptable );

  paramclose(pf);

  tf_leave();
}

