/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of quef
 *
 * quef is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * quef is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <libgen.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "quef_def.h"
#include "quef.h"
#include "gparam.h"

#include "usage.h"

#include "config.h"

/*
  PROCEDURE
    Clean up the memory that were dynamically allocated

  RETURN 
    void
*/
static void cleanup( QuantumEfficiency* qe, InOut* io )
{
  tf_enter( "cleanup" );

  free( qe );
  free( io->in );
  free( io->out );

  tf_leave();

  return;
}


int
main( int argc, char** argv )
{
  InOut io;
  QE lookuptable;
  int version, verbose, help;

  char* prog = basename( argv[0] );

  tf_init( prog, 0, 0 );

  get_parameters( argc, argv,
		  &lookuptable,
		  &io,
		  &version,
		  &verbose,
		  &help
		  );

  if ( version )
  {
    tf_message( "%s\t%s\n", prog, VERSION );
    return EXIT_SUCCESS;
  }

  if ( help )
  {
    usage();
    return EXIT_SUCCESS;
  }

  quantum_efficiency( &lookuptable, &io, verbose );

  cleanup( lookuptable.qe, &io );

  return EXIT_SUCCESS;
}

