/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of quef
 *
 * quef is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * quef is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>

#include <tracefct/exiterrvals.h>
#include <tracefct/tracefct.h>

#include <bpipe/bpipe.h>
#include <bpipe/bpipe_axaf.h>
#include <bpipe/datatypes.h>

#include "quef_def.h"
#include "readbp.h"

/* simple routine to get the data pointer for a named data packet field */
static char *dpktf_data( Bpipe *bpipe, char *name )
{
  DpktField *field = bpipe_dpktf( bpipe->bpipe, name );

  if ( field == NULL )
    tf_exit( ExitERR_misc, name );

  return bpipe_dpktf_data( field, bpipe->data );
}


void
init_bpipe( InOut *io, 
	    Bpipe* pipe, 
	    Photon* photon )
{
  tf_enter( "init_bpipe" );

  if( NULL == (pipe->bpipe=bpipe_new( )) )
    tf_exit( ExitERR_misc, bpipe_strerror( bpipe_errno ) );

  if ( bpipe_input( pipe->bpipe, io->in ) )
    tf_exit( ExitERR_fopen, "%s: %s", io->in,
	     bpipe_strerror( bpipe_errno ) );

  if ( NULL == (pipe->outstream=bpipe_output( pipe->bpipe, io->out ) ) )
    tf_exit( ExitERR_fopen, "%s: %s", io->out,
	     bpipe_strerror( bpipe_errno ) );


  /* map data packet fields */
  if ( NULL == ( pipe->data = bpipe_map_alloc( pipe->bpipe, 1, NULL ) ) )
    tf_exit( ExitERR_illegalArg, bpipe_strerror( bpipe_errno ) );

  /* output header. must do this after calling bpipe_map */
  if ( bpipe_write_hdr( pipe->bpipe ) )
    tf_exit( ExitERR_illegalArg, bpipe_strerror( bpipe_errno ) );

  photon->weight = (double*) dpktf_data( pipe, BP_AXAF_phot_wt );
  photon->energy = (double*) dpktf_data( pipe, BP_AXAF_phot_energy );
  photon->id     = (unsigned int*) dpktf_data( pipe, BP_AXAF_phot_id );

  tf_leave();
  return;
}
