/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of quef
 *
 * quef is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * quef is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <string.h>

#include <tracefct/exiterrvals.h>
#include <tracefct/tracefct.h>

#include "util.h"

void free_tokens( void *ptr )
{
  int i=0;
  char **str_argv = (char **) ptr;

  tf_enter( "free_tokens" );

  while( str_argv[i] )  {
    free( str_argv[i] );
    i++;
  }

  free( str_argv );
  tf_leave();
  return;  
}


/*



  PROCEDURE
    Open a file named `filename'.

  RETURN
    returns a pointer to the FILE structure associated with the stream.

*/
FILE *
open_file_with_mode( char *filename /* The file to be open */, 
		   char *mode /* The mode which the file shall be open with */)
{
  FILE *fp;

  tf_enter( "open_file_with_mode" );

  if ( 0 == strcmp( filename, "stdin" ) )
    fp = (FILE *) stdin;

  else if ( 0 == strcmp( filename, "stdout" ) )
    fp = (FILE *) stdout;

  else if ( 0 == strcmp( filename, "stderr" ) )
    fp = (FILE *) stderr;

  else if ( NULL == (fp=fopen( filename, mode )) )
    tf_exit( ExitERR_fopen, "Can't open file `%s' with mode `%s'\n", filename, mode );

  tf_leave();

  return fp;
}
