/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <float.h>

#include <luasup/luasup.h>

#include <suplib/str.h>

#include <suplib/units.h>
#include <suplib/units_linear.h>

#include "main.h"
#include "lua_interface.h"
#include "misc.h"

/**********************************************************************
   Axial Source Position

    the position of the source relative to the telescope node defaults
    to infinity.  The distance is used to 1) convert from linear to
    angular pixels; 2) correct the direction of the ray to take into
    account the difference between the ray's position in the entrance
    aperture and the telescope node.

*/
void
get_src_z( const char *err_pfx, lua_Object optargs, double *z )
{
  char *ptr_s;
  UnitVal uv;
  uv.unit = UNIT_millimeter;

  if ( ptr_s = slua_tbl_el_si( optargs, SRC_Z ) )
  {
    switch( units_parse( ptr_s, &uv, &UnitsLinear_def ) )
    {
    case UNITS_BADNUM:
    case UNITS_BADSPEC:
      define_error( "%s: bad specification for %s: `%s`", err_pfx, SRC_Z, ptr_s );
    }
    *z = uv.value * units_cvt( &UnitsLinear_def, uv.unit,
				    UNIT_millimeter );
    slua_tbl_el_delete( optargs, SRC_Z );
  }
  else
    *z = -FLT_MAX;
}


/**********************************************************************
   Angular Source Position
*/

void
get_src_pos( const char *err_pfx, lua_Object optargs, double *theta, double *phi )
{
  char *ptr_s;

  if ( ptr_s = slua_tbl_el_si( optargs, THETA ) )
  {
    *theta = parse_angle( err_pfx, ptr_s, "theta" );
    if ( NULL == ( ptr_s = slua_tbl_el_si( optargs, PHI ) ) )
      define_error( "%s: must specify phi as well as theta", err_pfx );
    *phi = parse_angle( err_pfx, ptr_s, "phi" );
    slua_tbl_el_delete( optargs, THETA );
    slua_tbl_el_delete( optargs, PHI );
  }
  else
  {
    *theta = *phi = 0;
  }
}
