/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <luasup/luasup.h>
#include <randlib_c/randlib.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "lua_interface.h"

static void	define(void);
static int	init(FluxGeneratorData *data);
static int	generate( void *data, double *energy );
static void	cleanup(void *data);

Module flat_module = { "flat", define };

typedef struct
{
  double	energy_min;	/* energy */
  double	energy_max;	/* energy */
  double	base_flux;	/* base flux in photons/s/cm^2 */
} FlatData;

static FluxGeneratorSpec spec =
{
  "flat",
  init,
  generate,
  cleanup
};

static void
define( void )
{
  FlatData flat, *data;
  char *name;
  char err_pfx[1024];

  slua_args( define_error, "flat spectrum generator",
		"name",			SLua_STRING,	&name,
		"minimum energy",	SLua_NUMBER,	&flat.energy_min,
		"maximum energy",	SLua_NUMBER,	&flat.energy_max,
		"photon flux",		SLua_NUMBER,	&flat.base_flux,
		NULL
		);

  /* generate error message prefix */
  sprintf( err_pfx, "flat `%s'", name );

  if ( NULL == ( data = (FlatData *) malloc( sizeof(FlatData) ) ) )
    define_error( "%s: error allocating memory", err_pfx );

  if ( flat.energy_max < flat.energy_min || flat.energy_max ==
       flat.energy_min )
    define_error( "%s: energy max (%f) must be greater than energy min (%f)",
		  err_pfx, flat.energy_max, flat.energy_min );

  *data = flat;

  new_flux_gen( &spec, name, data );

  free( name );
}


static int
init( FluxGeneratorData *data )
{
  data->base_flux = ((FlatData *) data->data)->base_flux;
  data->iter_lim = 100;
  data->iter_warn = 10;

  return 0;
}

static int
generate( void *data, double *energy )
{
  FlatData *flat =  (FlatData *) data;

  *energy = genunf( flat->energy_min, flat->energy_max );

  return 0;
}

static void
cleanup( void *data )
{
  free ( data );
}
