/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <string.h>

#include <suplib/str.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "common.h"


void
new_flux_modifier(
  FluxModifierSpec *spec,
  char *name,
  void *data
)
{
  FluxModifier *mod;

  tf_enter( "new_flux_modifier" );

  /* ensure that we're creating a flux generator */
  if ( NULL == sys.current_fgen )
  {
    if ( NULL == sys.current_source )
      tf_exit( ExitERR_misc, "flux_mod `%s': not part of a source", name );
    else
      tf_exit( ExitERR_misc, "source `%s', flux_mod `%s': attempt "
                             "to modify non-existent flux generator",
                             sys.current_source->name, name );
  }

  if ( NULL == (mod = (FluxModifier *) malloc( sizeof(FluxModifier) )) )
    tf_exit( ExitERR_alloc, "%s: unable to create modifier", sys.current_fgen->name );

  memcpy( &mod->spec, spec, sizeof(FluxModifierSpec) );
  mod->spec.name = str_dup( spec->name );
  mod->name = str_join( "/", 2, sys.current_fgen->name, name );
  mod->data = data;

  mod->gen = sys.current_fgen;
  mod->source = sys.current_source;
  mod->factor = NULL;

  tf_leave( );
}

void
delete_flux_modifier(
  void *data
)
{

  FluxModifier *mod = (FluxModifier *) data;

  free( mod->name );
  free( mod->spec.name );
  mod->spec.cleanup( mod->data );
  free( mod );
}

int
init_flux_modifier( FluxModifier *mod )
{
  tf_enter( "init_flux_modifier" );

  mod->factor = &mod->gen->flux_factors[mod->id];

  if ( mod->spec.init( mod->data ) )
    tf_exit( ExitERR_misc, "%s: error initializing flux modifier", mod->name );

  tf_leave( );

  return 0;
}

void
cleanup_flux_modifier_event(
  Event *event
)
{
  FluxModifier *mod = (FluxModifier *) event->module;

  calculate_eff_flux( mod->gen );
}
