/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "lua_interface.h"

static void	define(void);
static int	init(FluxGeneratorData *data);
static int	generate( void *data, double *energy );
static void	cleanup(void *data);

Module mono_module = { "mono", define };

typedef struct
{
  double	energy;		/* energy */
  double	base_flux;	/* base flux in photons/s/cm^2 */
} MonoData;

static FluxGeneratorSpec spec =
{
  "mono",
  init,
  generate,
  cleanup
};

static void
define( void )
{
  MonoData *data;
  char *name;
  double energy, base_flux;
  char err_pfx[1024];

  tf_enter( "mono" );

  get_lua_args( "mono",
		"name",		Lua_STRING,	&name,
		"energy",	Lua_NUMBER,	&energy,
		"photon flux",	Lua_NUMBER,	&base_flux,
		NULL
		);

  /* generate error message prefix */
  sprintf( err_pfx, "mono `%s'", name );

  if ( NULL == ( data = (MonoData *) malloc( sizeof(MonoData) ) ) )
    define_error( "%s: error allocating memory", err_pfx );


  data->energy = energy;
  data->base_flux = base_flux;

  new_flux_gen( &spec, name, data );

  free( name );

  tf_leave( );
}


static int
init( FluxGeneratorData *data )
{
  data->base_flux = ((MonoData *) data->data)->base_flux;
  data->iter_lim = 100;
  data->iter_warn = 10;

  return 0;
}

static int
generate( void *data, double *energy )
{
  *energy = ((MonoData *) data)->energy;

  return 0;
}

static void
cleanup( void *data )
{
  free ( data );
}
