/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <string.h>
#include <math.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <luasup/luasup.h>

#include "main.h"

#include "args.h"
#include "lua_interface.h"

static void	define(void);
static int	init(PositionGeneratorData *data);
static int	generate(void *data, DVector3 *direction, double *z);
static void	cleanup(void *data);

Module point_module = { "point", define };

typedef struct
{
  double	theta;		/* theta coord of point */
  double	phi;		/* phi coord of point */
  DVector3	direction;	/* direction of point */
  double	z;		/* z coord of point */
} PointData;

static PositionGeneratorSpec spec =
{
  "point",
  init,
  generate,
  cleanup
};

static void
define( void )
{
  lua_Object optargs = LUA_NOOBJECT;
  PointData *data;
  char *name;
  char err_pfx[1024];

  tf_enter( "point" );

  slua_args( define_error, "point",
	     "name",		SLua_STRING,	&name,
	     "optargs",	SLua_TABLE | SLua_OPTIONAL,	&optargs,
	     NULL
	     );

  /* generate error message prefix */
  sprintf( err_pfx, "point `%s'", name );

  if ( NULL == ( data = (PointData *) malloc( sizeof(PointData) ) ) )
    define_error( "%s: error allocating memory", err_pfx );

  /* copy attributes, so can delete elements from it as we go along */
  optargs = optargs == LUA_NOOBJECT ? lua_createtable() : slua_tbl_copy( optargs );

  get_src_z( name, optargs, &data->z );
  get_src_pos( name, optargs, &data->theta, &data->phi );

  data->direction.x = -sin( data->theta ) * cos( data->phi );
  data->direction.y = -sin( data->theta ) * sin( data->phi );
  data->direction.z =  cos( data->theta );

  /* we should have read (and deleted) all relevant optargs.  Those
     that remain should be flagged as errors */
  verify_empty_optargs( err_pfx, optargs );

  new_pos_gen( &spec, name, data );

  free( name );

  tf_leave( );
}


static int
init( PositionGeneratorData *data )
{
  data->max_extent	= ((PointData *) data->data)->theta;
  data->z		= ((PointData *) data->data)->z;
  data->factor		= 1.0;

  return 0;
}

static int
generate( void *data, DVector3 *direction, double *z )
{
  *z = ((PointData *) data)->z;

  memcpy(direction, &((PointData *) data)->direction, sizeof(DVector3) );

  return 0;
}

static void
cleanup( void *data )
{
  free ( data );
}
