/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <string.h>

#include <suplib/str.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "common.h"


void
new_pos_gen(
  PositionGeneratorSpec *spec,
  char *		name,
  void *		data
)
{
  PositionGenerator *gen;

  tf_enter( "new_pos_gen" );

  /* ensure that we're creating a source */
  if ( NULL == sys.current_source )
    tf_exit( ExitERR_misc, "pos_gen `%s': not part of a source", name );

  if ( NULL ==
       (gen = (PositionGenerator *) malloc( sizeof(PositionGenerator) )) )
    tf_exit( ExitERR_alloc, "%s: unable to create pos_gen",
                            sys.current_source->name );

  memcpy( &gen->spec, spec, sizeof(PositionGeneratorSpec) );
  gen->spec.name = str_dup( spec->name );
  gen->name = str_join( "/", 2, sys.current_source->name, name );

  gen->source = sys.current_source;
  gen->data.data	= data;

  sys.current_source->pos_gen = gen;

  tf_leave( );
}

int
delete_pos_gen(
  void *data
)
{
  PositionGenerator *gen = (PositionGenerator *) data;

  free( gen->name );
  free( gen->spec.name );
  gen->spec.cleanup( gen->data.data );
  free( gen );

  return 0;
}


void
init_pos_gen(  PositionGenerator *gen )
{
  tf_enter( "init_pos_gen" );

  if ( gen->spec.init( &gen->data ) )
    tf_exit( ExitERR_misc, "%s: error initializing position generator",
	     gen->name );

  tf_leave( );
}

void
cleanup_extent_event( Event *event )
{
  PositionGenerator *gen = (PositionGenerator *) event->module;

  /* reinitialize the effective fluxes of all of the flux generators */
  sll_traverse( gen->source->flux_gen, (SLL_ACTION) calculate_eff_flux );
}
