/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <math.h>

#include <linklist/linklist.h>
#include <randlib_c/randlib.h>
#include <mathconst/mathconst.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "common.h"

void
raygen_init( void )
{
  tf_enter( "raygen_init" );

  if ( 
       NULL == ( sys.gen		= sll_new( flux_gen_time_cmp ) )
       ||
       NULL == ( sys.spare_events	= sll_new( SLL_NULL_CMP ) )
     )
    tf_exit( ExitERR_alloc, "unable to initialize system information" );

  sys.counts = 0;

  /* BEGIN NODE FIX */
#if 0
  tf_message( "ro max\t= %f\n", sys.mirror.rho_f );
  tf_message( "ri max\t= %f\n", sys.mirror.rho_a - sys.mirror.l *
	   tan( sys.max_open_angle) );

  sys.ro = sys.mirror.rho_f + sys.ea_pad_o;
  sys.ri = sys.mirror.rho_a - sys.mirror.l * sys.t_open_angle - sys.ea_pad_i;
  tf_message( "ro\t= %f\n", sys.ro );
  tf_message( "ri\t= %f\n", sys.ri );

  tf_message( "eapad o\t= %f\n", sys.ea_pad_o );
  tf_message( "eapad i\t= %f\n", sys.ea_pad_i );

  sys.ri2 = sys.ri * sys.ri;
  sys.dr2 = sys.ro * sys.ro - sys.ri2;
  sys.ea_area = M_PI * sys.dr2;

  sys.z  = sys.mirror.z_f;
  tf_message( "z\t= %f\n", sys.mirror.z_f );

#endif
  /* END NODE FIX */

  tf_leave( );
}

void
raygen_cleanup( void )
{
  tf_enter( "raygen_cleanup" );

  sll_delete( sys.gen, SLL_NULL_DELETE );

  sll_delete( sys.spare_events, free );
  tf_leave( );
}
