/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <suplib/io.h>

#include <tracefct/tracefct.h>

#include "image.h"

#define WS " \t"

double *
rd_image_data_ascii( const char *err_pfx, const char* image, size_t npix )
{
  char *buf;
  void *rec;
  double *data, *datap, *data_lim;
  FILE *file;

  if ( NULL == ( file = fopen( image, "r" ) ) )
  {
    tf_message( "%s: error opening\n", err_pfx );
    return NULL;
  }

  if ( NULL == (rec = fget_rec_new( 8192, 1024 )) )
  {
    tf_message( "%s: error allocating input buffer\n", err_pfx );
    return NULL;
  }

  if ( NULL == ( data = (double *) malloc( npix * sizeof(double) )) )
  {
    tf_message( "%s: error allocating image data (%d bytes)\n", err_pfx,
    		 npix * sizeof(double) );
    return NULL;
  }

  errno = 0;
  datap = data;
  data_lim = data + npix;
  while ( datap < data_lim && NULL != ( buf = rd_line( file, rec ) ) )
  {
    char *pixel;

    pixel = strtok( buf, WS );
    while ( datap < data_lim && pixel )
    {
      *datap++ = strtod( pixel, NULL );
      pixel = strtok( NULL, WS );
    }
  }
  if ( ENOMEM == errno )
  {
    tf_message( "%s: error extending input buffer\n", err_pfx );
    return NULL;
  }

  if ( ferror( file ) || datap - data != npix )
  {
    tf_message( "%s: error reading image data (read %d pixels, expected %d)\n",
		err_pfx, datap - data, npix );
    return NULL;
  }

  fget_rec_delete( rec );
  fclose( file );
  return data;
}
