/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>

#include <tracefct/tracefct.h>

double *
rd_image_data_float( const char* err_pfx, const char* image, size_t npix )
{
  float *buf;
  double *data, *datap, *data_lim;
  size_t nr;
  FILE *file;

  if ( NULL == ( file = fopen( image, "r" ) ) )
  {
    tf_message( "%s: error opening\n", err_pfx );
    return NULL;
  }

  if ( NULL == ( buf = (float *) malloc( 8192 * sizeof( float ) ) ) )
  {
    tf_message( "%s: error allocating input buffer\n", err_pfx );
    return NULL;
  }

  if ( NULL == ( data = (double *) malloc( npix * sizeof(double) )) )
  {
    tf_message( "%s: error allocating image data (%d bytes)\n", err_pfx, 
    		 npix * sizeof(double) );
    return NULL;
  }

  datap = data;
  data_lim = data + npix;
  while( datap < data_lim &&
	( nr = fread( buf, sizeof(float), 8192, file ) ))
  {
    float *fl, *fl_lim;

    if ( nr > data_lim - datap)
      nr = data_lim - datap;

    fl_lim = buf + nr;
    fl = buf;
    while ( datap < data_lim && fl < fl_lim )
      *datap++ = *fl++;
  }

  if ( ferror( file ) || datap - data != npix )
  {
    tf_message( "%s: error reading image data (read %d pixels, expected %d)\n",
		err_pfx, datap - data, npix );
    return NULL;
  }

  free( buf );

  return data;
}
