/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <mathconst/mathconst.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <linklist/linklist.h>
#include <randlib_c/randlib.h>

#include "main.h"
#include "common.h"
#include "lua_interface.h"

void
system_init( void )
{
  size_t n;

  tf_enter( "system_init" );

  /* initialize random number generator and get to correct block */

  setall( sys.seed1, sys.seed2 );
  for ( n = 0 ; n < sys.block ; n++ )
    initgn(1);

  raygen_init( );

  lua_init( );

  /* create the sources */
  sources_create( );

  /*
    create the entrance apertures. must be done after sources in case
    entrance apertures need to know the maximum extension of the
    sources from the optical axis.
  */
  ea_init( );

  /* now, initialize sources once we know what the entrance aperture
     area is */
  sources_init( );

  /* create the output stream */
  output_init( );

  /* write some messages out */
  tf_message( "ea_area = %f\n", sys.ea_area );

  /* if limit is density, convert to counts */
  if ( Limit_DENSITY == sys.limit_type )
  {
    sys.limit *= sys.ea_area;
    sys.limit_type = Limit_COUNTS;
  }

  tf_message( "limit\t= %f\n", sys.limit );

  tf_leave( );
}

void
system_cleanup( void )
{
  tf_enter( "system_cleanup" );

  output_cleanup( );
  sources_cleanup( );
  ea_cleanup( );
  raygen_cleanup( );

  tf_leave( );
}
