#ifndef _RBTREEP_H
#define _RBTREEP_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of rbtree
 *
 * rbtree is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * rbtree is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include "rbtree.h"

enum { RED=0, BLACK=1 };

typedef enum { LEFT=0, RIGHT=1 } Direction;

typedef struct Node
{
  struct Node *descendent[2];
  struct Node *parent;
  void *data;
  struct
  {
    unsigned color:1;
  }p;
} Node;

typedef struct
{
  Node *root;
  int (*cmp)(const void *, const void *);
  unsigned long count;
  Node nil;
} Tree;


static void Insert(Tree *tree, Node *x);
static Node *Delete(Tree *tree, Node *z);
static void LeftRotate(Tree *tree, Node *x);
static void RightRotate(Tree *tree, Node *y);
static Node *Successor(Tree *tree, Node *x);
static Node *Predecessor(Tree *tree, Node *x);
static Node *Minimum(Tree *tree, Node *x);
static Node *Maximum(Tree *tree, Node *x);
static Node *newNode(Tree *tree, void *data);
static void deleteNode(Node *node);
static Node *Search(Tree *tree, Node *x, const void *data,
		    int (*cmp)(const void *, const void *));
static Node *BndSearch(Tree *tree, Node *x, Node **prev, Node **next,
		       const void *data, int (*cmp)(const void *, const void *));
static int Traverse(Tree *tree, Node *node, int (*action)(void *),
		    SiblingOrder sibling_order);
static int uTraverse(Tree *tree, Node *node,
		     int (*action)(void *, void *udata), void *udata,
		     SiblingOrder sibling_order);
static int Walk(Tree *tree, Node *node,
		int (*action)(void *data, Visit visit, unsigned long level),
		SiblingOrder sibling_order, Visit visit, unsigned long level);
static int uWalk(Tree *tree, Node *node,
		 int (*action)(void *data, Visit visit, unsigned long level, void *udata),
		 void *udata, SiblingOrder sibling_order, Visit visit,
		 unsigned long level);
static void Free_post_order(Tree *tree, void (*nfree)(void *),
			    SiblingOrder sibling_order);
static void Free_in_order(Tree *tree, Node *node, void (*nfree)(void *),
			  SiblingOrder sibling_order);
static void uFree_post_order(Tree *tree, void (*nfree)(void *, void *),
			     void *udata, SiblingOrder sibling_order);
static void uFree_in_order(Tree *tree, Node *node,
			   void (*nfree)(void *, void *), void *udata,
			   SiblingOrder sibling_order);
static void JoinTrees(Tree *tree1, Tree *tree2);

#endif
