#include <cstdlib>
#include <cmath>
#include <iostream>
#include <rdbxx/RDB.h>

using namespace std;

// Write out an RDB file containing values from a sine and cosine function.
int main( int argc, char* argv[] ) {
  try {
    RDB ordb( &cout );

    // Put the program name in the header...
    ordb.setComment( string(": program = ") + argv[0]  );

    // Set up the columns...
    //   Use the automatic row counter to deal with "x" column...
    ordb.setColumn( ordb.getColumn( "_NR" ), "x" );
    ordb.getColumn( "x" )->setDesc( "[radians]" );
    //   Create columns to deal with sine and cosine
    RDBColumn* sin_x = (RDBColumn *)new RDBColumnTmplt<double,long,string>( "sin_x", "N" );
    RDBColumn* cos_x = (RDBColumn *)new RDBColumnTmplt<double,long,string>( "cos_x", "N" );

    ordb.setColumn( sin_x );
    ordb.setColumn( cos_x );

    // Map the sin column to the local variable sine...
    double sine = 0.0;
    ordb.getColumn( "sin_x" )->mapData( &sine, 1 );

    for ( double radians = 0.0; radians < 2 * M_PI; radians += 0.1 ) {
      // This value will show up inside the RDBColumn object sin_x
      // because we mapped it above...
      sine = sin( radians );

      // Since, cos_x isn't mapped to a local variable, we'll just set
      // it through the RDBColumn interface...
      ordb.getColumn( "cos_x" )->setData( cos( radians ) );

      // Now just write out the values...
      ordb.write( );

    }

    delete sin_x;
    delete cos_x;

  // See if RDB failed...
  } catch ( RDBErr& re ) {
    cerr << "Caught an RDBErr object" << endl;
    cerr << re << endl;
    return EXIT_FAILURE;

  // See if anything else failed...
  } catch ( exception& e ) { 
    cerr << "Caught an exception object" << endl;
    cerr << e.what( ) << endl;
    return EXIT_FAILURE;

  // And just in case you missed something...
  } catch ( ... ) {
    cerr << "Caught something unexpected..." << endl;
    return EXIT_FAILURE;

  }

  return EXIT_SUCCESS;
}
