#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>
#include <rdbxx/RDBColumnTmplt.h>
#include <rdbxx/RDBErr.h>

using namespace std;

int main( int argc, char* argv[] ) {
  START( "RDBColumn Header Test Suite" );
  
  string name( "my column name" );
  string def( "15N< What description?" );
  long width = 15;
  RDBColumn::Type type = RDBColumn::NUMERIC;
  RDBColumn::Just just = RDBColumn::LEFT;
  string desc( "What description?" );  
  RDBStringColumn c1( name, def );

  TEST( "RDBColumn::getName( )",  c1.getName( ),  name );
  TEST( "RDBColumn::getDef( )",   c1.getDef( ),   def );
  TEST( "RDBColumn::getWidth( )", c1.getWidth( ), width );
  TEST( "RDBColumn::getType( )",  c1.getType( ),  type );
  TEST( "RDBColumn::getJust( )",  c1.getJust( ),  just );
  TEST( "RDBColumn::getDesc( )",  c1.getDesc( ),  desc );

  def.replace( def.find( 'N' ), 1, 1, 'S' );
  type = RDBColumn::STRING;
  c1.setDef( def );
  TEST( "RDBColumn::getName( )",  c1.getName( ),  name );
  TEST( "RDBColumn::getDef( )",   c1.getDef( ),   def );
  TEST( "RDBColumn::getWidth( )", c1.getWidth( ), width );
  TEST( "RDBColumn::getType( )",  c1.getType( ),  type );
  TEST( "RDBColumn::getJust( )",  c1.getJust( ),  just );
  TEST( "RDBColumn::getDesc( )",  c1.getDesc( ),  desc );

  def.replace( def.find( 'S' ), 1, 1, 'M' );
  type = RDBColumn::MONTH;
  c1.setDef( def );
  TEST( "RDBColumn::getName( )",  c1.getName( ),  name );
  TEST( "RDBColumn::getDef( )",   c1.getDef( ),   def );
  TEST( "RDBColumn::getWidth( )", c1.getWidth( ), width );
  TEST( "RDBColumn::getType( )",  c1.getType( ),  type );
  TEST( "RDBColumn::getJust( )",  c1.getJust( ),  just );
  TEST( "RDBColumn::getDesc( )",  c1.getDesc( ),  desc );

  def.replace( def.find( 'M' ), 1, 1, '-' );
  type = RDBColumn::STRING;
  c1.setDef( def );
  TEST( "RDBColumn::getName( )",  c1.getName( ),  name );
  TEST( "RDBColumn::getDef( )",   c1.getDef( ),   def );
  TEST( "RDBColumn::getWidth( )", c1.getWidth( ), width );
  TEST( "RDBColumn::getType( )",  c1.getType( ),  type );
  TEST( "RDBColumn::getJust( )",  c1.getJust( ),  just );
  TEST( "RDBColumn::getDesc( )",  c1.getDesc( ),  desc );

  def   = "N<";
  width = -1;
  type  = RDBColumn::NUMERIC;
  just  = RDBColumn::LEFT;
  desc  = "";
  c1.setDef( def );
  TEST( "RDBColumn::getName( )",  c1.getName( ),  name );
  TEST( "RDBColumn::getDef( )",   c1.getDef( ),   def );
  TEST( "RDBColumn::getWidth( )", c1.getWidth( ), width );
  TEST( "RDBColumn::getType( )",  c1.getType( ),  type );
  TEST( "RDBColumn::getJust( )",  c1.getJust( ),  just );
  TEST( "RDBColumn::getDesc( )",  c1.getDesc( ),  desc );

  bool caught = false;
  try {
    c1.setDef( "" );
  } catch ( RDBErr& re ) {
    caught = true;
  } catch ( ... ) { 
    caught = false;
  }
  TEST( "RDBColumn::setDef( )", caught, true );

  caught = false;
  try {
    c1.setDef( "12<N foo" );
  } catch ( RDBErr& re ) {
    caught = true;
  } catch ( ... ) { 
    caught = false;
  }
  TEST( "RDBColumn::setDef( )", caught, true );

  caught = false;
  try {
    c1.setDef( "P" );
  } catch ( RDBErr& re ) {
    caught = true;
  } catch ( ... ) { 
    caught = false;
  }
  TEST( "RDBColumn::setDef( )", caught, true );

  caught = false;
  try {
    c1.setDef( "14N-" );
  } catch ( RDBErr& re ) {
    caught = true;
  } catch ( ... ) { 
    caught = false;
  }
  TEST( "RDBColumn::setDef( )", caught, true );

  caught = false;
  try {
    c1.setDef( "14N<Description..." );
  } catch ( RDBErr& re ) {
    caught = true;
  } catch ( ... ) { 
    caught = false;
  }
  TEST( "RDBColumn::setDef( )", caught, true );

  caught = false;
  try {
    c1.setDef( "14N<\tDescription..." );
  } catch ( RDBErr& re ) {
    caught = true;
  } catch ( ... ) { 
    caught = false;
  }
  TEST( "RDBColumn::setDef( )", caught, true );

  SUMMARY( );
  
  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
