#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>
#include <rdbxx/RDBComment.h>

using namespace std;

void visualtest( RDBComment& c, string& commexp, string& keyexp, string& valexp ) {
  cerr << "RDBComment::getComment( ):  `" << c.getComment( ) << "' == `" << commexp << "'" << endl;
  cerr << "RDBComment::getKeyword( ):  `" << c.getKeyword( ) << "' == `" << keyexp << "'" << endl;
  cerr << "RDBComment::getValue(   ):  `" << c.getValue( )   << "' == `" << valexp << "'" << endl << endl;

}

void autotest( RDBComment& c, string& commexp, string& keyexp, string& valexp ) {
  TEST( "RDBComment::getComment( )", c.getCommentText( ), commexp );
  TEST( "RDBComment::getKeyword( )", c.getKeyword( ),  keyexp );
  TEST( "RDBComment::getValue(   )", c.getValue( ),    valexp );

}

void test( RDBComment& c, string& commexp, string& keyexp, string& valexp, bool automatic ) {
  if ( automatic ) 
    autotest(   c, commexp, keyexp, valexp );
  else 
    visualtest( c, commexp, keyexp, valexp );

}

int main( int argc, char* argv[] ) {
  bool automatic = true;
  if ( 1 < argc ) {
    if ( 0 == strcmp( argv[1], "visual" ) ) {
      automatic = false;

    }
  }

  START( "RDBComment Test Suite" );

  RDBComment c1( "# One plain header comment" );
  string commexp("One plain header comment");
  string keyexp("");
  string valexp("");
  test( c1, commexp, keyexp, valexp, automatic );

  RDBComment c2( " One plain header comment " ); 
  commexp = "One plain header comment ";
  keyexp  = "";
  valexp  = "";
  test( c2, commexp, keyexp, valexp, automatic );

  RDBComment c3( "One plain header comment\t" ); 
  commexp = "One plain header comment\t";
  keyexp  = "";
  valexp  = "";
  test( c3, commexp, keyexp, valexp, automatic );

  RDBComment c4( "#:Header = Variable comment..." );
  commexp = "Header = Variable comment...";
  keyexp  = "Header";
  valexp  = "Variable comment...";
  test( c4, commexp, keyexp, valexp, automatic );

  RDBComment c5( ":Header =Variable comment..." );
  commexp = "Header =Variable comment...";
  keyexp  = "Header";
  valexp  = "Variable comment...";
  test( c5, commexp, keyexp, valexp, automatic );

  RDBComment c6( "Header= Variable comment..." );
  commexp = "Header= Variable comment...";
  keyexp  = "";
  valexp  = "";
  test( c6, commexp, keyexp, valexp, automatic );

  SUMMARY( );
  
  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
