#include <cstdlib>
#include <iostream>
#include <map>
#include <rdbxx/RDB.h>

using namespace std;

// Using group columns, find the min and max values as well as the
// number of elements in a group.
int main( int argc, char* argv[] ) {

  try {
    // Set up input and output files...
    RDB irdb( &cin );
    RDB ordb( &cout );

    // Get the number of input columns...
    size_t nColumns       = irdb.nColumns( );
    vector<int> mincols(nColumns), maxcols(nColumns), cntcols(nColumns), groupcols(nColumns);

    // Foreach input column...
    for ( size_t idx = 0; idx < nColumns; idx++ ) {
      string name = irdb.getName( idx );
      string def  = irdb.getDef(  idx );
	
      // If its numeric, setup the min, max, and count column...
      if ( RDBColumn::NUMERIC == irdb.getType( idx ) ) {
	// Create a min, max, and count column foreach numeric input column...
	ordb.setColumn( name + "_min", def );
	ordb.setColumn( name + "_max", def );
	ordb.setColumn( name + "_cnt", def );

	mincols[idx] = ordb.nColumns( ) - 3;
	maxcols[idx] = ordb.nColumns( ) - 2;
	cntcols[idx] = ordb.nColumns( ) - 1;

	groupcols[idx] = -1;

      // Else assume its a break column...
      } else {
	// Set up a group on any non-numeric field...
	ordb.setColumn( name, def );
	irdb.setGroup( idx );

	mincols[idx] = -1;
	maxcols[idx] = -1;
	cntcols[idx] = -1;

	groupcols[idx] = ordb.nColumns( ) - 1;

      }
    }
    
    bool ok = false;
    int status;
    // Read the input...
    while ( RDB::REOF != (status = irdb.read( )) ) {
      //      cerr << irdb.getDataDouble( 0 ) << "\t" << irdb.getDataDouble( 1 ) << "\t" << irdb.getDataString( 2 ) << "\t" << status << endl;

      // New group...
      if ( RDB::RBOG & status ) {

	// Write out stats from last group...
	if ( 1 != irdb.getDataLong( "_NR" ) ) 
	  ordb.write( );
	else
	  ok = true;

	// Begin stats for new group...
	for ( size_t idx = 0; idx < nColumns; idx++ ) {
	  if ( RDBColumn::NUMERIC == irdb.getType( idx) ) {
	    ordb.setData( mincols[idx], irdb.getDataDouble( idx ) );
	    ordb.setData( maxcols[idx], irdb.getDataDouble( idx ) );
	    ordb.setData( cntcols[idx], 1.0 );
	    
	  } else {
	    ordb.setData( groupcols[idx], irdb.getDataString( idx ) );
	  
	  }
	}

      // Old group...
      } else {
	for ( size_t idx = 0; idx < nColumns; idx++ ) {
	  // Keep accumulating stats...
	  if ( RDBColumn::NUMERIC == irdb.getType( idx ) ) {
	    if ( ordb.getDataDouble( mincols[idx] ) > irdb.getDataDouble( idx ) ) {
	      ordb.setData( mincols[idx], irdb.getDataDouble( idx ) );
	      
	    }
	    if ( ordb.getDataDouble( maxcols[idx] ) < irdb.getDataDouble( idx ) ) {
	      ordb.setData( maxcols[idx], irdb.getDataDouble( idx ) );
	      
	    }
	    
	    ordb.setData( cntcols[idx], ordb.getDataDouble( cntcols[idx] ) + 1.0 );
	    
	  }
	}
      }
    }

    // Write the last group's statistics out...
    if ( ok ) 
      ordb.write( );
    
  } catch ( RDBErr& re ) {
    cerr << "RDBErr:" << endl << re << endl;
    return EXIT_FAILURE;
    
  } catch ( Exception& E ) {
    cerr << "Exception:" << endl << E << endl;
    return EXIT_FAILURE;

  } catch ( exception& e ) {
    cerr << "exception:" << endl << e.what( ) << endl;
    return EXIT_FAILURE;
    
  } catch ( ... ) {
    cerr << "Caught something unexpected..." << endl;
    return EXIT_FAILURE;

  }

  return EXIT_SUCCESS;
}
