// File:    rl_BasicRay.cc
// Author:  Terry Gaetz
//
// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rl_ray
//
// rl_ray is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_ray is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <rl_basicray/rl_BasicRay.h>

#include <iostream>
#include <sstream>
#include <iomanip>

using namespace std;

//=========================================================================
// dtor, ctors, initializers...

//-------------------------------------------------------------------------
rl_BasicRay::
~rl_BasicRay()
{}

//=========================================================================
// mutators...

//=========================================================================
// ray reflection...

//-------------------------------------------------------------------------
double rl_BasicRay::
reflect_direction_vector( dvm3_Vector const& normal )
{
  // calculate sine of the grazing angle
  double sg = dot( dir_, normal );

  // reflect the direction vector:
  //    vo = (1 - 2 norm <diadprod> norm) <dotprod> vi

  dir_.lincomb( 1.0, dir_, -2.0 * sg, normal );

  return sg;
}

//=========================================================================
// I/O...

//-------------------------------------------------------------------------
ostream& rl_BasicRay::
print_on( ostream& os, char const prefix[], char const postfix[] ) const
{
  os.setf( ios::scientific, ios::floatfield );
  os.precision(15);
  if ( ::strlen(prefix) ) { os << prefix; }
  os << id_ << ": " << energy_ << " " << pos_ << " " << dir_;
  if ( ::strlen(postfix) ) { os << postfix; }
  return os;
}

//-------------------------------------------------------------------------
ostream&
operator<<( ostream& os, rl_BasicRay const& r )
{
  return r.print_on( os, "ray[", "]" );
}
