// File:   t_rl_RayMath.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rl_ray
//
// rl_ray is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_ray is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <rl_basicray/rl_RayMath.h>

#include <mathconst/mathconst.h>
#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>
#include <dvm3/dvm3.h>

static
int rl_RayMath_test01()    /* test pure z rotation */
{
  double az   = 0.0;
  double el   = 0.0;
  double zrot = 0.1;
  dvm3_RotMat m(az, el, zrot);
  dvm3_Vector v(1.0, 0.0, 0.0);
  rl_RayMath::rotate( m, v );
  int rc0; 
  rc0 = !fuzzy_compare(v[0], cos(zrot)) &&
        !fuzzy_compare(v[1], sin(zrot)) &&
        !fuzzy_compare(v[2], 0.0      );

  // cerr << endl;  cerr << "rc0 " << rc0 << " ";  v.print_on( cerr );

  rl_RayMath::derotate( m, v );

  int rc1;
  rc1 =  !fuzzy_compare(v[0],  1.0) &&
         !fuzzy_compare(v[1],  0.0) &&
         !fuzzy_compare(v[2],  0.0);

  return rc0 && rc1;
}

static
int rl_RayMath_test02()    /* test translation + rotation */
{
  double az   = 0.0;
  double el   = 0.0;
  double zrot = 0.1;
  dvm3_RotMat m(az, el, zrot);
  dvm3_Vector v(1.0, 0.0, 0.0);
  dvm3_Vector trans(1.0, 0.0, 0.0);
  rl_RayMath::translate_rotate( trans, m, v );
  int rc0;
  rc0 =  !fuzzy_compare(v[0],  2.0 * cos(zrot)) &&
         !fuzzy_compare(v[1],  2.0 * sin(zrot)) &&
         !fuzzy_compare(v[2],  0.0);

  rl_RayMath::derotate_detranslate( trans, m, v );

  int rc1;
  rc1 =  !fuzzy_compare(v[0],  1.0) &&
         !fuzzy_compare(v[1],  0.0) &&
         !fuzzy_compare(v[2],  0.0);

  return rc0 && rc1;
}


int
main() {

  int rc = 1;
  int rc1;
  rc1 = rl_RayMath_test01();
  TEST( "pure z rotation",        rc1, 1 );
  rc &= rc1;

  rc1 = rl_RayMath_test02();
  TEST( "translation + rotation", rc1, 1 );
  rc &= rc1;

  return !rc;
}
