// File:   rl_Ray.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006, 2007 Smithsonian Astrophysical Observatory
//
// This file is part of rl_raylib
//
// rl_raylib is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_raylib is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <rl_raylib/rl_Ray.h>
#include <iostream>
#include <iomanip>

using namespace std;

//=========================================================================
// dtor, ctors, initializers...

//-------------------------------------------------------------------------
// virtual destructor

rl_Ray::
~rl_Ray()
{}

//=========================================================================
// mutators...

//-------------------------------------------------------------------------
// reflect this ray's direction vector and polarization state.

void rl_Ray::
reflect( dvm3_Vector const& normal,
         rl_ReflectionCoefPOD const& rflcoef )
{
  dvm3_Vector old_dir( dir_ );
  reflect_direction_vector( normal );
  pol_.reflect( normal, old_dir, dir_, rflcoef );
}

//-------------------------------------------------------------------------
// translate to to bcs origin; rotate from std to bcs coordinates

void rl_Ray::
translate_rotate( dvm3_Vector const& trans,
                  dvm3_RotMat const& rotmat )
{
  rl_BasicRay::translate_rotate( trans, rotmat );
  rl_Polarization tmp( pol_ );
  tmp.rotate( pol_, rotmat );
}

//-------------------------------------------------------------------------
// derotate back to std coordinates; detranslate back to std origin

void rl_Ray::
derotate_detranslate( dvm3_Vector const& trans,
                      dvm3_RotMat const& rotmat )
{
  rl_BasicRay::derotate_detranslate( trans, rotmat );
  rl_Polarization tmp( pol_ );
  tmp.derotate( pol_, rotmat );
}

//-------------------------------------------------------------------------
// write out ray contents with optional pre and post comment strings
std::ostream& rl_Ray::
print_on( std::ostream&   os,          //i  output stream
          char const      pre[],       //i  optional prefix comment
          char const      pst[]        //i  optional postfix comment
        ) const
{
  if ( ::strlen(pre) ) { os << pre; }
  os.setf(ios::showpoint);
  os.setf(ios::fixed);
  os << setw(8) << id_ << " : ";
  os << " " << setw(18) << energy_ << " : (";
  pos_.print_on( os );
  os << ") (" << setw(18);
  dir_.print_on( os );
  os << ")";
  if ( ::strlen(pst) ) { os << pst; }
  os << endl;

  if ( ::strlen(pre) ) { os << pre; }
  os << setw(8) << id_ << " : ";
  pol_.print_on( os );
  if ( ::strlen(pst) ) { os << pst; }
  os << endl;

  return os;
}
