// File:   t_rl_DielectricData.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rl_raylib
//
// rl_raylib is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_raylib is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <rl_raylib/rl_DielectricData.h>
#include <rl_raylib/rl_DielectricPODArray.h>

#include <cstdlib>
#include <cstdio>

using namespace std;

rl_DielectricPODArray*
new_dielectric_pod_array( char const srcfile[] )
{
  char* line     = new char[1024];
  double* energy = new double[2048];
  double* alpha  = new double[2048];
  double* gamma  = new double[2048];

  // skip to data columns
  FILE* src = fopen( srcfile, "r" );

  while ( (line = fgets(line, 1023, src)) )
  {
    if ( feof(src) ) break;
    if ( line[0] != '#' ) break;
  }
  // throw away the column names and widths
  line = fgets(line, 1023, src);
  // line = fgets(line, 1023, src);

  // read the data
  int nrows = 0;
  while ( (line = fgets(line, 1023, src)) && ! feof( src ) )
  {
    double energy_;
    double alpha_;
    double gamma_;
    int nmatch = sscanf( line, "%le\t%le\t%le", 
                         &energy_, &alpha_, &gamma_ );
    if ( nmatch == 3 )
    {
      energy[nrows] = energy_;
      alpha[nrows]  = alpha_;
      gamma[nrows]  = gamma_;
      ++nrows;
    }
    else
    {
      break;
    }
  }
  return new rl_DielectricPODArray( nrows, energy, alpha, gamma );
}
