// File:   t_rl_DielectricPOD_rdb.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rl_raylib
//
// rl_raylib is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_raylib is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

// t_rl_dielectricpod_rdb.cc

#include <rl_raylib/rl_DielectricPODArray.h>

#include <mathconst/mathconst.h>
#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>

using namespace std;

extern
rl_DielectricPODArray*
new_dielectric_pod_array( char const srcfile[] );

static
int rl_DielectricPODArray_test01(char const* s0,
                                 char const* outfile)    /* test ctor, dtor */
{
  rl_DielectricPODArray* diel = new_dielectric_pod_array(s0);

  FILE* out = fopen(outfile, "w");
  if (! out )
  {
    fprintf(stderr, "Failed to open output file %s\n", outfile);
    exit(EXIT_FAILURE);
  } 

  diel->cprint_on(out,
                  static_cast<char const*>(""),
                  static_cast<char const*>("") );


  fclose(out);

  return 1;
}

int
main( int argc, char* argv[] )
{
  if ( argc != 2 )
  {
    cerr << "Usage: " << argv[0] << " srcdir\n";
    return EXIT_FAILURE;
  }
  char f0[]   = "/tests/inputs/all_dbs_fit_i_stripped.rdb";

  START("Testing reading of dielectric constants");

  int srclen = strlen((++argv)[0]);

  char* src0 = new char[static_cast<size_t>(srclen + sizeof(f0) + 1)];

  strncpy( src0, argv[0], srclen+1 );
  strncat( src0, f0, sizeof(f0)+1 );

  int rc = rl_DielectricPODArray_test01(src0, "dpa.log");
  TEST( "ctor, dtor", rc, 1 );

  SUMMARY();

  delete [] src0;

  return !rc;
}
