// File:   t_rl_Multilayer_1.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2008 Smithsonian Astrophysical Observatory
//
// This file is part of rl_raylib
//
// rl_raylib is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_raylib is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "rl_raylib/rl_Multilayer.h"
#include "rl_raylib/rl_ReflectionCoefPOD.h"

#include <rl_raylib/rl_DielectricLayer.h>
#include <rl_raylib/rl_DielectricPODArray.h>

#include <mathconst/mathconst.h>
#include <cmath>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <iostream>

// #define DEBUG_MULTILAYER

using namespace std;

extern
rl_DielectricPODArray*
new_dielectric_pod_array( char const srcfile[] );

static
int rl_Multilayer_test01( char const src0[],
                          char const src1[],
                          char const src2[],
                          FILE* out
                        )
           /* test multilayer_reflectivity() */
{

  rl_DielectricPODArray* diel0 = new_dielectric_pod_array( src0 );
  rl_DielectricPODArray* diel1 = new_dielectric_pod_array( src1 );
  rl_DielectricPODArray* diel2 = new_dielectric_pod_array( src2 );

  rl_DielectricLayer diel[3];
  
  diel[0].init( diel0->const_data_ptr(), diel0->num_elts(),
                327.8525, 5.0229055, rl_Traits::ERoughNone,
                rl_Traits::ELinLin, 1.0, "Ir" );
  diel[1].init( diel1->const_data_ptr(), diel1->num_elts(),
                97.22625, 5.32, rl_Traits::ERoughNone,
                rl_Traits::ELinLin, 1.0, "Cr" );
  diel[2].init( diel2->const_data_ptr(), diel2->num_elts(),
                1.e30, 5.32, rl_Traits::ERoughNone,
                rl_Traits::ELinLin, 1.0, "Zerodur", rl_Traits::True );

  rl_Multilayer ml( 3, diel, rl_Traits::False );

  double sg[] = { 51.2, 41.2, 36.4, 27.0 };

  ml.cdump_on( out, 0, const_cast<char const*>(""),
                       const_cast<char const*>("") );
  ml.cdump_on( out, 1, const_cast<char const*>(""), 
                       const_cast<char const*>("") );
  ml.cdump_on( out, 2, const_cast<char const*>(""),
                       const_cast<char const*>("") );

  rl_Traits::rl_DielectricPOD const* data = diel0->const_data_ptr();

  size_t n;
  double const deg2rad = 3.14159265358979323846 / 180.0;
  for ( n = 0 ; n < 4 ; ++n )
  {
    sg[n]  = sin( deg2rad * sg[n] / 60.0 );
  }

  int rc = 1;

  for ( n = 0 ; n < diel0->num_elts() ; n += 20 )
  {
    double rfl1 = 0.0;
    double rfl2 = 0.0;
    double rfl3 = 0.0;
    double rfl4 = 0.0;
    rc    &= !(ml.multilayer_reflectivity(rfl1, data[n].energy_, sg[0]));
    rc    &= !(ml.multilayer_reflectivity(rfl2, data[n].energy_, sg[1]));
    int i = ml.multilayer_reflectivity(rfl3, data[n].energy_, sg[2]);
    i = ml.multilayer_reflectivity(rfl4, data[n].energy_, sg[3]);
    fprintf(out, "%d\t%d\t%15.9e\t%15.9e\t%15.9e\t%15.9e\t%15.9e\n",
             rc, static_cast<int>(n), data[n].energy_, rfl1, rfl2, rfl3, rfl4 );
  }

  delete diel2;
  delete diel1;
  delete diel0;

  return rc;
}

int
main( int argc, char* argv[] ) 
{
  string inputdir( argv[1] );

  string src0 = inputdir + "/all_dbs_fit_i_stripped.rdb";
  string src1 = inputdir + "/Cr_Gullikson95_stripped.rdb";
  string src2 = inputdir + "/Zerodur_Gullikson95_stripped.rdb";

  FILE* out = fopen("ml_test.out", "w");

  int rc  = 1;
  rc &= rl_Multilayer_test01( src0.c_str(), 
                              src1.c_str(), 
                              src2.c_str(), 
                              out);

  fclose(out);

  return !rc;
}
