
***** file: initsaofocus.f  <> $Revision: 1.3 $

*+**********************************************************************
      integer function initsaofocus(rcsid_string)
************************************************************************
*
*kwic initsaofocus --- initialization for saofocus
*
*  arguments:
*     rcsid_string -- ca, i  -- rcs id string
*
*  return:
*     0 for successful initialization; 1 if unable to open gi file
*
*  comments:
*
*  USER INPUTS :
*  via parameter file: 
*             fnm_gi:  'gi' file name
*
*  USER OUTPUTS :
*  via # 8 : printed information about the ray trace
*
*  BINARY I/O:
*  via #  5 (stdin) : current surface ray file for last surface
*  via #  6 (stdout): focal plane ray file
*
*  END OF USER INPUT & OUTPUT
*
*  INPUT via the OS supplied pgms DATE & TIME !***
*
*  Output via /PARM/ and /PARX/ :   (see RDGI)
*
*  XR : PROGID, VINIT, TIME, DATE
*
*  history:
*
*    98-Feb-05 [T. Gaetz]
*       . eliminate srfno
*    97-Oct-27 [D. Grumm]
*       . external routine c_paccess to input parameters replaced by getparam.f
*       . attach_bpipe call added to acquire & attach the BPipe
*    93-Nov-10[T. Gaetz]
*       . add save stmt and move /raystuff/ to include file
*       . initialize KURF to NS
*       . clean up documentation to reflect the world as it is.
*    93-Nov-08[T. Gaetz]
*       . reorder /DEFORM/ for alignment; add save stmt; move to include file
*       . reorder /PARX/   for alignment; add save stmt; move to include file
*    93-Oct-20[T. Gaetz]
*       . original version
*
*$Id: initsaofocus.f,v 1.3 1998/03/11 21:45:14 gaetz Exp $
*-
      implicit none

*---  named constants...

      INTEGER     LUNERR
      PARAMETER ( LUNERR = 0 )   ! unit number for 'stderr'

*---  passed parameter...
      CHARACTER*(*) rcsid_string

*---  global variables...

      include 'saosacLib/deform.h'    ! fourier-legendre deformation coeffs
      include 'saosacLib/iofiles.h'   ! i/o filenames
      include 'saosacLib/parx.h'      ! gfoc, ndim, ns, kurf, ...
      include 'saosacLib/raystuff.h'  ! energy, norm, g_ang, posdir_old...

      DOUBLE PRECISION GTIME
      CHARACTER*(4)    GID, RID, DFID
      CHARACTER*(8)    GDATE , RDATE, RTIME, RDUMMY !*** WERE REAL*8
      COMMON /IDENT/   GID (16), GDATE, GTIME (2),
     1                 RID (16), RDATE, RTIME (2),
     2                 DFID (16)

      DOUBLE PRECISION AQ, WF1, WF2, WS1, WS2, XMIN, XMAX, YMIN, YMAX,
     1                 WX, WY, WR, WXSQ, WYSQ, STAV
      INTEGER          JAIL1, JAIL2, LEFT1, LEFT2
      COMMON /SUMRY/   AQ (10), WF1, WF2, WS1, WS2, JAIL1, JAIL2,
     1                 LEFT1, LEFT2, XMIN, XMAX, YMIN, YMAX,
     2                 WX, WY, WR, WXSQ, WYSQ, STAV (2, 21)

      LOGICAL *1       LSW0, LSW
      COMMON /SWITCH/  LSW0 (10), LSW (10, 21)

*---  external routines...
      integer  len_trim
      external len_trim

*---  local variables...
      integer           ier, ln1, ln2
* debug parameter options:
      integer   db_extend

*CODE initsaofocus -----------------------------------------------------

*---  data initializations...
      initsaofocus = 0

*---  Read in parameter file...(and debug flags)
      call getparam(rcsid_string, db_extend)

*---  open log file...
      open (8, FILE = log_file )

*---  Identify program and its release number
*     and specify that the identifier has 2 lines
      CALL PROGID ('SAOFOCUS', '1.00', 2)

*---  open the binary ray streams...
      ln1 = len_trim(in)
      ln2 = len_trim(out)
      call attach_bpipe( in(:ln1), out(:ln2), db_extend, fgoodrays )

*---  Get rdate and rtime
      CALL SAODATE (RDATE)   !*** WAS ZTIME
      CALL SAOTIME (RDUMMY)
      RTIME(1) = RDUMMY   

*===> ------------------------------------------------------------------
*
*---     Read the 'gi' file into memory
         open(9, FILE = fnm_gi, STATUS ='OLD', ERR = 300)
         CALL RDGI(9, ier)
         close(9)

*---     initialize KURF ('current surface') to last surface 
*        (given by NS, the number of surfaces).
         KURF = NS

*---     VERFOC:
*               1 - calculate the geometric center, radius of curvature,
*                   vertices, foci, and eccentricity for the surfaces.
*               2 - calculate the confocal delta between two consecutive
*                   surfaces.
*               3 - find the geometric focus of the system.
         CALL VERFOC

*---     Initialize the vector quantities
         CALL VINIT

*---     Initialize no. of rays and weight
         LEFT2       = 0
         WS2         = 0.0d0
*
*<=== ------------------------------------------------------------------

*---  normal exit...

      initsaofocus = 0
      return

*---  error exits...

300   continue
      write (0, *) 'saofocus: Error opening .gi file - exiting...'
      initsaofocus = 1
      return

      END ! initsaofocus -----------------------------------------------
