*+
*KWIC saofocus.f
*
*$Id: saofocus.f,v 1.2 1998/03/11 21:45:15 gaetz Exp $
*
*Revisions:
*   95-Mar-15[T. Gaetz]
*      . change common /raystuff/ to include '../include/raystuff.h'
*   93-Nov-16[T. Gaetz]
*      . move c_open() call to after reading of gi file; need current surface
*        number to ensure correct sin(graze angle) entry is read by c_read
*        (so it can be passed to c_write)
*   93-Oct-20[T. Gaetz]
*      . snip out initialization stuff and move to initsaodrat.f
*      . block002.f renamed to block002.h and moved to include directory
*   93-Sep-29[T. Gaetz]
*      . renamed to saofocus; change i/o messages to conform;
*        rename nexray2 to saonexray, trader_foc to saotrader_foc,
*        raydum2 to saoraydum
*      . increase size of filenames gi_file, summary_file to 256 chars
*   93-Apr-09[T. Gaetz]
*      . change over to implicit none;
*   93-Apr-15[T. Gaetz]
*      . rename common /graze/ to /raystuff/; add energy, norm to common
*      . strip out commented-out statements; clean up commenting
*      . eliminate dead code
*      . clean up structure
*-

*  Created by Mark Freeman
*  on Thu Jan 14 08:57:57 EST 1993
*  Add command line arguments (2) for input file (.gi) and
*  deformations file (.DFR or .SPL)
*  '.gi' file is read directly by drat2, not '.gx' file
*  Rays piped in via stdin(5), out via stdout(6) in binary ray
*    format (see c_read_, c_write_, etc.)
*  'nexray2' always reads from pipe, no source calculations.
*  'trader2' writes them to the pipe.
*  'fovel' (focal plane file creation, etc.) to be moved to
*    another module.
*
*  Based on DRAT as below:
*
!  UPDATED 02/26/88 TO PRINT "VERSION 4.0"
!  PAUL GLENN, BAUER ASSOCIATES, INC.
!
!  UPDATED 5/9/88 TO PRINT "VERSION 5.0", TO CALL REDECO WITH THE
!  FILE NUMBER AND PRING FLAG AS CALLING PARAMETERS, TO CALL
!  ZBUILD TO DEFINE THE ANNULAR ZERNIKE POLYNOMIALS IF APPROPRIATE,
!  AND TO MAKE DFID CHARACTER * 4
!  PAUL GLENN, BAUER ASSOCIATES, INC.
!
*  MAIN PGM  : DRAT  Deformed surface RAy Trace
*   /******************************************/
*    *
*    *    PERKIN-ELMER CORPORATE COMPUTING
*    *      SOFTWARE ENGINEERING SECTION
*    *
*    *    DRAT2 FORTRAN
*    *    WRITTEN BY JOHN LEGG
*    *            ON 04/29/80
*    *
*    *    UPDATE:   11/01/83
*    *    TIME:     13:03:25
*    *
*    ******************************************/
*
*  PURPOSE: To trace an input collimated bundle of rays
*           through a deformed optical system
*
*  USER INPUTS :  see initsaofocus.f
*
*  i/o streams :
*
*     USER INPUTS  :
*        via # 5 :  KURF : surface number (ascii)
*                   name of input 'GI' file
*        via # 5 : (stdin) binary ray input stream
*
*     USER OUTPUTS :
*        via # 6 : (stdout) binary ray output stream
*        via # 8 : printed information about the ray trace
*
*  END OF USER INPUT & OUTPUT
*  XR : FOVEL, SAONEXRAY, SAORAYDUM, SAOTRADER_FOC, INITSAOFOCUS, C_CLOSE
*
*
      IMPLICIT NONE

      CHARACTER*(*) RCSID_STRING
      PARAMETER   ( RCSID_STRING =
     &'$Id: saofocus.f,v 1.2 1998/03/11 21:45:15 gaetz Exp $' )

*---  global variables...
      INCLUDE 'saosacLib/raystuff.h'  ! info needed for binary pipeline

*---  local variables...
      DOUBLE PRECISION P (3), SPOT
      INTEGER          ier

*---  support for get_ray, nexray
      double precision pos_c(3), dir_c(3), weight_c, energy_c
      complex*16       c2comp_c(2), s2comp_c(2)
      integer          rayid_c, raycode_c


*---  external routines...
      integer  initsaofocus
      external initsaofocus

      INCLUDE 'saosacLib/block002.h'

*CODE saofocus ---------------------------------------------------------

*---  initialize; open i/o streams...
      ier = initsaofocus(rcsid_string)
      if ( ier .ne. 0 ) then
*---     Close the ray streams and exit...
         call detach_bpipe  ()
         call flush(8)
         close(8)
         stop 10
      endif

*===> DO ...............................................................
100   continue

      CALL GET_RAY( pos_c, dir_c, weight_c, energy_c, c2comp_c,
     &              s2comp_c, rayid_c, raycode_c, ier )

      if (ier .ne. 0) go to 101  ! IF (...) EXIT ....................

      CALL SAONEXRAY( pos_c, dir_c, weight_c, energy_c, c2comp_c,
     &                s2comp_c, rayid_c, raycode_c, ier )


      if (ier .ne. 0) go to 101  ! IF (...) EXIT ....................

*---     Trace ray to geometric focus
      CALL saotrader_foc

      go to 100

*<=== END DO ...........................................................
101   continue

*---  Write ray summary report on unit 8
      CALL SAORAYDUM

*---  Find focus, write focus report and quadratic coefficients
      CALL FOVEL (AQ, P, SPOT)

*---  Close the BPipe and logfile..

      call detach_bpipe ()

      call flush(8)
      close(8)
      stop           ! normal exit...

      END ! saofocus ---------------------------------------------------








