      FUNCTION DCADRE (F, A, B, AERR, RERR, ERROR, IER)
C   /******************************************/
C    *
C    *    BAUER ASSOCIATES, INC.
C    *
C    *    DCADRE FORTRAN
C    *    WRITTEN BY P. GLENN (617) 235-8775
C    *            ON 01/28/88
C    *
C   /******************************************/
C
C  PURPOSE: ALLOW A CALL TO THE IMSL ROUTINE DCADRE TO BE REPLACED
C           (TRANSPARENTLY TO THE USER) BY A CALL TO CMLIB ROUTINES,
C           SO THAT THE IMSL LIBRARY NEED NOT BE AVAILABLE
C
C  RESTRICTIONS: THE ERROR CODE INTERPRETATION IS DIFFERENT, AS
C                DETAILED BELOW.  ALL OF THE CMLIB ERROR CODES
C                ARE LUMPED INTO TWO OF THE IMSL ERROR CODES.  THE
C                CMLIB ROUTINE IS ALLOWED A MAXIMUM OF 100 SUBDIVISIONS
C                OF THE REGION OF INTEGRATION.  THE ROUTINE'S NEED
C                TO GO OVER THIS LIMIT WILL GIVE AN ERROR.
C
C  INPUT PARAMETERS:
C    F = SINGLE-ARGUMENT DOUBLE PRECISION EXTERNAL FUNCTION SUPPLIED BY THE USER
C    A = LOWER LIMIT OF INTEGRATION
C    B = UPPER LIMIT OF INTEGRATION
C    AERR = DESIRED ABSOLUTE ERROR LIMIT
C    RERR = DESIRED RELATIVE ERROR LIMIT
C
C  OUTPUT PARAMETERS:
C    DCADRE = THE ESTIMATED INTEGRAL
C    ERROR = THE ESTIMATED ERROR BOUND
C    IER = ERROR FLAG (SEE RESTRICTIONS ABOVE)
C          0   = NO PROBLEMS
C          132 = ANY OF THE CMLIB ERROR CODES 1-5
C                (INDICATES PROBLEMS IN THE INTEGRATION)
C          133 = CMLIB ERROR CODE 6
C                (INDICATES PROBLEMS IN THE INPUT PARAMETERS)
C
C
      IMPLICIT DOUBLE PRECISION (A - H, O - Z)
      IMPLICIT INTEGER (I-N)
      EXTERNAL F
      DIMENSION IWORK (100), WORK (400)
      DATA LIMIT, LENW /100, 400/
C
C
C  SIMPLY CALL THE CMLIB ROUTINE WITH THE PROPER PARAMETERS
C
      CALL DQAGS (F, A, B, AERR, RERR, RESULT, ERROR, NEVAL, IER,
     *            LIMIT, LENW, LAST, IWORK, WORK)
      IF (IER .GE. 1 .AND. IER .LE. 5) IER = 132
      IF (IER .EQ. 6) IER = 133
      DCADRE = RESULT
      RETURN
      END
