*+
*KWIC fft2c.f
*
*$Id: fft2c.f,v 1.1 2004/03/12 16:33:11 dtn Exp $
*
*Revisions:
*   93-Nov-10[T. Gaetz]
*      . make A, IWK into assumed size arrays; allows 
*        bounds-checking compiler option to be used.
*-

      SUBROUTINE FFT2C (A, M, IWK)
C   /******************************************/
C    *
C    *    BAUER ASSOCIATES, INC.
C    *
C    *    FFT2C FORTRAN
C    *    WRITTEN BY P. GLENN (617) 235-8775
C    *            ON 01/27/88
C    *
C   /******************************************/
C
C  PURPOSE: ALLOW A CALL TO THE IMSL ROUTINE FFT2C TO BE REPLACED
C           (TRANSPARENTLY TO THE USER) BY A CALL TO CMLIB ROUTINES,
C           SO THAT THE IMSL LIBRARY NEED NOT BE AVAILABLE
C
C  RESTRICTIONS: THIS SECOND-CUT VERSION WORKS TO DOUBLE PRECISION,
C                AND UTILIZES THE VERSION OF THE CMLIB ROUTINES WHICH
C                HAVE BEEN MODIFIED BY HARRY FRISCH AT NASA GSFC TO
C                WORK IN DOUBLE PRECISION.
C                ALSO, M MUST BE LESS THAN OR EQUAL TO 9, SO
C                THAT THE TOTAL DIMENSION IS A POWER OF TWO LESS THAN
C                OR EQUAL TO 512.
C
C  INPUT PARAMETERS:
C    A (COMPLEX*16) = INPUT ARRAY TO BE TRANSFORMED
C                     (A IS TREATED INTERNALLY AS 2-D R*8
C    M (INTEGER) = LOG-BASE-2 OF THE DIMENSION OF A
C    IWK (INTEGER) = WORK ARRAY, WHICH IS NOT USED IN THIS VERSION
C
C  OUTPUT PARAMETERS:
C
C    FOR J=0,...,N-1
C        A(J+1)=the sum from K=0,...,N-1 of
C                A(K+1)*EXP(I*J*K*2*PI/N)
C
C
      IMPLICIT DOUBLE PRECISION (A - H, O - Z)
      IMPLICIT INTEGER (I-N)
      DIMENSION A (2, *)
      DIMENSION WSAVE (2063), NARR (9), IWK (*)
      DATA NARR /2, 4, 8, 16, 32, 64, 128, 256, 512/
      DATA MOLD /-1/
C
C
C  CHECK FOR LEGAL RANGE OF M
C
      IF (M .LT. 1 .OR. M .GT. 9) RETURN
C
C  DEFINE THE DIMENSION, N
C
      N = NARR (M)
C
C  INITIALIZE THE CMLIB ROUTINE IF M HAS CHANGED
C
      IF (M .EQ. MOLD) GO TO 200
      MOLD = M
      CALL CFFTI (N, WSAVE)
C
C  DO THE FOURIER TRANSFORM AND RETURN
C
200   CALL CFFTB (N, A, WSAVE)
      RETURN
      END
