/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>
#include <stdio.h>

#include "debug/debug.h"

static char
hex[]=
{
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  'A', 'B', 'C', 'D', 'E', 'F'
};

/*
  PROCEDURE
  hexdump - print out memory as hexidecimals

  DESCRIPTION
  @code{hexdump} will pretty print out a section of memory in base 16.

  AUTHOR
  Diab Jerius
*/  

void
hexdump(
  FILE *stream,		/* where to print */
  void *buf,		/* the start of memory to dump */
  size_t length		/* the number of bytes to dump */
)
{
  size_t i;
  char *start = buf;

  for (i = 0; i < length; i++, start++)
    if (i % 16)
      fprintf(stream, "%c%c ",
	          hex[(*start & 0xf0) >> 4], hex[(*start & 0x0f)]);
    else
      fprintf(stream, "\n%08lx: %c%c ", start,
	          hex[(*start & 0xf0) >> 4], hex[(*start & 0x0f)]);
  fprintf(stderr,"\n");
}
