/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <string.h>

#include "file/file.h"

#define DIRSEP '/'


char *
basename(
  char *string,		/* the string to process */
  char *suffix		/* an optional suffix to remove.  Set it
			   to @code{NULL} to do nothing */
  )
{ return base_name( string, suffix ) ; }

/*

  PROCEDURE
  base_name - Remove the prefix and optionally a suffix of a string.

  DESCRIPTION
  @code{base_name} removes all but the last level in a path (the file name)
  and, optionally, a suffix.  The former is accomplished by returning a
  pointer to the first character in the filename.  The latter is accomplished
  by writing an end of string character into the string.

  RETURNS
  It returns a pointer to the beginning of the filename.
  
  AUTHOR
  Diab Jerius
 */
char *
base_name(
  char *string,		/* the string to process */
  char *suffix		/* an optional suffix to remove.  Set it
			   to @code{NULL} to do nothing */
  )
{
  char *s;

  /* first remove the suffix, if any */
  if ( suffix )
  {
    size_t len = strlen(string);
    if ( strcmp(suffix, string + len - strlen( suffix ) ) == 0 )
      string[len - strlen( suffix )] = '\0';
  }

  return ( s = strrchr( string, DIRSEP ) ) ? s + 1 : string;
}
