/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>

#include "imagefcts/imagefcts.h"

/* 

   PROCEDURE
   wtvar - determine the unnormalized variance of objects' distance from a given point

   DESCRIPTION
   @code{wtvar} determines the unnormalized variance (from zero) of
   objects' distance from a given point.  It requires a user supplied
   function which extracts an object's position.

   RETURNS
   It returns the variance.

*/

double
wtvar(
  double x,			/* x coordinate of point from which to determine distance */
  void *objs, 			/* pointer to list of objects to process */
  unsigned long n_obj,		/* number of objects in list */
  double tot_wt, 		/*  if non-zero, indicates that objects are
				    weighted */
  size_t s_obj, 		/* size of an object, in bytes */
  double (*get_x)(const void *obj, double *x) /* pointer to function which returns the coordinates
						  of an object as well as its weight */ 
  )
{
  double var = 0.0;
  char *obj_lim;
  char *obj = objs;

  if (tot_wt)
  {
    for (obj_lim = obj + n_obj * s_obj ; obj < obj_lim ; obj += s_obj)
    {
      double xo;
      double wt = get_x(obj, &xo);
      xo -= x;
      var += wt * xo * xo;
    }
  }
  else
  {
    for (obj_lim = obj + n_obj * s_obj ; obj < obj_lim ; obj += s_obj)
    {
      double xo;
      get_x(obj, &xo);
      xo -= x;
      var += xo * xo;
    }
  }

  return var;
}
