/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include "range/range.h"

static char *range_error[] =
{
  "no error\n",
  "out of memory\n",
  "incomplete range\n",
  "number out of bounds\n",
  "not a number\n",
  "negative number\n",
  "overflow of start value\n",
  "non-positive count\n",
  "overflow of end value\n",
  "internal error\n",
  "start greater than end\n",
  "unknown error\n",
};

/*

  PROCEDURE
  range_perror - output an error message associated with a @code{RangeERR}

  DESCRIPTION
  Error messages are returned by the @code{range} routines via an integer
  code of type @code{RangeERR}.  This routine acts much like the system
  @code{perror} routine.  It takes as input the output stream, the error
  code, the range specification passed to the range parsing routines, and
  the @var{where} argument returned by those routines.  It prints an
  error message to the passed output stream.

  AUTHOR
  Diab Jerius
*/
void
range_perror(
  FILE *fout,			/* where to print the error message */
  RangeErr error,		/* which error to print */
  const char* spec,		/* the specification which caused the error */
  long where			/* where in the spec the error ocurred */
)
{
  const char *ptr = spec + where;
  
  if ( error > RangeErr_MAXERR )
    error = RangeErr_MAXERR;

  fputs(spec, fout);
  fputc('\n', fout);

  for (; spec != ptr; spec++)
    fputc((*spec == '\t') ? '\t' : '-', fout);

  fputs( "^\n", fout);
  fputs( range_error[error], fout );
}

