/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>
#include <string.h>

#include "str/str.h"

/*

  PROCEDURE
  str_dtokcnt - count the number of tokens in a string.

  DESCRIPTION
  @code{str_dtokcnt} parses a string in the same manner as
  @code{str_dtoksplit} and returns the number of tokens.  It does not
  alter the string.

  RETURNS
  It returns the number of tokens in the string.

  AUTHOR
  Diab Jerius

*/

size_t
str_dtokcnt(
	  const char *string,		/* string to parse */
	  char *delim			/* the delimiters to use */
	  )
{
  const char	*tok_end;
  size_t ntok = 0;

  tok_end = string;
  do
  {
    tok_end = strpbrk(tok_end, delim);
    ntok++;
  } while (tok_end++);

  return ntok;
}
