/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <string.h>

/*

  PROCEDURE
  str_dtoksplit - split a string into tokens.

  DESCRIPTION
  This routine splits a string into a series of tokens. Unlike the
  system library function @code{strtok}, each instance of a delimiter
  implies a token.  In @code{strtok}, sequential delimiters are
  collapsed into one.  This function allows one to have empty tokens.
  It fills a caller provided array with pointers to the tokens.  The
  caller may specify a maximum number of tokens to read.  Note that an
  empty string corresponds to a single, empty, token.

  The passed string is modified (end of string characters are inserted 
  where necessary).

  RETURNS
  It returns the number of tokens read.

  AUTHOR
  Diab Jerius

*/

size_t
str_dtoksplit(
	 char *str,		/* the string to split up */
	 char *tok[],		/* the array to stick */
	 const char *delim,	/* the delimiters to split on */
	 int ntok		/* the maximum number of tokens to read */
	 )
{
  char *tok_end;
  size_t n = 0;

  /* first token */
  
  tok_end = str;
  do
  {
    tok[n] = tok_end;
    if ( (tok_end = strpbrk(tok_end, delim)) )
      *tok_end = '\0';
    n++;
  } while (n < ntok && tok_end++);

  return n;
}
