/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <string.h>

#include "str/str.h"

/*

  PROCEDURE
  str_dup - duplicate a string

  DESCRIPTION
  @code{str_dup} makes a duplicate copy of a string, allocating
  space and copying the original into the new space.  If the
  passed pointer is @code{NULL}, it returns @code{NULL}.

  RETURNS
  Upon success, it returns a pointer to the duplicate string.
  The user is responsible for freeing this memory.  Upon failure
  (i.e., out of memory) it returns @code{NULL}

  AUTHOR
  Diab Jerius

*/

char *
str_dup(
  const char *string		/* the string to duplicate */
  )
{
  char *dup = NULL;

  if ( string )
    dup = (char *) malloc( strlen(string) + 1 );

  if ( dup )
    strcpy(dup, string);

  return dup;
}
