/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "str/str.h"

/*
  PROCEDURE
  str_join - concatenate strings with delimiters

  DESCRIPTION
  @code{str_join} concatenates a list of strings, separated by the
  given delimiter.  If any of the passed pointers is @code{NULL}, it
  is ignored.

  RETURNS
  It returns a pointer to a newly allocated buffer containing the the
  concatenated string, @code{NULL} if it couldn't allocate the
  requisite memory.  The caller is responsible for freeing the buffer.

  */
char *
str_join(
  const char *delim,	/* the inter-string delimiter, may be @code{NULL} */
  size_t n,		/* the number of strings */
  ...			/* the strings */
  )
{
  va_list ap;
  char *copy;
  size_t i;
  size_t len = 1;

  va_start( ap, n );
  for ( i = 0 ; i < n ; i++ )
  {
    char *string = (char *) va_arg( ap, char *);
    if ( string )
      len += strlen( string );
  }
  va_end( ap );

  if ( delim )
    len += strlen(delim) * ( n - 1 );

  if ( NULL == ( copy = (char *) malloc( len ) ) )
    return NULL;

  copy[0] = '\0';
  va_start( ap, n );
  for ( i = 0 ; i < n ; i++ )
  {
    char *string = (char *) va_arg( ap, char *);
    if ( NULL == string )
      continue;

    strcat( copy, string );
    if ( delim && i < n - 1 )
      strcat( copy, delim );
  }
  va_end( ap );

  return copy;
}
