/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <ctype.h>

#include "str/str.h"

/*

  PROCEDURE
  str_strim -  removes white space from front of a string

  DESCRIPTION
  @code{str_strim} removes white space from the front of a
  string by shifting the string to the left, starting at the
  first non-white space character.  It works in place,
  altering the passed string.

  RETURNS
  It returns the pointer which it was passed

  AUTHOR
  Diab Jerius

*/
char *
str_trim(
  char *str		/* the string to trim */
  )
{
  char *s, *t;

  if (str && *str)
  {
    /* find start of non-white space */
    for (s = str; isspace(*s); s++);

    /* t is the location to copy to */
    t = str;

    /* move string over */
    while (*s) *t++ = *s++;

    /* seal off end of string */
    *t = '\0';
  }

  return str;
}
