#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include <errno.h>

#include "str/str.h"
#include "debug/debug.h"

#include <testlib/testlib.h>

#define LABEL(extra) (sprintf( label, "%-34s: %s", prefix, extra ), label)


void test_tokbq(
  size_t line,
  char *string,
  const char *delim,
  const char *open_quote,
  const char *close_quote,
  char  escape_char,
  int actions,
  int exp_error,
  ...
  )
{
  char save[1024];
  char label[1024];
  char prefix[1024];

  struct str_tokbqdata tbqd;
  char *buf;

  /* because of the logic below, if no results, just an error, are
     expected, the for loop won't be entered, and if result isn't
     preset to something, str_tokbq will never get called
  */
     
  char *result = 0xDEADBEEF;
  char *expected;

  int error = 99;

  va_list ap;

  va_start( ap, exp_error );

  

  sprintf( prefix, "%-17s: S:%d R:%d E:%d", string,
	   actions & STRTOK_SKIP,
	   actions & STRTOK_RESTORE,
	   actions & STRTOK_ESCAPE
	   );
  
  /* make string r/w */
  strcpy( save, string );
  
  str_tokbq_init( &tbqd );

  buf = save;

  for( expected = (char*) va_arg(ap, char*);
       expected ;
       expected = (char*) va_arg(ap, char*)
       )
  {
    result = str_tokbq( buf, delim, open_quote, close_quote, escape_char,
			actions, &tbqd, &error );
    /* this ain't good */
    if ( NULL == result )
    {
      TEST_LINE(line, LABEL( "PREMATURE NULL" ), 1, 0 );
      break;
    }

    TESTSTRING_LINE(line, LABEL( expected ), result, expected );
    if ( strcmp( result, expected ) )
      fprintf( stderr, ">%s< <- >%s<\n", result, expected );
  }

  if ( result )
  {
    result = str_tokbq(buf, delim, open_quote, close_quote, escape_char,
		       actions, &tbqd, &error );

    TEST_LINE( line, LABEL("NULL"), result, NULL );
  }

  TEST_LINE( line, LABEL("error"), error, exp_error );

  if ( actions & STRTOK_RESTORE )
    TESTSTRING_LINE( line, LABEL("restore"), save, string );


  va_end(ap);

}


#define DEL ","
#define OQ "\"\'<[({"
#define CQ "\"\'>])}"
#define ESC '\\'

int main( int argc, char** argv )
{
  char buf[1024];
  int  action;

  /*******************************************************************/
  START( "str_tokbq" );

  /*#################################################################*/
  /* ideal input: no quotes, no skips */

  strcpy( buf, "a,b,c" );

  /******************************/
  action = 0;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "a", "b", "c", NULL );

  /******************************/
  action = STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "a", "b", "c", NULL );

  /******************************/
  action = STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "a", "b", "c", NULL );

  /******************************/
  action = STRTOK_SKIP;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "a", "b", "c", NULL );


  /*#################################################################*/
  /* ideal input: no quotes, skips */

  strcpy( buf, ",a,b,,c,," );

  /******************************/
  action = 0;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "", "a", "b", "", "c", "", "", NULL );

  /******************************/
  action = STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "", "a", "b", "", "c", "", "", NULL );

  /******************************/
  action = STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "a", "b", "c", NULL );

  /******************************/

  action = STRTOK_SKIP;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "a", "b", "c", NULL );



  /*#################################################################*/
  /* ideal input: quotes, no skips */

  strcpy( buf, "'a',[b],<c>" );

  /******************************/
  action = 0;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "'a'", "[b]", "<c>", NULL );

  /******************************/
  action = STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "'a'", "[b]", "<c>", NULL );

  /******************************/
  action = STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "'a'", "[b]", "<c>", NULL );

  /******************************/
  action = STRTOK_SKIP;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0, "'a'", "[b]", "<c>", NULL );

  /*#################################################################*/
  /* ideal input: quotes, skips */

  strcpy( buf, ",'a',[b],<,c>,[d}>,]," );

  /******************************/
  action = 0;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	     "", "'a'", "[b]", "<,c>", "[d}>,]", "", NULL );

  /******************************/
  action = STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	     "", "'a'", "[b]", "<,c>", "[d}>,]", "", NULL );

  /******************************/
  action = STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "'a'", "[b]", "<,c>", "[d}>,]",  NULL );

  /******************************/
  action = STRTOK_SKIP;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "'a'", "[b]", "<,c>", "[d}>,]", NULL );


  /*#################################################################*/
  /* ideal input: escapes */
  strcpy( buf, "\\,'a',[b],\\\\,c\\',,[d\\]],[e\\\\]" );

  /******************************/
  action = STRTOK_ESCAPE | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "\\,'a'", "[b]", "\\\\", "c\\'", "", "[d\\]]", "[e\\\\]", NULL );


  /******************************/
  action = STRTOK_ESCAPE | STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "\\,'a'", "[b]", "\\\\", "c\\'", "[d\\]]", "[e\\\\]", NULL );

  /*#################################################################*/
  /* ideal input: more escapes */
  strcpy( buf, "\\\\,'a'" );

  /******************************/
  action = STRTOK_ESCAPE | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "\\\\", "'a'", NULL );


  /******************************/
  action = STRTOK_ESCAPE | STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "\\\\", "'a'", NULL );
  
  /*#################################################################*/
  /* ideal input: more escapes */

  strcpy( buf, "\\\\\\,'a'" );

  /******************************/
  action = STRTOK_ESCAPE | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "\\\\\\,'a'", NULL );


  /******************************/
  action = STRTOK_ESCAPE | STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, 0,
	      "\\\\\\,'a'", NULL );

  /*#################################################################*/
  /* broken input: unbalanced quotes */
  strcpy( buf, "'a,[b],,,," );

  /******************************/
  action = STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, EINVAL, NULL );

  /******************************/
  action = STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, EINVAL, NULL );

  /*#################################################################*/
  /* broken input: more unbalanced quotes */
  strcpy( buf, "a,,[b,,," );

  /******************************/
  action = STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, EINVAL, "a", "", NULL );

  /******************************/
  action = STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, EINVAL, "a", NULL );


  /*#################################################################*/
  /* broken input: more unbalanced quotes */
  strcpy( buf, "a,,]b,,," );

  /******************************/
  action = STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, EINVAL, "a", "", NULL );

  /******************************/
  action = STRTOK_SKIP | STRTOK_RESTORE;

  test_tokbq( __LINE__,  buf, DEL, OQ, CQ, ESC, action, EINVAL, "a", NULL );


  /*******************************************************************/
  SUMMARY();
  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;


}
