/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of tests
 *
 * tests is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * tests is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>

#include "str/str.h"
#include "debug/debug.h"

#include <testlib/testlib.h>

#define STRING "now,,is,the,time,for,all,good,men,to,come,,"

/* tokens, skip = 1 */
char *tokens_skip[] =
{ "now", "is", "the", "time", "for", "all", "good", "men", "to", "come" };

#define N_TOKENS_SKIP  ( sizeof( tokens_skip ) / sizeof( char *) )

/* tokens, skip = 0 */
char *tokens_no_skip[] =
  { "now", "", "is", "the", "time", "for", "all", "good", "men", "to", "come", "", "" };

#define N_TOKENS_NO_SKIP  ( sizeof( tokens_no_skip ) / sizeof( char *) )

#define TEST_SETUP							\
  do{									\
    struct str_tokbqdata tpqd;						\
    /* so we have a writeable string, as required by str_tokenize */	\
    string = str_dup( STRING );						\
    if ( NULL == string ) die( "error allocating string\n" );		\
    num = str_tokbqenize( string, &string_argv, ",", "'", "'", '\\', actions, \
			  &tpqd, &error );				\
  }while (0)

#define TEST_TEARDOWN					\
  do{							\
    free( string );					\
    str_tokenize_free( string_argv, actions );		\
  }while (0)
  

int main( int argc, char** argv )
{
  char label[1024];
  int i, num = 0;
  int actions;
  char** string_argv=NULL;
  char* string;
  int error;

  START( "str_tokenize" );

  
  /*******************************************************************/
  /* SKIP = 1 RESTORE = 1 */

#undef LABEL
#define LABEL "S:1 R:1"
  

  actions = STRTOK_SKIP | STRTOK_RESTORE;

  TEST_SETUP;

  TEST( LABEL ": num", num, N_TOKENS_SKIP);

  for( i = 0; i < num; i++ )
  {
    sprintf( label, LABEL ": token[%d] = \"%s\"", i, tokens_skip[i] );
    TESTSTRING( label, string_argv[i], tokens_skip[i] );
  }

  /* check that restored string is correct */
  TESTSTRING( LABEL ": restored?", string, STRING );

  TEST_TEARDOWN;

  /*******************************************************************/
  /* SKIP = 1 RESTORE = 0 */

#undef LABEL
#define LABEL "S:1 R:0"


  actions = STRTOK_SKIP;

  TEST_SETUP;

  TEST( LABEL ": num", num, N_TOKENS_SKIP);

  for( i = 0; i < num; i++ )
  {
    sprintf( label, LABEL ": token[%d] = \"%s\"", i, tokens_skip[i] );
    TESTSTRING( label, string_argv[i], tokens_skip[i] );
  }

  /* check that the string is NOT restored */
  TEST( LABEL ": not restored?", strlen(string), 3 );

  TEST_TEARDOWN;

  
  /*******************************************************************/
  /* SKIP = 0 RESTORE = 1 */

#undef LABEL
#define LABEL "S:0 R:1"

  actions = STRTOK_RESTORE;

  TEST_SETUP;

  TEST( LABEL ": num", num, N_TOKENS_NO_SKIP);

  for( i = 0; i < num; i++ )
  {
    sprintf( label, LABEL ": token[%d] = \"%s\"", i, tokens_no_skip[i] );
    TESTSTRING( label, string_argv[i], tokens_no_skip[i] );
  }

  /* check that restored string is correct */
  TESTSTRING( LABEL ": restored?", string, STRING );

  TEST_TEARDOWN;


  /*******************************************************************/
  SUMMARY();
  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}


