/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <stddef.h>

#include "str/str.h"

/*

   PROCEDURE
   unescape - replace escaped characters with their true values

   DESCRIPTION
   @code{unescape} replaces escaped characters with their true values.
   The escape prefix is the character @samp{\}.  It recognizes the
   following special characters: @samp{\t}, @samp{\n}.  All other
   escaped characters are replaced by the character (i.e., @samp{\g}
   is turned into @samp{g}).  It makes the changes in-place.

   RETURNS
   It returns a pointer to the original string upon success, @code{NULL}
   if the escape prefix occurred without a character to escape.

   AUTHOR
   Diab Jerius
   
*/
char *
unescape(
  char *string	/* the string to unescape */
)
{
  char *sptr, *dptr;

  sptr = dptr = string;

  for (; *sptr ; sptr++)
    if (*sptr == '\\')
    {
      if (*++sptr == '\0')
	return NULL;
      if      (*sptr == 'n')
	*dptr++ = '\n';
      else if (*sptr == 't')
	*dptr++ = '\t';
      else
	*dptr++ = *sptr;
    }
    else
      *dptr++ = *sptr;

  *dptr = '\0';
  return string;
}



#ifdef TEST

int
main(int argc, char *argv[])
{
  char buf[1024];
  char *string;

  fputs("> ", stdout);
  while (fgets(buf, 1024, stdin))
  {
    if (buf[strlen(buf) - 1] == '\n')
      buf[strlen(buf) - 1] = '\0';

    if ( ( string = unescape(buf)) == NULL)
      puts("error");
    else
      puts(string);

    fputs("> ", stdout);
  }
  fputc('\n', stdout);
  return 0;
}


#endif /* TEST */
