/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <stddef.h>

#include "str/str.h"

/*

   PROCEDURE
   unquote - remove quotes from a string

   DESCRIPTION
   @code{unquote} removes pairs of quotes from a string.  The first
   quote need not be at the beginning of the string.  It makes the
   changes in-place.  For example, @samp{foo" "bar} is turned into
   @samp{foo bar}, just like in a UNIX shell.  It can deal with either
   single (forward or backward) or double quotes, and understands
   escaped characters (with @samp{\} as the escape prefix).

   RETURNS
   It returns a pointer to the original string upon success, @code{NULL}
   if the quotes were unbalanced or an escape prefix occurred without
   a character to escape.

   AUTHOR
   Diab Jerius
*/
char *
unquote(
  char *string	/* the string to unquote */
)
{
  char *sptr, *dptr;

  sptr = dptr = string;

  for (; *sptr ; sptr++)
    if (*sptr == '\\')
    {
      *dptr++ = *sptr++;
      if (*sptr == '\0')
	return NULL;
      *dptr++ = *sptr;
    }
    else if (*sptr == '"' || *sptr == '\'' || *sptr == '`')
    {
      /* found a quote */
      char quote = *sptr++;

      /* find its partner */
      while ( *sptr && *sptr != quote )
	if (*sptr == '\\' && *++sptr == '\0')
	  break;
	else
	  *dptr++ = *sptr++;

      /* only a single quote. bogus dude! */
      if (*sptr != quote)
	return NULL;
    }
    else
      *dptr++ = *sptr;

  *dptr = '\0';
  return string;
}



#ifdef TEST

int
main(int argc, char *argv[])
{
  char buf[1024];
  char *string;

  fputs("> ", stdout);
  while (fgets(buf, 1024, stdin))
  {
    if (buf[strlen(buf) - 1] == '\n')
      buf[strlen(buf) - 1] = '\0';

    if ( ( string = unquote(buf)) == NULL)
      puts("error");
    else
      puts(string);

    fputs("> ", stdout);
  }
  fputc('\n', stdout);
  return 0;
}


#endif /* TEST */
