/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include "str/str.h"
#include "units/units.h"

/*
  PROCEDURE
  units_cvt - get the conversion factor from one unit to another

  DESCRIPTION
  @code{units_cvt} returns a conversion factor from one unit to
  another.  Both units must be in the passed @code{UnitConvertList}.

  RETURNS
  Upon success (i.e., the units are in the @code{UnitConvertList}),
  it returns the conversion factor.  Upon error, it returns
  @samp{0.0}.

  AUTHOR
  Diab Jerius

*/
  

double
units_cvt(
  const UnitsList *list, 		/* the Units list */
  int from,			/* the unit id of the source unit */
  int to 			/* the unit id of the destination unit
				 */
  )
{
  if ( from < 0 || from >= list->convert->nconv ||
       to < 0 ||  to >= list->convert->nconv )
    return 0;

  return list->convert->cvt[from].cnvfact / list->convert->cvt[to].cnvfact;
}
