/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>

#include <testlib/testlib.h>

#include <str/str.h>

#include <units/units.h>
#include <units/units_linear.h>
#include <units/units_angular.h>
#include <units/units_time.h>
#include <units/units_energy.h>

void	check_map( UnitsList *list );

int main( int argc, char *argv[] )
{

  START( "units test" );

  /* check for consistency in default maps */
  
  check_map( &UnitsLinear_def );
  check_map( &UnitsAngular_def );
  check_map( &UnitsTime_def );
  check_map( &UnitsEnergy_def );

  TEST_FP( "m -> mm", units_cvt( &UnitsLinear_def, UNIT_meter,
			      UNIT_millimeter ), 1000.0 );
  
  TEST_FP( "m -> um", units_cvt( &UnitsLinear_def, UNIT_meter,
			      UNIT_micrometer ), 1e6 );

  TEST_FP( "deg -> arcminute", units_cvt( &UnitsAngular_def, UNIT_degree,
			      UNIT_arcminute ), 60.0 );

  SUMMARY();

  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;

}


void
check_map( UnitsList *list )
{
  TokList *map;
  int i;

  map = list->map;
  for ( i = 0 ; i < map->ntoks ; i++ )
  {
    UnitVal uv;
    char buf[1024];
    char desc[1024];
    sprintf( buf, "%f %s", 3.33, map->toks[i].name );
    sprintf( desc, "test parse of `%s'", buf );
    TEST( desc, units_parse( buf, &uv, list), UNITS_OK );
    sprintf( desc, "test value of `%s'", buf );
    TEST( desc, uv.value, 3.33 );
    sprintf( desc, "test id of `%s'", buf );
    TEST( desc, uv.unit, map->toks[i].id );
  }
}
