// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "colselect.h"

namespace suplib {

  /*!
   * \ingroup colselect
   * \brief handles Perl regular expression matching.
   *
   *
   * \param str the string
   * \param pattern the regular expression pattern with which to compare
   *
   * \exception Exception errors related to pattern matching
   *
   * match uses the pcre library's functionality to compare str with
   * pattern.
   *
   * \return true is str matches pattern, false if not.
   */
bool 
match( 
  const string& str, 
  const string& pattern
  ) throw ( Exception ) {

  pcre* re;
  pcre_extra* pe = NULL;
  const char* error;
  int options = 0;
  int erroffset;
  int rc;
  int* ovector = NULL;
  int ovecsize = 0;
  
  if ( NULL == (re = pcre_compile( pattern.c_str( ),
				   options,
				   &error,
				   &erroffset,
				   NULL )) )
    throw( Exception( error ) );
  
  if ( 0 > (rc = pcre_exec( re, 
			    pe, 
			    str.c_str( ),
			    str.size( ),
			    0,
			    options,
			    ovector,
			    ovecsize
			    )) ) {
    switch(rc) {
    case PCRE_ERROR_NOMATCH: 
      break;
      
    case PCRE_ERROR_NULL:
      throw( Exception( "NULL pattern or subject string." ) );
      
    case PCRE_ERROR_BADOPTION:
      throw( Exception( "Bad option." ) );
      
    case PCRE_ERROR_BADMAGIC:
      throw( Exception( "Bad magic number(possible junk regex pointer?)" ) );
      
    case PCRE_ERROR_UNKNOWN_NODE:
      throw( Exception( "Was your compiled pattern overwritten?" ) ); 
      
    case PCRE_ERROR_NOMEMORY:  
      throw( Exception( "Out of memory." ) ); 
      
    default: 
      throw( Exception( "Unforeseen matching error." ) ) ;
      
    }
    
    return false;
    
  }
  
  return true;
  
}
  
} // namespace suplib
