// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <string>

using namespace std;

#include "str.h"

/*!
 * \ingroup str
 * \brief determine if the string is a comment.
 *
 *
 * \param str the string upon which to operate
 *
 * iscomment scans str to determine if the first character following
 * all the ignore characters is a comment character. It compares the 
 * position of the first non-ignore character with the position of the
 * first comment character. If they are the same and occur before the
 * end of the string, it returns true. Otherwise it returns false.
 *
 * \return It returns true if line is a comment.
 */
bool 
suplib::iscomment( 
  const string& str, 
  const string& ignore, 
  const string& comment 
  ) 
{
  size_t comment_pos = str.find_first_of( comment );
  if ( str.npos != comment_pos && comment_pos == str.find_first_not_of( ignore ) )
    return true;
  else
    return false;
}
