// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <errno.h>
#include <cstdlib>

using namespace std;

#include "str.h"

/*!
 * \ingroup str
 * \brief convert string to double-precision number
 *
 * \param txt the string upon which to operate
 *
 * str2d() converts the initial portion of the string pointed 
 * to by txt to type double representation.  It throws an exception,
 * of type Exception, if txt is not a legitimate double precision number.
 *
 * \return It returns the double precision number
 */
double suplib::str2d( const char* txt ) throw( Exception ) {

  // Field is empty
  if ( NULL == txt || '\0' == *txt )
    throw Exception( "suplib::str2d( ) : Empty field" );

  char* ptr=NULL;

  double result = strtod( txt, &ptr );

  if ( ERANGE == errno )
    throw Exception( "suplib::str2d( " + string( txt ) +
		     ") : Variable overflow" );

  if ( '\0' != *ptr )
    throw Exception( "suplib::str2d( " + string( txt ) +
		     " ) Not a number, problem at `" + string( ptr ) + "'" );

  return result;

}
