// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <limits.h>
#include "str.h"

/*!
 * \ingroup str
 * \brief convert string to integer number
 *
 * \param txt the string upon which to operate
 *
 * str2i() converts the initial portion of the string pointed 
 * to by txt to type integer representation.  It throws an exception,
 * of type Exception, if txt is not a legitimate integer.
 *
 * \return It returns the integer.
 */
int suplib::str2i( const char* txt, int base ) throw( Exception ) {

  try {

    long lresult = str2l( txt, base );

    if ( lresult < INT_MIN || INT_MAX < lresult )
      throw Exception( "suplib::str2i( " + string( txt ) + "int ) "
		       "Integer out of range" );

    return (int) lresult;

  } catch( Exception& E ) {

    throw;

  }

}
