// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <errno.h>
#include <cstdlib>

#include <limits.h>

#include "str.h"

/*!
 * \ingroup str
 * \brief convert string to long number
 *
 * \param txt the string upon which to operate
 *
 * str2l() converts the initial portion of the string pointed 
 * to by txt to type long representation.  It throws an exception,
 * of type Exception, if txt is not a legitimate long.
 *
 * \return It returns the long.
 */
long suplib::str2l( const char* txt, int base ) throw( Exception ) {

  // The field cannot be empty. The user must enter a value.
  if ( NULL == txt || '\0' == *txt )
    throw Exception( "suplib::strol( ) : Empty field" );

  char* ptr = NULL;
  long result = ::strtol( txt, &ptr, base );
  
  if ( (result == LONG_MAX || result == LONG_MIN) && errno == ERANGE )
    throw Exception( "suplib::str2l( " + string( txt ) +
		     ", int ) : Possibly under/over flow" );

  if ( '\0' != *ptr )
    throw Exception( "suplib::str2l( " + string( txt ) +
		     ", int ) : Not a number, problem starting at `" +
		     string( ptr ) + "'" );

  return result;

}
