// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <sys/types.h>
#include <unistd.h>

#include <fstream>
#include <iostream>
#include <string>

using namespace std;

#include "io.h"
#include <testlib/testlib.h>

#define TESTSTR1  "  now, is, the, time  "
#define TESTSTR2  'x'
#define TESTSTR2S "x"
#define TESTSTR3  "for"
#define TESTSTR4  "FOR"
#define TESTSTR5  "now, is, the, time  "
#define TESTSTR6  "  now, is, the, time"
#define TESTSTR7  "now, is, the, time"
#define TESTSTR10 "now,is,the,time,\"for,all,good\", men, to come"
#define TESTSTR11 "now"
#define TESTSTR12 " to come"
#define TESTSTR13 "# This is a comment"
#define TESTSTR14 "                            "
#define TESTSTR15 "/data/hxds3/pipe/data/950504/lin409y0z0.pha"
#define TESTSTR16 "10.3"
#define TESTSTR17 "10.3e4"
#define TESTSTR18 "1234"
#define TESTSTR19 "123456789123412345678912341234567891234"
#define TESTSTR20 "/data/hxds3/pipe/data/950504/lin409y0z0.pha\\"
#define TESTSTR21 "/data/hxds3/pipe`/data/950504/lin409y0z0.pha"


#ifdef VERBOSE

# define WHAT(GOT,EXPECTED) \
	 cerr << "return:  '" << GOT << "'" << endl; \
 	 cerr << "expect:  '" << EXPECTED << "'" << endl;

#else
# define WHAT(GOT,EXPECTED)

#endif // VERBOSE


void test_getline( void ) {

  string s;
  string a;
  char file[20];
  sprintf( file, "tmp-test.%d", getpid() );

  {
    ofstream ofs( file );
    ofs << TESTSTR1;
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_PHYS );
    s = TESTSTR1;
    WHAT( s, a );
    TEST( "getrecord(), physical", a == s, 1 );
  }
  unlink(file);

  {
    ofstream ofs( file );
    ofs << TESTSTR1 << "\n";
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_PHYS | suplib::STRIP );
    s = TESTSTR6;
    WHAT( s, a );
    TEST( "getrecord(), physical|strip", a == s, 1 );
  }
  unlink(file);

  {
    ofstream ofs( file );
    ofs << TESTSTR1 << "\\  \n" << TESTSTR1 << "\\  \n" << TESTSTR1;
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_LOGICAL );
    s  = TESTSTR1;
    s += TESTSTR1;
    s += TESTSTR1;
    WHAT( s, a );
    TEST( "getrecord(), logical", a == s, 1 );
  }
  unlink(file);

  {
    ofstream ofs(file);
    ofs << TESTSTR1 << "\\  \n" << TESTSTR1 << "\\  \n" << TESTSTR1;
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_LOGICAL | suplib::CLEAN );
    s = TESTSTR1;
    s += "\\\n";
    s += TESTSTR1;
    s += "\\\n";
    s += TESTSTR1;
    WHAT( s, a );
    TEST( "getrecord(), logical|clean", a == s, 1 );
  }
  unlink(file);
    
  {
    ofstream ofs( file );
    ofs << TESTSTR1 << "\n";
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_LOGICAL | suplib::STRIP );
    s = TESTSTR6;
    WHAT( s, a );
    TEST( "getrecord(), logical|strip, no cont.", a == s, 1 );
  }
  unlink(file);

  {
    ofstream ofs( file );
    ofs << TESTSTR1 << "\\  \n" << TESTSTR1 << "\\  \n" << TESTSTR1;
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_LOGICAL | suplib::STRIP);
    s = TESTSTR1;
    s += TESTSTR1;
    s += TESTSTR6;
    WHAT( s, a );
    TEST( "getrecord(), logical|strip, cont.", a == s, 1 );
  }
  unlink(file);

  {
    ofstream ofs(file);
    ofs << TESTSTR1 << "\\  \n" << TESTSTR1 << "\\  \n" << TESTSTR1;
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_LOGICAL | suplib::STRIP | suplib::CLEAN );
    s  = TESTSTR1;
    s += "\\\n";
    s += TESTSTR1;
    s += "\\\n";
    s += TESTSTR6;
    WHAT( s, a );
    TEST( "getrecord(), logical|strip|clean", a == s, 1 );
  }
  unlink(file);

  {
    ofstream ofs(file);
    ofs << TESTSTR1 << "-  \n" << TESTSTR1 << "-  \n" << TESTSTR1;
    ofs.close( );

    ifstream ifs( file );
    suplib::getrecord( ifs, a, suplib::READ_LOGICAL, '\n', '-' );
    s = TESTSTR1;
    s += TESTSTR1;
    s += TESTSTR1;
    WHAT( s, a );
    TEST( "getrecord(), logical, `-' is cont. char", a == s, 1 );
  }
  unlink(file);

  {
    ofstream ofs(file, ios::out); 

#define TESTSTR1_0 "#This file contains an erroneous lower limit for the integer"
#define TESTSTR1_1 "junk,i,a,1,3,4,\"Level of debugging messages to be displayed "
#define TESTSTR1_2 "now is the time for all\""
#define TESTSTR1_3 "verbose,i,a,1,-999999999999999999999999999,4,\"Level of debugging messages to be displayed\""
#define TESTSTR1_4 "mode,s,h,\"q\",,,\"mode\""

    ofs << TESTSTR1_0 << "  \n";
    ofs << TESTSTR1_1 << "\\" << "  \n";
    ofs << TESTSTR1_2 << "  \n";
    ofs << TESTSTR1_3 << "  \n";
    ofs << TESTSTR1_4 << "  \n";
    ofs.close( );

    ifstream ifs( file );
    int count = 0;
    while (suplib::getrecord( ifs, s, suplib::READ_LOGICAL | suplib::STRIP  )) {
      switch( count ) {
      case 0:
	a = TESTSTR1_0;
	WHAT(s,a);
	TEST( "getrecord(), logical|strip (0)", a, s  );
	break;
      case 1:
	a = TESTSTR1_1;
	a += TESTSTR1_2;
	WHAT(s,a);
	TEST( "getrecord(), logical|strip (1)", a, s );
	break;
      case 2:
	a = TESTSTR1_3;
	WHAT(s,a);
	TEST( "getrecord(), logical|strip (2)", a, s );
	break;
      case 3:
	a = TESTSTR1_4;
	WHAT(s,a);
	TEST( "getrecord(), logical|strip (3)", a, s );
	break;
      }
      count++;
    }
  }


  {
    ofstream ofs(file, ios::out); 

    ofs << TESTSTR1_0 << "  \n";
    ofs << TESTSTR1_1 << "\\" << "  \n";
    ofs << TESTSTR1_2 << "  \n";
    ofs << TESTSTR1_3 << "  \n";
    ofs << TESTSTR1_4 << "  ";
    ofs.close( );

    ifstream ifs( file );
    int count = 0;
    while (suplib::getrecord( ifs, s, suplib::READ_LOGICAL | suplib::RAW  )) {
      switch( count ) {
      case 0:
	a = TESTSTR1_0;
	a += "  \n";
	WHAT(s,a);
	TEST( "getrecord(), logical|raw (0)", a, s  );
	break;
      case 1:
	a = TESTSTR1_1;
	a +=  "\\" ;
	a += "  \n";
	a += TESTSTR1_2;
	a += "  \n";
	WHAT(s,a);
	TEST( "getrecord(), logical|raw (1)", a, s );
	break;
      case 2:
	a = TESTSTR1_3;
	a += "  \n";
	WHAT(s,a);
	TEST( "getrecord(), logical|raw (2)", a, s );
	break;
      case 3:
	a = TESTSTR1_4;
	a += "  \n";
	WHAT(s,a);
	TEST( "getrecord(), logical|raw (3)", a, s );
	break;
      }
      count++;
    }
  }


  unlink(file);

  return;

}

int main( void ) {
  
  START( "suplib::io" );

  test_getline( );

  SUMMARY( );

  return FAILED( );
}

