// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <limits>
#include <list>
#include <string>
#include <vector>

using namespace std;

#include "str.h"
#include <testlib/testlib.h>


#define TESTSTR22  "now,is,the,time"
#define TESTSTR23  " now, is, the, time "
#define TESTSTR24  "one\ttwo\t\tfour"
const char *tokens_22[] = { "now", "is", "the", "time" };
const char *tokens_24_skip[] = { "one", "two", "four" };
const char *tokens_24_noskip[] = { "one", "two", "", "four" };
const char *strlist[] = { "1.1", "-3.0032", "sdaf" };
double dexpected[] = { 1.1, -3.0032, 0 };
double fexpected[] = { 1.1, -3.0032, 0 };
double iexpected[] = { 0, 0, 0 };
double lexpected[] = { 0, 0, 0 };

void
test_str2d( void )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    double result;
    try { 
      result = suplib::str2d( strlist[i] );
    } catch ( Exception& E ) {
      result = 0;
    }
    TEST( "str2d", dexpected[j] == result, 1 );

    j++;
  }
}

void
test_str2f( void )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    float result;
    try { 
      result = suplib::str2f( strlist[i] );
    } catch ( Exception E ) {
      result = 0;
    }
    TEST( "str2f", fexpected[j] - result < 100*FLT_EPSILON, 1 );

    j++;
  }
}

void
test_str2i( void )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    int result;
    try {
      result = suplib::str2i( strlist[i] );
    } catch ( Exception E ) {
      result = 0;
    }
    TEST( "str2i", iexpected[j] == result, 1 );

    j++;
  }
}

void
test_str2l( void )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    long result;
    try {
      result = suplib::str2l( strlist[i] );
    } catch ( Exception E ) {
      result = 0;
    }
    TEST( "str2l", lexpected[j] == result, 1 );

    j++;
  }
}

void
test_str2ul( void )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    long result;
    try {
      result = suplib::str2ul( strlist[i] );
    } catch ( Exception E ) {
      result = 0;
    }
    TEST( "str2ul", lexpected[j] == result, 1 );

    j++;
  }
}

void
fill_list( vector<string> &l, const char* str[], int n )
{
  for( int i = 0 ; i < n ; i++ )
    l.push_back( str[i] );
}

void
test_tok( void )
{
  vector <string> expected;
  vector <string> tokens;

  // should be able to use fancy stuff to do this, but don't know how
  fill_list( expected, tokens_22, 4 );

  suplib::tok( tokens, TESTSTR22, ",");
  TEST( "tok(), no spaces", expected == tokens, 1 );

  tokens.erase(tokens.begin(), tokens.end());
  suplib::tok( tokens, TESTSTR23, ", ");
  TEST( "tok(), spaces", expected == tokens, 1 );

  expected.erase(expected.begin(), expected.end());
  fill_list( expected, tokens_24_skip, 3 );

  tokens.erase(tokens.begin(), tokens.end());
  suplib::tok( tokens, TESTSTR24, "\t\n");
  TEST( "tok(), tabs w/ skipping", expected == tokens, 1 );

  expected.erase(expected.begin(), expected.end());
  fill_list( expected, tokens_24_noskip, 4 );

  tokens.erase(tokens.begin(), tokens.end());
  suplib::tok( tokens, TESTSTR24, "\t\n", false);

  TEST( "tok(), tabs w/o skipping", expected == tokens, 1 );

}

void
test_trim( void )
{
  string s1 = "abcde";
  string s2 = s1;

  TEST( "trim, no leading blanks", suplib::trim(s2), s1 );

  s2 = " \t" + s1;
  TEST( "trim, leading blanks", suplib::trim(s2), s1 );


}

void
test_prune( void )
{
  string s1 = "abcde";
  string s2 = s1;

  TEST( "prune, no blanks", suplib::prune(s2), s1 );

  s2 = " \t" + s1 + " \t";
  TEST( "prune, leading and trailing blanks", suplib::prune(s2), s1 );

  s2 = s1 + " \t";
  TEST( "prune, trailing blanks", suplib::prune(s2), s1 );

  s2 = " \t" + s1;
  TEST( "prune, leading blanks", suplib::prune(s2), s1 );

}

void test_iscomment( void ) {

  string line( "nnn" );

  TEST( "iscomment, no sharp", suplib::iscomment( line ), false );
  line = "# comment";
  TEST( "iscomment, plain comment", suplib::iscomment( line ), true );
  line = " \t #comment";
  TEST( "iscomment, ws prefaced comment", suplib::iscomment( line ), true );
  line = " \t #";
  TEST( "iscomment, ws prefaced comment", suplib::iscomment( line ), true );
  line = " \t a";
  TEST( "iscomment, ws prefaced comment", suplib::iscomment( line ), false );
  line = " \t ";
  TEST( "iscomment, ws prefaced comment", suplib::iscomment( line ), false );

}


int main( void ) {
  
  START( "suplib::str" );

  test_iscomment( );

  test_tok( );

  test_trim( );

  test_prune( );

  test_str2d( );

  test_str2f( );

  test_str2i( );

  test_str2l( );

  test_str2ul( );

  SUMMARY( );

  return FAILED( );
}

