/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of testlib
 *
 * testlib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * testlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

/*

  DO NOT WRAP LINES!!!
  different platforms give different __LINE__ results for wrapped
  macro invocations.

*/
#include <float.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <testlib/testlib.h>

int main( int argc, char *argv[] )
{
  int expected_failures = 11;

  START("Testing Test");

  {
    int a = 3;
    int b = 2;

    TEST("TEST, PASSED",  1, 1);
    TEST("TEST, PASSED",  a, a);
    TEST("TEST, **FAILED**", 1, 0);
    TEST("TEST, **FAILED**", a, b);

    TEST_RUN("TEST_RUN, PASSED", a = 4, a, 4 );
    TEST_RUN("TEST_RUN, **FAILED**", a = 3, a, 4 );

  }
  
  {
    char *a = "foo";
    char *b = "bar";

    TESTSTRING( "TESTSTRING, PASSED", a, a );
    TESTSTRING( "TESTSTRING, **FAILED**", a, b );
  }

  {
    double d = 22.2;
    double e = 22.3;

    TEST_FP_TOL( "TEST_FP_TOL, **FAILED**", d, e, 0.001 );
    TEST_FP( "TEST_FP, exact, PASSED", d, d );

    TEST_FPABS_TOL( "TEST_FPABS_TOL, **FAILED**", d, e, .01);
    TEST_FPABS_TOL( "TEST_FPABS_TOL, PASSED", d, e, .15);
    TEST_FPABS( "TEST_FPABS, *FAILED*", d, e );
    TEST_FPABS( "TEST_FPABS, exact, PASSED", d, d );


    d = e + 20 * DBL_EPSILON;
    TEST_FP( "TEST_FP, fuzzy, PASSED", d, e );

    TEST_FP_TOL( "TEST_FP_TOL, exact, PASSED", d, d, 0.0 );

    d = 22.2;
    e = 22.3;
    TEST_FP_TOL( "TEST_FP_TOL, **FAILED**", d, e, 0.001 );
    TEST_FP_TOL( "TEST_FP_TOL, fuzzy, PASSED", d, e, 0.01 );

    TEST_RUN_FP( "TEST_RUN_FP, exact, PASSED", double g = d, g, d );
    TEST_RUN_FP( "TEST_RUN_FP, fuzzy, PASSED", double g = d + 20 * DBL_EPSILON, g, d );
    
    TEST_RUN_FP( "TEST_RUN_FP, **FAILED**", double g = d * (1.0 + 200 * DBL_EPSILON ), g, d );
    
    TEST_RUN_FP_TOL( "TEST_RUN_FP_TOL, exact, PASSED", double g = d, g, d, 1 );
    TEST_RUN_FP_TOL( "TEST_RUN_FP_TOL, fuzzy, PASSED", double g = d + 10 * DBL_EPSILON, g, d, 20 * DBL_EPSILON );
    
    TEST_RUN_FP_TOL( "TEST_RUN_FP_TOL, **FAILED**", double g = d + 200 * DBL_EPSILON, g, d, 200 * DBL_EPSILON / d );
  }


  TEST_LINE( 33, "TEST_LINE", 1, 1 );

  TEST_FAIL( "TEST_FAIL, **FAILED**" );
  TEST_PASS( "TEST_PASS, PASSED" );


  SUMMARY();

  printf( "Expect %d failures\n", expected_failures );

  return FAILED() != expected_failures ? EXIT_FAILURE : EXIT_SUCCESS;
}
