/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of tparam
 *
 * tparam is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * tparam is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "tparam.h"

paramfile
tparamopen(char *filename, char **argv, int argc, char *mode)
{
  paramfile pf;
  tf_enter("tparamopen");
  if (NULL == (pf = paramopen(filename, argv, argc, mode) ) )
  {
    if (filename)
      tf_exit(ExitERR_misc, "error opening parameter file `%s'",
				    filename);
    else
      tf_exit(ExitERR_misc, "error opening parameter file");
  }
  tf_leave();
  return pf;
}

#define TPGET(type,suffix) \
  type tpget##suffix(paramfile pf, char *pname) \
  { \
    tf_enter("tpget" #suffix); \
    if (! paccess(pf, pname) ) \
    { \
      tf_exit(ExitERR_misc, "unable to access parameter %s",pname); \
    } \
    tf_leave(); \
    return pget##suffix(pf, pname); \
  }

TPGET(short,s)
TPGET(int,i)
TPGET(double,d)
TPGET(float,f)
TPGET(int,b)

char *
tpgetstr(paramfile pf, char *pname, char *val, int nchars,
	 int abort_if_empty)
{

  tf_enter("tpgetstr");

  /* check if parameter is really there */

  if (! paccess(pf, pname) )
    tf_exit(ExitERR_misc, "unable to access parameter %s",pname);

  pgetstr(pf, pname, val, nchars);

  if (abort_if_empty && val[0] == '\0')
    tf_exit(ExitERR_misc, "must specify %s", pname);

  tf_leave();
  return val;
}
