#============================================================================
# doxygen.mk - generate PDF and HTML output from doxygen input

# prereq: create_am_macros.mk
#         doxygen.ac

# The caller must define
#  DOC_DIR - relative path from top_builddir to builddir



include $(top_srcdir)/snippets/dx_doxygen.am

local_data =
local_data_deps =

##################
# html

if    DX_COND_html


#-----------------------------------
# distribution

# depend upon doxygen-run as there's no specific target for HTML
local_data_deps		+= doxygen-run

# what to distribute
EXTRA_DIST		+= $(DX_DOCDIR)/html

#-----------------------------------
# installation

include $(top_srcdir)/snippets/install_dir.mk

# depend upon doxygen-run as there's no specific target for HTML
local_data              += doxy-html

src_htmldir		= $(DX_DOCDIR)/html
dst_htmldir		= $(datadir)/doc/$(PACKAGE)/html

install-doxy-html: doxygen-run
	@$(NORMAL_INSTALL)
	src_dir=$(src_htmldir); dst_dir=$(dst_htmldir); $(install_dir)

uninstall-doxy-html:
	@$(NORMAL_UNINSTALL)
	src_dir=$(src_htmldir); dst_dir=$(dst_htmldir); $(uninstall_dir)

endif DX_COND_html


##################
# pdf

if    DX_COND_pdf

#-----------------------------------
# distribution

local_data_deps		+= doxygen-pdf

# what to distribute
EXTRA_DIST		+= $(DX_DOCDIR)/$(PACKAGE).pdf

#-----------------------------------
# installation

pdf_data                = $(DX_DOCDIR)/$(PACKAGE).pdf
pdfdir                  = $(datadir)/doc/$(PACKAGE)


local_data += doxy-pdf

include $(top_srcdir)/snippets/install_files.mk

install-doxy-pdf: $(pdf)
	@$(NORMAL_INSTALL)
	files=$(pdf_data); dir=$(pdfdir); $(install_files)

uninstall-doxy-pdf:
	@$(NORMAL_UNINSTALL)
	files=$(pdf_data); dir=$(pdfdir); $(uninstall_files)

endif DX_COND_pdf


##################
# general
if DX_COND_doc

# this is needed to trigger doxygen run
noinst_DATA = $(local_data_deps)

install-data-local  : $(local_data:%=install-%)
uninstall-local     : $(local_data:%=uninstall-%)

# generate DX_CONFIG here rather than in configure so that DX_CONFIG
# can be distributed and won't be recreated.  this allows
# the dx_doxygen targets to still depend upon it yet not have
# the docs be regenerated by the distribution
#
# config.status must be run from $(top_builddir) or it doesn't
# get the relative paths right.  however, config.status needs
# the relative path to DX_CONFIG from $(top_builddir). can't
# figure a way to do that automatically!  caller must set DOC_DIR
# to top_builddir - builddir
$(DX_CONFIG) : $(top_srcdir)/docs/$(DX_CONFIG).in
	cp $< $(DX_CONFIG).tmp.in
	DOC_DIR="$(DOC_DIR)"; \
	test "$${DOC_DIR+set}" = set || DOC_DIR=.; \
	cd $(top_builddir); \
	./config.status --file="$${DOC_DIR}/$(DX_CONFIG).tmp"; rc=$$? ; \
	cd $(abs_builddir) ; \
	if [ $$rc == 0 ]; then mv $(DX_CONFIG).tmp $(DX_CONFIG); \
	else                   rm $(DX_CONFIG).tmp; \
	fi ;\
	rm $(DX_CONFIG).tmp.in; \
	exit $$rc
endif DX_COND_doc
