// File:   t_vm_vmath.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of vm_math.cvs
//
// vm_math.cvs is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// vm_math.cvs is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <vm_math/vm_vmath.h>            // vm_VMath<T,N>

#include <cstdlib>
#include <iostream>
#include <testlib/testlib.h>

using std::cerr;

static
int dvm3_MMath_test01()    /* test copy(m,cm) */
{
  double   cm[]  = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0 };
  double   m[9];
  vm_VMath<double,9>::copy( m, cm );
  return (m[0] == 1.0) && (m[1] == 2.0) && (m[2] == 3.0) &&
         (m[3] == 4.0) && (m[4] == 5.0) && (m[5] == 6.0) &&
         (m[6] == 7.0) && (m[7] == 8.0) && (m[8] == 9.0);
}

static
int dvm3_MMath_test02()    /* test set(m,2) */
{
  double   m[9];
  vm_VMath<double,9>::set( m, 2.0 );
  return (m[0] ==  2.0) && (m[1] ==  2.0) && (m[2] ==  2.0) &&
         (m[3] ==  2.0) && (m[4] ==  2.0) && (m[5] ==  2.0) &&
         (m[6] ==  2.0) && (m[7] ==  2.0) && (m[8] ==  2.0);
}

static
int dvm3_MMath_test03()    /* test {add,sub,mul,div}_eq(m,m) */
{
  double m[]  = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };
  double m1[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };

  vm_VMath<double,9>::add_eq( m, m1 );
  vm_VMath<double,9>::sub_eq( m, m1 );
  vm_VMath<double,9>::mul_eq( m, m1 );
  vm_VMath<double,9>::div_eq( m, m1 );
  return (m[0] == 1.0) && (m[1] == 2.0) && (m[2] == 3.0) &&
         (m[3] == 4.0) && (m[4] == 5.0) && (m[5] == 6.0) &&
         (m[6] == 2.0) && (m[7] == 4.0) && (m[8] == 8.0);
}

static
int dvm3_MMath_test04()    /* test {add,sub,mul,div}_eq(m,r) */
{
  double m[]  = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };
  double m1[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };

  vm_VMath<double,9>::add_eq( m1, 2.0 );
  vm_VMath<double,9>::sub_eq( m1, 2.0 );
  vm_VMath<double,9>::mul_eq( m1, 2.0 );
  vm_VMath<double,9>::div_eq( m1, 2.0 );
  return (m[0] == 1.0) && (m[1] == 2.0) && (m[2] == 3.0) &&
         (m[3] == 4.0) && (m[4] == 5.0) && (m[5] == 6.0) &&
         (m[6] == 2.0) && (m[7] == 4.0) && (m[8] == 8.0);
}

static
int dvm3_MMath_test05()    /* test {add,sub,mul,div}(m,cm,cm) */
{
  double m1[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };
  double m2[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };

  double m[9];

  vm_VMath<double,9>::add(  m,  m1, m2 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::sub(  m,  m1, m2 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::mul(  m,  m1, m2 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::div(  m,  m1, m2 );
  return !fuzzy_compare(m[0], 1.0) &&
         !fuzzy_compare(m[1], 2.0) &&
         !fuzzy_compare(m[2], 3.0) &&
         !fuzzy_compare(m[3], 4.0) &&
         !fuzzy_compare(m[4], 5.0) &&
         !fuzzy_compare(m[5], 6.0) &&
         !fuzzy_compare(m[6], 2.0) &&
         !fuzzy_compare(m[7], 4.0) &&
         !fuzzy_compare(m[8], 8.0);
}

static
int dvm3_MMath_test06()    /* test {add,sub,mul,div}(m,cm,r) */
{
  double m[]  = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };
  double m1[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };

  vm_VMath<double,9>::add(  m,  m1, 2.0 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::sub(  m,  m1, 2.0 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::mul(  m,  m1, 2.0 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::div(  m,  m1, 2.0 );

  return !fuzzy_compare(m[0], 1.0) &&
         !fuzzy_compare(m[1], 2.0) &&
         !fuzzy_compare(m[2], 3.0) &&
         !fuzzy_compare(m[3], 4.0) &&
         !fuzzy_compare(m[4], 5.0) &&
         !fuzzy_compare(m[5], 6.0) &&
         !fuzzy_compare(m[6], 2.0) &&
         !fuzzy_compare(m[7], 4.0) &&
         !fuzzy_compare(m[8], 8.0);
}

static
int dvm3_MMath_test07()    /* test {add,sub,mul,div}(m,r,cm) */
{
  double m[9];
  double m1[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };

  vm_VMath<double,9>::add(  m,  2.0, m1 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::sub(  m,  2.0, m1 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::mul(  m,  2.0, m1 );
  vm_VMath<double,9>::copy( m1, m );
  vm_VMath<double,9>::div(  m,  2.0, m1 );

  return !fuzzy_compare(m[0], -1.0    ) &&
         !fuzzy_compare(m[1], -1.0/2.0) &&
         !fuzzy_compare(m[2], -1.0/3.0) &&
         !fuzzy_compare(m[3], -1.0/4.0) &&
         !fuzzy_compare(m[4], -1.0/5.0) &&
         !fuzzy_compare(m[5], -1.0/6.0) &&
         !fuzzy_compare(m[6], -1.0/2.0) &&
         !fuzzy_compare(m[7], -1.0/4.0) &&
         !fuzzy_compare(m[8], -1.0/8.0);
}

static
int dvm3_MMath_test08()    /* test lincomb(m3,2,m1,3,m2) */
{
  double m1[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 2.0, 4.0, 8.0 };
  double m2[] = { 2.0, 1.0, 4.0, 6.0, 4.0, 5.0, 3.0, 2.0, 1.0 };
  double m3[9];
  vm_VMath<double,9>::lincomb( m3, 2.0, m1, 3.0, m2 );
  return (m3[0] ==  8.0) && (m3[1] ==  7.0) && (m3[2] == 18.0) &&
         (m3[3] == 26.0) && (m3[4] == 22.0) && (m3[5] == 27.0) &&
         (m3[6] == 13.0) && (m3[7] == 14.0) && (m3[8] == 19.0);
}

static
int dvm3_MMath_test09()    /* test negate(m) */
{
  double m[] = { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0 };
  vm_VMath<double,9>::negate( m );
  return (m[0] == -1.0) && (m[1] == -2.0) && (m[2] == -3.0) &&
         (m[3] == -4.0) && (m[4] == -5.0) && (m[5] == -6.0) &&
         (m[6] == -7.0) && (m[7] == -8.0) && (m[8] == -9.0);
}

void
test_vm_VMath() 
{

  TEST( "copy(m,cm)",                 dvm3_MMath_test01(), 1 );
  TEST( "set(m,2.0)",                 dvm3_MMath_test02(), 1 );
  TEST( "{add,sub,mul,div}_eq(m,m)",  dvm3_MMath_test03(), 1 );
  TEST( "{add,sub,mul,div}_eq(m,r)",  dvm3_MMath_test04(), 1 );
  TEST( "{add,sub,mul,div}(m,cm,cm)", dvm3_MMath_test05(), 1 );
  TEST( "{add,sub,mul,div}(m,cm,r)",  dvm3_MMath_test06(), 1 );
  TEST( "{add,sub,mul,div}(m,r,cm)",  dvm3_MMath_test07(), 1 );
  TEST( "lincomb(m3,c1,cm1,c2,cm2)",  dvm3_MMath_test08(), 1 );
  TEST( "negate(m)",                  dvm3_MMath_test09(), 1 );

}
