*+
*KWIC besjn.f
*
*$Id: besjn.f,v 1.1 2004/03/12 16:33:04 dtn Exp $
*
*Revisions:
*   95-Oct-10[T. Gaetz]
*      . eliminate output to unit 6
*-
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     SUBROUTINE BESJN(X,N,BJN,ERRA, IER)
C
C
C
C    PURPOSE
C        COMPUTE THE ORDINARY
C        BESSEL FUNCTIONS OF ORDER N
C              J (X)
C               N
C
C    USAGE
C         CALL BESJN (X, N, BJN, ERRA, IER)
C
C    PARAMETERS
C         BJN  - VALUE OF BESSEL FUNCTION
C         ERRA - ACCURACY
C         IER  - ERROR FLAG
C              = 1 => FATAL ERROR BESJN COULD NOT ACHIEVE DESIRED
C                     PRECISION
C
C         N    - ORDER
C         X    - ARGUMENT
C
C    METHOD
C         USING THE RATIO X/DFLOAT(N)
C         BESJN DIRECTS THE CALCULATION TO ONE OF 3 SUBROUTINES
C     X/N < 0.50  => CALL BESJN0 FOR SUMMATION OF ASCENDING SERIES
C     X/N > 100. => CALL BESJN1 FOR SUMMATION OF ASYMPTOTIC SERIES
C      OTHERWISE  => CALL BESJN2 FOR DOWNWARDS RECURSION
C
C
C
C
C    WRITTEN 4/12/84 BY JOHN PARKER OF SSAI UNDER CONTRACT WITH
C    GSFC.
C
C    REFERENCE: ABRAMOWITZ AND STEGUN, HANDBOOK OF MATHEMATICAL
C                FUNCTIONS (NATIONAL BUREAU OF STANDARDS)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE BESJN(X,N,BJN,ERRA, IER)

      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      IMPLICIT INTEGER (I-N)
      DATA ERRMIN/1.D-13/, ZERO/0.0D0/, ONE/1.0D0/
      DATA TEN/10.0D0/
C
      IER = 0
      IER2 = 0
      IER1 = 0
      BJN = ZERO
      ERR = DABS(ERRA)
      IF(ERR .LE. ERRMIN) ERR= ERRMIN
C
C        DETERMINE PHASE
C
        PHASE = ONE
      IF(((X .LT. ZERO) .OR. (N .LT. 0)) .AND.
     A   (MOD(N,2) .NE. 0)) PHASE = -ONE
      IF ((X .LT. ZERO) .AND. (N.LT.0)) PHASE = ONE
C
C        CHANGE INPUT VBL NAME SO INPUT NOT DESTROYED
C
      XA  = DABS(X)
      NA  = IABS(N)
      DNA1= DFLOAT(NA+1)
C
C        CHECK FOR SPECIAL VALUES
C
      IF( XA .GT. ZERO) GO TO 10
C
C         SPECIAL VALUES FOR X = 0
C
      BJN = ZERO
      IF( NA .EQ. 0) BJN = ONE
      GO TO 50
C
   10  CONTINUE
       TEST = XA/DNA1
       IF(TEST   .LE. 0.50D0) GO TO 20
       IF(TEST   .GE. 1.D02 ) GO TO 30
   12  CALL BESJN2 (XA, NA, BJN, ERR, IER)
       IF(IER .EQ. 0) GO TO 50
       IF( (IER2.NE.0) .OR. (IER3.NE.0)) GO TO 40
C
C    DOWNWARDS RECURSION FAILED TO CONVERGE.  TRY ASCENDING OR
C    ASYMPTOTIC SERIES
C
       IF(TEST.LE.ONE) GO TO 20
       IF(TEST.GT.TEN) GO TO 30
       GO TO 40
   20  CALL BESJN0(XA, NA, BJN, ERR, IER2)
       IF(IER2.EQ.0) GO TO 50
       IF(IER.EQ.0) GO TO 12
       IER = IER2
       GO TO 40
   30  CALL BESJN1(XA, NA, BJN, ERR, IER3)
       IF(IER3.EQ.0) GO TO 50
       IF(IER.EQ.0) GO TO 12
       IER = IER3
       GO TO 40
   40  continue
       WRITE(8,10010) IER, X, N
10010  FORMAT(/,' BESJN: ERROR. FAILURE TO CONVERGE',/
     A          ' BESJN:                 ERROR FLAG IER = ',I5,
     B        /,' BESJN:',20X,'(X, N) =    ',1PD15.6,I5,/)
   50  RETURN
       END
