CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     SUBROUTINE BESJN2(X,N,BJN,ERRA, IER)
C
C
C
C    PURPOSE
C        COMPUTE THE ORDINARY
C        BESSEL FUNCTIONS OF ORDER N
C              J (X)      BY DOWNWARDS RECURSION
C               N
C
C    USAGE
C         CALL BESJN (X, N, BJN, ERRA, IER)
C
C    PARAMETERS
C         BJN  - VALUE OF BESSEL FUNCTION
C         ERRA - ACCURACY
C         IER  - ERROR FLAG
C              = 1 => FATAL ERROR BESJN COULD NOT ACHIEVE DESIRED
C                     PRECISION
C              =100=> DETAILED PRINTS
C         N    - ORDER
C         X    - ARGUMENT
C
C    METHOD
C        STARTING AT A LARGE ORDER, RECUR DOWNWARDS
C              USING
C      K (X) = 2*(N+1)*K   (X)/X - K(X)
C       NN+1N+2
C
C      THE K'S ARE EQUAL TO J'S WITHIN A CONSTANT FACTOR DETERMINED BY
C       DNORM = K  + 2*(K  + K  + . . . )
C                0       2    4
C
C      THEN  J  = K  /DNORM
C             N    N
C
C    WRITTEN 4/12/84 BY JOHN PARKER OF SSAI UNDER CONTRACT WITH
C    GSFC.
C
C    REFERENCE: ABRAMOWITZ AND STEGUN, HANDBOOK OF MATHEMATICAL
C                FUNCTIONS (NATIONAL BUREAU OF STANDARDS)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE BESJN2 (X, N, BJN, ERR, IER)
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      IMPLICIT INTEGER (I-N)
      DATA ZERO/0.0D0/, ONE/1.0D0/
C
      BJN    = ZERO
      BSAVE  = ZERO
      NSTART = MAX0(N + 10, IDINT(X + 10.0D0) )
      ICHECK = 0
C
C FOR A GIVEN NSTART, THIS LOOP IS PASSED THROUGH TWICE
C    TO CHECK ERRORS
C
   40 M = NSTART
      DNORM  = ZERO
         BJMP1 = ZERO
         BJMP2 = ZERO
         BJM   = 1.D-28
C               RECURSION
   50    IF(M .LE. 0) GO TO 60
              M=M-1
              DM = DFLOAT(M)
C  C
              BJMP2 = BJMP1
              BJMP1 = BJM
C  C
              BJM   =(2.0D0 * (DM+ONE ) * BJMP1/X)- BJMP2
C
              IF (M.EQ. N) BJN = BJM
              IF (MOD(M,2).NE.0) GO TO 50
              IF (M .NE. 0) DNORM = BJM + DNORM + BJM
              IF (M .EQ. 0) DNORM = BJM + DNORM
          GO TO 50
C                NORMALIZE
   60         BJN = BJN/DNORM
              IF( ICHECK.EQ.1) GO TO 70
                 ICHECK = 1
                 BSAVE = BJN
                 NSTART = NSTART + 2
                 GO TO 40
C
C          ACCURATE???
C
   70        IF(DABS(BSAVE-BJN) .LE. DABS(BJN*ERR)) GO TO 90
C
C           TRY AGAIN
C
   80            BSAVE = BJN
                 ICHECK = 1
                 NSTART = NSTART  + 5
                 IF(NSTART-N  .LT. 100 ) GO TO 40
C
C       FATAL  ERROR TRAP
C
                     IER = NSTART
   90 RETURN
      END
